/***************************************************************************
                           m_chgident.c
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "handlers.h"
#include "client.h"
#include "hash.h" /* if you use hybrid7-beta7 comment this line out */
#include "common.h"
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "whowas.h"
#include <string.h>

static void ms_chgident( struct Client *client_p, struct Client *source_p,
			 int part, char *parv[]);

struct Message chgident_msgtab = {
  "CHGIDENT", 0, 3, 3, 0, MFLG_SLOW, 0,
  /*   "CHGIDENT", 0, 3, 3, MFLG_SLOW, */ /* <- old line for hybrid 7-beta7 servers */
  { m_ignore, m_ignore, ms_chgident, m_ignore }
};

char * _version = "20020401";

void _modinit( void )
{
  mod_add_cmd(&chgident_msgtab);
}

void _moddeinit( void )
{
  mod_del_cmd(&chgident_msgtab);
}

/*
 * The incoming message has been send from an 'average' user and not
 * a server. Because only servers are permitted to reach command
 * 'CHGIDENT' through this module average users / irc operators cannot
 * reach CHGIDENT command
 */
static void ms_chgident( struct Client *client_p, struct Client *source_p,
			int parc, char *parv[])
{
  struct Client *target    = NULL;
  dlink_node    *ptr       = NULL;
  struct Client *bclient_p = NULL;
  
  if( IsServer( source_p ) ) 
    {
      /*       if( !(target = find_client( parv[1], NULL ) ) ) { */ /* this is the old version for beta 7 servers */
      if( !(target = find_client( parv[1] ) ) ) { /* this is the version for beta  9 hybrid servers */
	/* 
	 * chasing client now 
	 * dont know which timeout to use... *shame*
	 */
	if( !(target = get_history( parv[1], (long) KILLCHASETIMELIMIT ) ) ) 
	  {
	    ms_error( client_p, source_p, parc, parv );
	    
/* 	    sendto_one( client_p, form_str( ERR_NOSUCHNICK ), */
/* 			me.name, parv[0], parv[1] ); */
	    return;
	  }
      }

      if( strcmp( target->username, parv[2] ) ) {
	strcpy( & ( target->username[0] ), parv[2] );
	
	
	for( ptr = serv_list.head; ptr; ptr = ptr->next ) 
	  {
	    bclient_p = ptr->data;
	  if( bclient_p == source_p ) continue;
	  
	  sendto_one( bclient_p, ":%s CHGIDENT %s %s", source_p->name, 
		      parv[1],
		      target->username );
	  }
      }
    }
}
