/***************************************************************************
                           cmdlinekey.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: cmdlinekey.hpp,v 1.5 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef CMDLINEKEY_HPP
#define CMDLINEKEY_HPP

#include <string>
#include <iostream>
#include "config.h"

class CMDLineKey : public std::string 
{
public:
  CMDLineKey()
    : std::string(),
      m_gnuStyle( false )
  {}
  CMDLineKey( const std::string& str, bool gnuStyle = false )
    : std::string( str ),
      m_gnuStyle( gnuStyle )
  {}
  CMDLineKey( const char* str, bool gnuStyle = false )
    : std::string( str ),
      m_gnuStyle( gnuStyle )
  {}
  CMDLineKey( const CMDLineKey& key )
    : std::string( key ),
      m_gnuStyle( key.m_gnuStyle )
  {}

  const CMDLineKey& operator=( const CMDLineKey& key )
  {
    if( &key != this )
    {
       std::string::operator=( key );
       m_gnuStyle = key.m_gnuStyle;
    }
    return *this;
  }
  
  /**
   * this function does <b>not</b> affect 'gnuStyle' flag
   */
  const CMDLineKey& operator=( const std::string& key )
  {
    std::string::operator=( key );
    
    return *this;
  }

  const CMDLineKey& operator=( const char* str )
  {
    std::string::operator=( str );
    
    return *this;
  }

  void setGNUStyle( bool gnuStyle )
  { m_gnuStyle = gnuStyle; }
  bool isGNUStyle() const
  { return m_gnuStyle; };

private:
  bool m_gnuStyle;
};

std::ostream& operator<<( std::ostream& out, const CMDLineKey& key );

#endif
