/***************************************************************************
                           fifo.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: fifo.hpp,v 1.2 2002/05/16 17:50:23 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef FIFO_HPP
#define FIFO_HPP

#include <list>

template<class queueElemType>
class Fifo : protected list<queueElemType>
{
public:
  typedef list<queueElemType> container_type;
  typedef typename container_type::value_type value_type;
  typedef typename container_type::reference reference_type;

  /**
   * gets thrown by Queue after pop() of an empty queue
   */
  class RequestQueueEmpty
  {};

  Fifo()
     : container_type(),
       m_elementNum( 0 )
  {}

  virtual ~Fifo()
  {}
  
  bool empty() const
  { return m_elementNum == 0; }

  void push( const reference_type elem )
  {
    container_type::push_back( elem ); 
    ++m_elementNum; 
  }

  value_type pop() throw()
  {
    if( m_elementNum == 0 )
       throw new RequestQueueEmpty();

    value_type ret = container_type::front();

    container_type::pop_front();

    --m_elementNum;

    return ret;
  }

private:
  unsigned int m_elementNum;
};

#endif
