/***************************************************************************
                           file.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: file.hpp,v 1.11 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FILE_HPP
#define FILE_HPP

extern "C" {
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
}

class File {
public:
  typedef int            len_type;
  typedef unsigned char  data_type;
  typedef unsigned char* data_reference;
  typedef int            fd_type;

  static const len_type eof = -1;

  File() 
    :fd( -1 )
  {}

  virtual void close()
  { ::close( fd ); fd = eof; }

  virtual ~File()
  { close(); }

  fd_type getFD() const
  { return fd; };

  virtual len_type read( data_reference,
			 len_type len ) = 0;
  virtual len_type write( const data_reference,
			  len_type len ) = 0;

protected:
  fd_type fd;

  File( fd_type f )
    :fd( f )
  {
#ifdef DEBUG
    std::cout << "File] created: fd=" << fd << std::endl;
#endif
  }

private:
  File( const File& );
  const File& operator=( const File& );
};

class UNIXFile : public File
{
public:
  typedef int      int_type;
  typedef int_type flag_type;

  const static flag_type FL_RDONLY = O_RDONLY;
  const static flag_type FL_WRONLY = O_WRONLY;
  const static flag_type FL_RW     = O_RDWR;
  const static flag_type FL_APPEND = O_APPEND;
  const static flag_type FL_CREAT  = O_CREAT;

  explicit UNIXFile( const char* filename, flag_type flags )
    : File( ::open( filename, flags ) )
  {}

  explicit UNIXFile( fd_type FD )
    : File( FD )
  {}

  virtual ~UNIXFile()
  {}
      
  virtual len_type read( data_reference buf, len_type len )
  {
    const len_type rxLen = ::read( fd, buf, len );
    if( rxLen < 0 )
      return File::eof;
    else return rxLen;
  }

  virtual len_type write( const data_reference buf,
			  len_type len )
  {
    return ::write( fd, buf, len );
  }
};

#endif
