/***************************************************************************
                           hostserv.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hostserv.hpp,v 1.40 2002/05/23 15:02:17 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HOSTSERV_HPP
#define HOSTSERV_HPP

#include "application.hpp"
#include "hsircserverconnection.hpp"
#include "hsconfiguration.hpp"
#include "httpd.hpp"
#include "config.h"

class HostServ : public Application
{
public:
  static const string APP_NAME;
  static const string PID_FILENAME;
  
  // sa -> short arg
  // la -> long arg
  static const CMDLineKey SA_SERVNAME;
  static const CMDLineKey LA_SERVNAME;
  static const CMDLineKey SA_SERVERDESCR;
  static const CMDLineKey LA_SERVERDESCR;
  static const CMDLineKey SA_REMOTEHOST;
  static const CMDLineKey LA_REMOTEHOST;
  static const CMDLineKey SA_REMOTEPASSWORD;
  static const CMDLineKey LA_REMOTEPASSWORD;
  static const CMDLineKey SA_REMOTEPORT;
  static const CMDLineKey LA_REMOTEPORT;
  static const CMDLineKey SA_HSNICKNAME;
  static const CMDLineKey LA_HSNICKNAME;
  static const CMDLineKey SA_HSUSERNAME;
  static const CMDLineKey LA_HSUSERNAME;
  static const CMDLineKey SA_HSADDRESS;
  static const CMDLineKey LA_HSADDRESS;
  static const CMDLineKey SA_HSREALNAME;
  static const CMDLineKey LA_HSREALNAME;
  static const CMDLineKey SA_HSMODES;
  static const CMDLineKey LA_HSMODES;
  static const CMDLineKey SA_HSCHANNEL;
  static const CMDLineKey LA_HSCHANNEL;
  static const CMDLineKey SA_CLIENTDEFAULTPERM;
  static const CMDLineKey LA_CLIENTDEFAULTPERM;
  static const CMDLineKey SA_VHOSTPUBLICPOLICY;
  static const CMDLineKey LA_VHOSTPUBLICPOLICY;
  static const CMDLineKey SA_VHOSTSFILENAME;
  static const CMDLineKey LA_VHOSTSFILENAME;
  static const CMDLineKey SA_USERSFILENAME;
  static const CMDLineKey LA_USERSFILENAME;
  static const CMDLineKey SA_HSCONFIGFILENAME;
  static const CMDLineKey LA_HSCONFIGFILENAME;
  static const CMDLineKey SA_FOREGROUND;
  static const CMDLineKey LA_FOREGROUND;
  static const CMDLineKey SA_HELP;
  static const CMDLineKey LA_HELP;
  static const CMDLineKey SA_ADMINLOGIN;
  static const CMDLineKey LA_ADMINLOGIN;
  static const CMDLineKey SA_ADMINPASSWORD;
  static const CMDLineKey LA_ADMINPASSWORD;
  static const CMDLineKey SA_ADMINVHOST;
  static const CMDLineKey LA_ADMINVHOST;
  static const CMDLineKey SA_HTTPDPORT;
  static const CMDLineKey LA_HTTPDPORT;
  static const CMDLineKey SA_HTTPDMAXCLIENTS;
  static const CMDLineKey LA_HTTPDMAXCLIENTS;
  static const CMDLineKey SA_HTTPDDOCUMENTROOT;
  static const CMDLineKey LA_HTTPDDOCUMENTROOT;
  static const CMDLineKey SA_HTTPDDEFDOCUMENT;
  static const CMDLineKey LA_HTTPDDEFDOCUMENT;

  static const int DEF_RECONNECT_TIMEOUT = 15; // 15 s

  HostServ( arg_type args, int_type argNum );

  virtual ~HostServ();

  const HSConfiguration& getConfiguration() const
  { return m_configuration; }

  const HSIRCServerConnection* getHSIRCServerConnection() const
  { return m_hostserv; }

  const httpd::HTTPd* getHTTPd() const
  { return m_httpd; }

  /**
   *  emits virtual 'SIGSEGV'
   */
  void reload()
  { 
     m_doIo = false; 
     m_reloadConfig = true;
  }

  /*
   * makes hostserv disconnect from irc server
   */
  void reconnect()
  { 
     m_hostserv->disconnect( "Shutdown requested" );
     m_doIo = false;
  }

  /*
   * overwrites 'Application::stop()'.
   * calls 'Application::stop()'. 
   * - This functions was introduced to make the `cgi-interface' able to 
   *   stop application ( -> MemberFunctionExecutor (cgi.hpp) )
   */
  void stop()
  { 
     m_run = false;
     m_doIo = false;
  }

protected:
  virtual void onStart();
  virtual void onStop();
  
  virtual void onSigHup();
  virtual void onSigInt();
  virtual void onSigSegv();

  void printHelp();
  int computePollerSize();

  httpd::HTTPd* makeHTTPd( const InetHostAddress& addr, Poller* poller );
  HSIRCServerConnection* makeHostserv( Poller* poller );
  bool setupHostserv( const InetHostAddress& addr, Poller* poller );

private:  
  HSConfiguration&   m_configuration;
  HSIRCServerConnection* m_hostserv;
  httpd::HTTPd*      m_httpd;
  volatile bool      m_run;  // signalizes onStart() is still running
  volatile bool      m_doIo;
  volatile bool      m_reloadConfig;
//   volatile bool      m_receivedSigHup;
  int_type           m_timeout;
};

#endif
