/***************************************************************************
                           hsclientlist.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsclientlist.hpp,v 1.13 2002/05/19 17:54:03 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _HSCLIENTLIST_HPP
#define _HSCLIENTLIST_HPP

#include <set>
#include "irctoken.hpp"
#include "hsclient.hpp"
#include "dereferencecomparator.hpp"
#include "config.h"

using namespace std;

/**
 * stores authed clients.
 * no ressource management implemented
 */
class HSClientList : protected set<HSClient*, DereferenceLess<HSClient*> > {
public:
  HSClientList();
  virtual ~HSClientList();

  // RedHat g++ 2.96 workaround.
  typedef set<HSClient*, DereferenceLess<HSClient*> >::const_iterator const_iterator;

  /**
   * inserts a client into the set. No checks are made on the new client
   * @param c new client.
   */
  bool insertClient(HSClient *c);

  /**
   * removes a client from the set. 
   * client must not be part of the set.
   * @param c client which should be removed.
   * @return client found and removed successfully
   */
  bool eraseClient(HSClient *c);

  /**
   * returns the HSClient object for 'username'
   * @param nickname name of the client to search for
   *
   */
  HSClient * getClient(const IRCToken &username) const ;

  using set<HSClient*, DereferenceLess<HSClient*> >::empty;
  using set<HSClient*, DereferenceLess<HSClient*> >::begin;
  using set<HSClient*, DereferenceLess<HSClient*> >::end;
  using set<HSClient*, DereferenceLess<HSClient*> >::size;

  void clear();

private:
  HSClientList(const HSClientList &);
};

class HSClientListPredicate {
public:
  HSClientListPredicate(const IRCToken &refName);
  bool operator() (const HSClient*c);

private:
  const IRCToken &refName;
};

#endif
