/***************************************************************************
                           hsconfiguration.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsconfiguration.hpp,v 1.15 2002/05/21 19:55:26 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#ifndef HSCONFIGURATION_HPP
#define HSCONFIGURATION_HPP

#include "hsconfigfile.hpp"
#include "application.hpp"
#include "hsclientlist.hpp"
#include "vhostlist.hpp"
#include "irc.hpp"
#include "config.h"

class HSConfiguration
{
   private:
      HSConfiguration();

   public:
      typedef int                      int_type;
      typedef HSConfigFile::ConfigMap  ConfigfileMap;
      typedef Application::ConfigMap   CMDLineArgMap;
      typedef pair<string, bool> pair_type;

      static HSConfiguration& getConfiguration()
      {
	 static HSConfiguration config;
	 return config;
      }

//       HSConfiguration();
  
      ~HSConfiguration();

      void processCMDLineArgs( const CMDLineArgMap& map );
  
      void processConfigfile();

      /**
       * loads vhost database and stores processed data in private vhost list
       * - notice: it is very important that this functions gets called 
       *   before 'processUsersConfigfile()' has been called
       */
      void processVHostsConfigfile();
  
      /**
       * loads user database and stores processed data in private user list
       * - notice: it is very important that this function gets called
       *   after 'processVHostsConfigfile()' has been called
       */
      void processUsersConfigfile();

      HSClientList& getClients()
      { return m_clients; }

      VHostList& getVHosts()
      { return m_vhosts; }

      const HSClientList& getClients() const
      { return m_clients; }
 
      const VHostList& getVHosts() const
      { return m_vhosts; }
  
      const string& getServerName() const
      { return m_configuration[IDX_SERVNAME].value(); }
      const string& getServerDescription() const
      { return m_configuration[IDX_SERVERDESCR].value(); }

      const string& getRemoteHost() const
      { return m_configuration[IDX_REMOTEHOST].value(); }
      const string& getRemotePassword() const
      { return m_configuration[IDX_REMOTEPASSWORD].value(); }
      int getRemotePort() const
      { return m_port; }

      const string& getHostservNickname() const
      { return m_configuration[IDX_HSNICKNAME].value(); }
      const string& getHostservUsername() const
      { return m_configuration[IDX_HSUSERNAME].value(); }
      const string& getHostservAddress() const
      { return m_configuration[IDX_HSADDRESS].value(); }
      const string& getHostservRealname() const
      { return m_configuration[IDX_HSREALNAME].value(); }
      const string& getHostservModes() const
      { return m_configuration[IDX_HSMODES].value(); }
      const string& getHostservChannel() const
      { return m_configuration[IDX_HSCHANNEL].value(); }

      const string& getClientDefaultPermissions() const
      { return m_configuration[IDX_CLIENTDEFAULTPERM].value(); }

      const string& getVHostPublicpolicy() const
      { return m_configuration[IDX_VHOSTPUBLICPOLICY].value(); }

      const string& getHostservConfigfilename() const
      { return m_configuration[IDX_HSCONFIGFILENAME].value(); }

      const string& getVHostsConfigfilename() const
      { return m_configuration[IDX_VHOSTSFILENAME].value(); }

      const string& getUsersConfigfilename() const
      { return m_configuration[IDX_USERSFILENAME].value(); }

      const string& getAdminLogin() const
      { return m_configuration[IDX_ADMINLOGIN].value(); }
      const string& getAdminPassword() const
      { return m_configuration[IDX_ADMINPASSWORD].value(); }
      const string& getAdminVHostName() const
      { return m_configuration[IDX_ADMINVHOSTNAME].value(); }

      const string& getHTTPdDocumentRoot() const
      { return m_configuration[IDX_HTTPDDOCUMENTROOT].value(); }

      const string& getHTTPdDefaultDocument() const
      { return m_configuration[IDX_HTTPDDEFDOCUMENT].value(); }

      const string& getHTTPdBindInterface() const
      { return m_configuration[IDX_HTTPDBINDIF].value(); }

      int getHTTPdMaxClientTimeout() const
      { return m_maxHTTPdClientTimeout; }

      int getHTTPdMaxClientNum() const
      { return m_maxHTTPdClientNum; }

      int getHTTPdPort() const
      { return m_httpdPort; }

      bool getForeground() const
      { return m_foreground; }

      bool getHelp() const
      { return m_help; }

      bool serverNameSet() const
      { return m_configuration[IDX_SERVNAME].set(); }
      bool serverDescriptionSet() const
      { return m_configuration[IDX_SERVERDESCR].set(); }

      bool remoteHostSet() const
      { return m_configuration[IDX_REMOTEHOST].set(); }
      bool remotePasswordSet() const
      { return m_configuration[IDX_REMOTEPASSWORD].set(); }
      bool remotePortSet() const
      { return m_port > 0; }

      bool hostservNicknameSet() const
      { return m_configuration[IDX_HSNICKNAME].set(); }
      bool hostservUsernameSet() const
      { return m_configuration[IDX_HSUSERNAME].set(); }
      bool hostservAddressSet() const
      { return m_configuration[IDX_HSADDRESS].set(); }
      bool hostservRealnameSet() const
      { return m_configuration[IDX_HSREALNAME].set(); }
      bool hostservModesSet() const
      { return m_configuration[IDX_HSMODES].set(); }
      bool hostservChannelSet() const
      { return m_configuration[IDX_HSCHANNEL].set(); }

      bool clientDefaultPermissionsSet() const
      { return m_configuration[IDX_CLIENTDEFAULTPERM].set(); }
  
      bool vhostPubilcPolicySet() const
      { return m_configuration[IDX_VHOSTPUBLICPOLICY].set(); }

      // these 3 values are _always_ set
      bool vhostsConfigurationfilenameSet() const
      { return true; }

      bool usersConfigurationfilenameSet() const
      { return true; }

      bool hostservConfigurationfilenameSet() const
      { return true; }

      bool adminLoginSet() const
      { return m_configuration[IDX_ADMINLOGIN].set(); }
      bool adminPasswordSet() const
      { return m_configuration[IDX_ADMINPASSWORD].set(); }
      bool adminVHostNameSet() const
      { return m_configuration[IDX_ADMINVHOSTNAME].set(); }

      bool httpdDocumentRootSet() const
      { return m_configuration[IDX_HTTPDDOCUMENTROOT].set(); }
  
      bool httpdDefaultDocumentSet() const
      { return m_configuration[IDX_HTTPDDEFDOCUMENT].set(); }

      bool httpdMaxClientNumSet() const
      { return true; }

      bool httpdMaxClientTimeoutSet() const
      { return true; }

      bool httpdPortSet() const
      { return m_httpdPort != -1; }

      bool httpdBindInterfaceSet() const
      { return true; }

      bool foregoundSet() const
      { return true; }

      bool helpSet() const
      { return true; }

      /**
       * @return failed configuration directive
       */
      const string* check();

      /**
       * writes configuration data ( only vhosts and users )
       */
      void serialize();
  
      /**
       * removes all client and vhost configuration data
       */
      void cleanUpLists();

      void setServerName( const string& str )
      { assignHostname( IDX_SERVNAME, str ); }
      void setServerDescription( const string& descr )
      { assignIgnoreEmpty( IDX_SERVERDESCR, descr ); }
      void setRemoteHost( const string& remoteHost )
      { assignIgnoreEmpty( IDX_REMOTEHOST, remoteHost ); }
      void setRemotePassword( const string& password )
      { assignIgnoreEmpty( IDX_REMOTEPASSWORD, password ); }
      void setHostservNickname( const string& nickname )
      { assignNickname( IDX_HSNICKNAME, nickname ); }
      void setHostservUsername( const string& username )
      { assignUsername( IDX_HSUSERNAME, username ); }
      void setHostservAddress( const string& address )
      { assignHostname( IDX_HSADDRESS, address ); }
      void setHostservRealname( const string& realname )
      { assignIgnoreEmpty( IDX_HSREALNAME, realname ); }
      void setHostservModes( const string& modes )
      { assignIgnoreEmpty( IDX_HSMODES, modes ); }
      void setHostservChannel( const string& channel )
      { assignChannel( IDX_HSCHANNEL, channel ); }
      void setClientDefaultPermissions( const string& str )
      { setDefaultClientPermission( IDX_CLIENTDEFAULTPERM, str ); }
      void setVHostPublicpolicy( const string& policy )
      { setDefaultVHostPublicPolicy( IDX_VHOSTPUBLICPOLICY, policy ); }
      void setHostservConfigfilename( const string& configfileName )
      { assignIgnoreEmpty( IDX_HSCONFIGFILENAME, configfileName ); }
      void setVHostsConfigfilename( const string& configfileName )
      { assignIgnoreEmpty( IDX_VHOSTSFILENAME, configfileName ); }
      void setUsersConfigfilename( const string& configfileName )
      { assignIgnoreEmpty( IDX_USERSFILENAME, configfileName ); }

   protected:
      static const int_type IDX_SERVNAME          =  0;
      static const int_type IDX_SERVERDESCR       =  1;
      static const int_type IDX_REMOTEHOST        =  2;
      static const int_type IDX_REMOTEPASSWORD    =  3;
      static const int_type IDX_HSNICKNAME        =  4;
      static const int_type IDX_HSUSERNAME        =  5;
      static const int_type IDX_HSADDRESS         =  6;
      static const int_type IDX_HSREALNAME        =  7;
      static const int_type IDX_HSMODES           =  8;
      static const int_type IDX_HSCHANNEL         =  9;
      static const int_type IDX_CLIENTDEFAULTPERM = 10;
      static const int_type IDX_VHOSTPUBLICPOLICY = 11;
      static const int_type IDX_HTTPDDEFDOCUMENT  = 12;
      static const int_type IDX_HTTPDBINDIF       = 13;
      // we do not have to check the directives above because they get set
      // explicitly by 'fillConfiguration()'
      static const int_type IDX_VHOSTSFILENAME    = 14;
      static const int_type IDX_USERSFILENAME     = 15;
      static const int_type IDX_HSCONFIGFILENAME  = 16;
      static const int_type IDX_ADMINLOGIN        = 17;
      static const int_type IDX_ADMINPASSWORD     = 18;
      static const int_type IDX_ADMINVHOSTNAME    = 19;
      static const int_type IDX_HTTPDDOCUMENTROOT = 20;

      // ! *AAAH* keep this _ALWAYS_ in sync with the number of indices above !
      static const int_type IDX_MAX = 21;

      // array handler indices 
      static const int_type MANDATORY_DECLARATIONS[];
  
      // configuration default values
      static const string DEF_HSCONFIGFILENAME;
      static const string DEF_USERSCONFIGFILENAME;
      static const string DEF_VHOSTSCONFIGFILENAME;
      static const string DEF_HSNICK;
      static const string DEF_HSUSERNAME;
      static const string DEF_HSADDRESS;
      static const string DEF_HSDESCRIPTION;
      static const string DEF_HSMODES;
      static const string DEF_VHOSTPUBLICPOLICY;
      static const string DEF_CLIENTDEFAULTPERM;
      static const string DEF_HTTPDDEFDOCUMENT;
      static const string DEF_HTTPDBINDIF;

      struct ConfigHandler
      {
	    typedef void (HSConfiguration::*cfgHandler)( int index, const string&);

	    const CMDLineKey* const shortArg; // cmdline arg (-> '-d' )
	    const CMDLineKey* const longArg;  // cmdline arg (-> '--debug' )
	    const string* const     cfgDirective; // directive name in hs configfile    
	    cfgHandler const    handler;
      };
  
      class Item : protected pair_type
      {
	 public:
	    Item()
	       : pair_type()
	    {}

	    const string& value() const
	    { return first; }

	    const bool set() const
	    { return second; }
    
	    void set( bool s )
	    { second = s; }

	    void value( const string& str )
	    { first = str; }

	    const Item& operator=( const string& str )
	    { first = str; second = true; return *this; }

	 private:
	    Item( const Item& );
	    const Item& operator=( const Item& );
      };

      // handler
      void assignCommon( int idx, const string& val )
      { m_configuration[idx] = val; }

      void assignIgnoreEmpty( int idx, const string& val )
      { if( !val.empty() ) m_configuration[idx] = val; }
  
      void setDefaultVHostPublicPolicy( int index, const string& val );
      void setDefaultClientPermission( int index, const string& val );
      void assignHostname( int index, const string& rawHostname );
      void assignUsername( int index, const string& rawUsername );
      void assignNickname( int index, const string& rawNickname );
      void assignChannel( int index, const string& rawChannel );
      void assignPort( int index, const string& port );
      void assignForeground( int index, const string& fg );
      void assignHelp( int index, const string& help );
      void assignAdminLogin( int index, const string& rawNick );
      void assignAdminPassword( int index, const string& rawPass );
      void assignHTTPdPort( int index, const string& rawPort );
      void assignHTTPdMaxClients( int index, const string& rawNum );
      void assignHTTPdDefaultDocument( int index, const string& filename );
      void assignHTTPdDocumentRoot( int index, const string& root );
      void assignHTTPdClientTimeout( int index, const string& timeout );
  
      // variables
      static const ConfigHandler HANDLERS[];
      Item m_configuration[IDX_MAX];

   private:
      HSConfiguration( const HSConfiguration& );
      const HSConfiguration& operator=( const HSConfiguration& );

      // fillls the configuration string array with default values
      void fillWithDefault();


      /**
       * processes 'map' and updates stored configuration
       * @param map map build by an Application object
       */
      void processHSConfigfileMap( const ConfigfileMap& map );

      VHostList    m_vhosts;
      HSClientList m_clients;

      /**
       *  m_port == -1 indicates unset value<br>
       *  default value is: -1
       */

      int          m_port;
  
      /**
       *  whether hostserv fork()s or not<br>
       * default value is: false
       */
      bool         m_foreground;

      /**
       * print help<br>
       * default value is: false
       */
      bool         m_help;

      /**
       * max number of httpd clients.
       * default value is: 2 ( rather low )
       */
      int          m_maxHTTPdClientNum;

      /**
       * the tcp port on which the httpd server is about to listen.<br>
       * -1 siganlizes deactived httpd support / uninitialized value
       */
      int          m_httpdPort;

      /**
       * the number of seconds a http client is allowed to idle
       * until it gets disconnected. <br>
       * <b>Note</b>: this is an approximative number. 
       *              hostserv will not check every second whether a
       *              http client is ideling too long.
       *              This value only defines the period after which hostserv
       *              will check for timeout'd out clients.<br>
       *
       * default value is: 30 
       */
      int          m_maxHTTPdClientTimeout;

};

#endif
