/***************************************************************************
                           hsircclient.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsircclient.hpp,v 1.9 2002/03/19 16:39:02 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _HSIRCCLIENT_HPP
#define _HSIRCCLIENT_HPP

#include <ircclient.hpp>
#include <hsclient.hpp>

class HSIRCClient : public virtual IRCClient
{

public:
  inline HSIRCClient( const IRCToken& nickname,
		      const IRCToken& username,
		      const IRCToken& address,
		      const IRCToken& realname,
		      const HSClient* hcl = NULL );
  virtual ~HSIRCClient();

  const HSClient* getHSClient() const
  { return hsclient; };

  void login( const HSClient* hscl );

  void logout();

  bool isAuthed() const 
  { return hsclient != NULL; };
  bool vhostApplied() const
  { return m_ircClientFlags & flagVHostApplied; }

  void setVHostApplied( bool on )
  {
    m_ircClientFlags = on ? 
      m_ircClientFlags | flagVHostApplied :
      m_ircClientFlags & ~flagVHostApplied; 
  }
      
    
  HSClient::flag_type getFlags() const // do _not_ change this function
  { return isAuthed() ? hsclient->getFlags() : HSClient::flagUnauthed; }

private:
  HSIRCClient();
  HSIRCClient( const HSIRCClient& cl );
  const HSIRCClient& operator=( const HSIRCClient& cl );

  typedef HSClient::flag_type flag_type;

  static const flag_type flagVHostApplied = 1;


  const HSClient *hsclient;
  flag_type m_ircClientFlags;  
};

ostream& operator<<( ostream& out, const HSIRCClient& client );

#endif
