/***************************************************************************
                           hsircclientlist.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsircclientlist.hpp,v 1.10 2002/05/19 17:54:03 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _HSIRCCLIENTLIST_HPP
#define _HSIRCCLIENTLIST_HPP

#include <set>
#include <hsircclient.hpp>
#include <config.h>

using namespace std;

class HSIRCClientList : protected set< HSIRCClient *, DereferenceLess< HSIRCClient* > > {
public:
  HSIRCClientList();
  ~HSIRCClientList();

  typedef set< HSIRCClient*, DereferenceLess< HSIRCClient* > >::const_iterator const_iterator;

  bool insertClient( HSIRCClient *c );
  HSIRCClient* eraseClient( const IRCToken& nickname );
  void updateClientNickname( const IRCToken &oldNickname, 
			     const IRCToken &newNickname );
  HSIRCClient* getClient( const IRCToken &nickname ) const;
  HSIRCClient* getClient( const HSClient* login ) const;
  using set< HSIRCClient *, DereferenceLess< HSIRCClient* > >::begin;
  using set< HSIRCClient *, DereferenceLess< HSIRCClient* > >::end;
  using set< HSIRCClient *, DereferenceLess< HSIRCClient* > >::size;

  void clear();

private:
  HSIRCClientList( const HSIRCClientList &);
  const HSIRCClientList & operator = ( const HSIRCClientList & );

  inline const_iterator getPos( const IRCToken& nickname ) const;
  inline const_iterator getPos( const HSClient* login ) const;
};

class HSIRCClientListPredicate {
public:
  HSIRCClientListPredicate( const IRCToken& refName );
  bool operator() ( const HSIRCClient *c );
  
private: 
  const IRCToken &refName;
};

class HSIRCClientListLoginPredicate {
public:
  HSIRCClientListLoginPredicate( const HSClient* cl );
  bool operator() ( const HSIRCClient *c );

private:
  const HSClient* m_login;
};

class DistributedHSIRCClientList
{
public:
  typedef enum {
    LIST_A = 0,
    LIST_B = 1,
    LIST_C = 2,
    LIST_D = 3,
    LIST_E = 4,
    LIST_F = 5,
    LIST_G = 6,
    LIST_H = 7,
    LIST_I = 8,
    LIST_J = 9,
    LIST_K = 10,
    LIST_L = 11,
    LIST_M = 12,
    LIST_N = 13,
    LIST_O = 14,
    LIST_P = 15,
    LIST_Q = 16,
    LIST_R = 17,
    LIST_S = 18,
    LIST_T = 19,
    LIST_U = 20,
    LIST_V = 21,
    LIST_W = 22,
    LIST_X = 23,
    LIST_Y = 24,
    LIST_Z = 25
  } listindex_type;
  
  DistributedHSIRCClientList()
  {}
  ~DistributedHSIRCClientList()
  { clear(); }

  bool insertClient( HSIRCClient* c );
  HSIRCClient* eraseClient( const IRCToken& nickname );
  void updateClientNickname( const IRCToken& oldNickname,
			     const IRCToken& newNickname );
  HSIRCClient* getClient( const IRCToken& nickname ) const;
  HSIRCClient* getClient( const HSClient* login ) const;

  const HSIRCClientList* getClients( const IRCToken::data_type c ) const;

  void clear();

private:
  HSIRCClientList m_clientLists[32];

  inline static int computeIndex( const IRCToken::data_type c )
  { return LISTINDEX_TAB[c]; }

  inline static int computeIndex( const IRCToken& nickname )
  { return nickname.length() > 0 ? computeIndex( nickname[0] ) : -1; }

  /**
   * checks 'index' and asserts whenever index < 0 || index > 32
   */
  inline       HSIRCClientList& getList( int index );
  inline const HSIRCClientList& getList( int index ) const;

  /*
   * we do not make calls which use this array directly
   * but we use the computeIndex functions
   */
  static const char LISTINDEX_TAB[];
};


#endif
