/***************************************************************************
                           hsircserverconnection.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsircserverconnection.hpp,v 1.23 2002/06/13 16:07:29 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HSIRCSERVERCONNECTION_HPP
#define HSIRCSERVERCONNECTION_HPP

#include "ircserverconnection.hpp"
#include "hsclientlist.hpp"
#include "hsircclientlist.hpp"
#include "vhostlist.hpp"
#include "pollerclient.hpp"
#include "config.h"

// forward declaration for HSIRCServerConnection
class HostServ;

namespace hostserv { 
   namespace commands {
      class AbstractHSCommandState;
   } 
}

class HSIRCServerConnection : public IRCServerConnection {
      const IRCToken hostservNick;
      const IRCToken hostservIdent;
      const IRCToken hostservAddress;
      const IRCToken hostservRealname;
      const IRCToken hostservModes;
      const IRCToken hostservChannel;
      
      HostServ*        const m_mother;
      HSClientList*    const hsClients;  // contains clients from configfile.
      HSIRCClientList* const ircClients; // stores authed clients in irc
      VHostList*       const vhosts;     // available vhosts
      
      hostserv::commands::AbstractHSCommandState* m_lastAuthedState;
      hostserv::commands::AbstractHSCommandState* m_lastUnauthedState;
      
      typedef enum {
	 PRINT_NOTHING = 0, // do not print anything
	 PRINT_OLD,         // print pending data from the command
	 PRINT_STATE        // process command states and print
      } printstate_type;
      printstate_type m_authStatePrintState;
      printstate_type m_unauthedStatePrintState;
      
      static const IRCToken DEFAULT_KILL_CAUSE;
      static const IRCToken IRCCMD_SETHOST;
      static const IRCToken IRCCMD_SETIDENT;
      

   public:
      HSIRCServerConnection( HostServ* mother,
			     const IRCToken &hostservName,
			     const IRCToken &remotePassword, 
			     const IRCToken &hostservDescription,
			     const IRCToken &hostservNick,
			     const IRCToken &hostservIdent,
			     const IRCToken &hostservAddress,
			     const IRCToken &hostservRealname,
			     const IRCToken &hostservModes,
			     const IRCToken &hostservChannel,
			     HSClientList* Clients,
			     VHostList* Vhosts,
			     Poller* poller,
			     len_type inBufLen,
			     len_type outBufLen );
      
      virtual ~HSIRCServerConnection();
      
      void disconnect( const char * cause );
      
//   const DistributedHSIRCClientList* getIRCClients() const
//   { return ircClients; }
      
      const VHostList* getVHosts() const
      { return vhosts; }
      
      VHostList* getVHosts()
      { return vhosts; }
      
      const HSClientList* getHSClients() const
      { return hsClients; }
      
      HSClientList* getHSClients()
      { return hsClients; }

      const HSIRCClientList* getHSIRCClients() const
      { return ircClients; }
 
      HSIRCClientList* getHSIRCClients()
      { return ircClients; }

      const IRCToken& getHostservNick() const
      { return hostservNick; }

      const IRCToken& getHostservIdent() const
      { return hostservIdent; }

      const IRCToken& getHostservAddress() const
      { return hostservAddress; }

      const IRCToken& getHostservRealname() const
      { return hostservRealname; }

      const IRCToken& getHostservChannel() const
      { return hostservChannel; }

      HostServ* getMother()
      { return m_mother; }

      const HostServ* getMother() const
      { return m_mother; }

      void printHelpArray( const IRCToken &target,
			   const IRCToken * msgArray, int len );
      void printHelpArray( const IRCToken &target,
			   const unsigned char *msgArray[]);

      /**
       * performs the actual hostchange
       * - sends a irc message to the upstream server
       *   in which the hostchange is propagated
       * 
       * @param target of which hostname is to be changed
       * @param newhost - new hostname for 'target'
       * @param noticeUser - indicates if the target should 
       *                     be notice'd that his / her hostname
       *                     has been changed
       */
      void changeHost( const IRCToken &target,
		       const IRCToken &newHost,
		       bool noticeUser = false );
      /**
       * @see #changeHost
       *
       */
      void changeUsername( const IRCToken &target,
			   const IRCToken &newIdent,
			   bool noticeUser = false );

      /**
       * is an frontend for changeHost and changeUsername
       * @see #changeHost
       * @see #changeUsername
       */
      void applyVHost( const HSIRCClient& client, 
		       const VHost * const v, 
		       bool noticeUser = false );

   protected:
      virtual void onOutput();

      virtual void onLogedin();

      virtual void onPrivmsg(const IRCToken &from, const IRCToken &to,
			     const IRCToken* msg, int numArgs ); 
      virtual void onNotice(const IRCToken &from, const IRCToken& to,
			    const IRCToken* msg, int numArgs ); 
      virtual void onNickChange( const IRCToken &oldick,
				 const IRCToken &newNick );
      virtual void onQuit( const IRCToken& from,
			   const IRCToken* msg, int tokenNum );
      virtual void onKill( const IRCToken& from, 
			   const IRCToken& target,
			   const IRCToken* cause,
			   int argc );
      virtual void onWhois( const IRCToken& from,
			    const IRCToken& who );
      virtual void onServerError( const IRCServerMessage &m );
      virtual void onNicknameInUse( const IRCToken &nicknameInUse );
      virtual void onNicknameCollision( const IRCToken &collidedNick );
      virtual void onNickIntroduce( const IRCToken &nick, 
				    const IRCToken &ident,
				    const IRCToken &address, 
				    const IRCToken &realname );

   private:
      HSIRCServerConnection();
      const HSIRCServerConnection& operator=( const HSIRCServerConnection & );
      HSIRCServerConnection( const HSIRCServerConnection& );
  
      void processMsg( const IRCToken& from,
		       const IRCToken& to,
		       const IRCToken* msg,
		       int argc );

      void printChangedHost( const IRCToken& target,
			     const IRCToken& newHost );
      void printChangedUsername( const IRCToken &target,
				 const IRCToken& newUsername );
      void processForciblyIntroduceHostserv();
      void processClientQuit( const IRCToken& quitter );

      static bool EqualsIgnoreCase( const IRCToken& lhs, const IRCToken& rhs );
      static IRCToken ToLower( const IRCToken& s );
      static inline bool IRCTokenToBool( const IRCToken& s);
};

#endif
