/***************************************************************************
                           hsirctraits.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsirctraits.hpp,v 1.4 2002/05/16 17:50:23 d3cline Exp $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HSIRCTRAITS_HPP
#define HSIRCTRAITS_HPP

class HSIRCTraits
{
public:
  typedef int int_type;

private:
  HSIRCTraits();
  HSIRCTraits( const HSIRCTraits& );
  const HSIRCTraits& operator=( const HSIRCTraits & );

  static const int_type PRINT_C    = 0x001;
  static const int_type CNTRL_C    = 0x002;
  static const int_type ALPHA_C    = 0x004;
  static const int_type FUNCT_C    = 0x008;
  static const int_type DIGIT_C    = 0x010;
  static const int_type SPACE_C    = 0x020;
  static const int_type NICK_C     = 0x040;
  static const int_type CHAN_C     = 0x080;
  static const int_type KWILD_C    = 0x100;
  static const int_type CHANPFX_C  = 0x200;
  static const int_type USER_C     = 0x400;
  static const int_type HOST_C     = 0x800;
  static const int_type NONEOS_C   = 0x1000;
  static const int_type SERV_C     = 0x2000;
  static const int_type EOL_C      = 0x4000;
  static const int_type HSCLIENT_C = 0x8000;

  static const unsigned int  CHAR_ATTRS[];
  static const unsigned char TOLOWER_TAB[];
  static const unsigned char TOUPPER_TAB[];

public:
  static bool isHostChar( unsigned char c )
  { return CHAR_ATTRS[c] & HOST_C; }
  static bool isUserChar( unsigned char c )
  { return CHAR_ATTRS[c] & USER_C; }
  static bool isChanChar( unsigned char c )
  { return CHAR_ATTRS[c] & CHAN_C; }
  static bool isNickChar( unsigned char c )
  { return CHAR_ATTRS[c] & NICK_C; }
  static bool isChanPrefix( unsigned char c )
  { return CHAR_ATTRS[c] & CHANPFX_C; }
  static bool isHSClientChar( unsigned char c )
  { return CHAR_ATTRS[c] & HSCLIENT_C; }
  static unsigned char toLower( unsigned char c )
  { return TOLOWER_TAB[c]; }
  static unsigned int toUpper( unsigned char c )
  { return TOUPPER_TAB[c]; }
  

  // MIRC format flags
  static const unsigned int FLAG_BOLD = static_cast<unsigned char>( 2 );
  static const unsigned int FLAG_UNDL = static_cast<unsigned char>( 31 );

};

#endif
