/***************************************************************************
                           http_common.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: http_common.hpp,v 1.15 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HTTP_COMMON
#define HTTP_COMMON


#include "httpstring.hpp"
#include "httptraits.hpp"

namespace http
{
  typedef enum { 
    HTTPVER_100,
    HTTPVER_110,
    HTTPVER_UNKNOWN
  } httpprotversion_type;

  typedef enum {
    HTTPCONN_CLOSE =     0,
    HTTPCONN_KEEPALIVE = 1,
    HTTPCONN_CHUNKED =   2,
    HTTPCONN_UNKNOWN =   3
  } httpresponseconnectiontype_type;

  typedef int httpmethod_type;

  typedef HTTPString RequestHeader;
  typedef HTTPString ResponseHeader;
  typedef HTTPString EntityHeader;
  typedef HTTPString GeneralHeader;
  typedef HTTPString FieldValue;


  /*
   * !do not change these values!
   */
  static const httpmethod_type REQ_METHOD_OPTIONS = 0;
  static const httpmethod_type REQ_METHOD_GET     = 1;
  static const httpmethod_type REQ_METHOD_HEAD    = 2;
  static const httpmethod_type REQ_METHOD_POST    = 3;
  static const httpmethod_type REQ_METHOD_PUT     = 4;
  static const httpmethod_type REQ_METHOD_DELETE  = 5;
  static const httpmethod_type REQ_METHOD_TRACE   = 6;
  static const httpmethod_type REQ_METHOD_CONNECT = 7;
  static const httpmethod_type REQ_METHOD_UNKNOWN = 8;

  static const GeneralHeader GENERAL_CACHE_CONTRAL =     "Cache-Control";
  static const GeneralHeader GENERAL_CONNECTION =        "Connection";
  static const GeneralHeader GENERAL_DATE =              "Date";
  static const GeneralHeader GENERAL_PRAGMA =            "Pragma";
  static const GeneralHeader GENERAL_TRAILER =           "Trailer";
  static const GeneralHeader GENERAL_TRANSFER_ENCODING = "Transfer-Encoding";
  static const GeneralHeader GENERAL_UPGRADE =           "Upgrade";
  static const GeneralHeader GENERAL_VIA =               "Via";
  static const GeneralHeader GENERAL_WARNING =           "Warning";

  static const RequestHeader REQ_ACCEPT =              "Accept";
  static const RequestHeader REQ_ACCEPT_CHARSET =      "Accept-Charset";
  static const RequestHeader REQ_ACCEPT_LANGUAGE =     "Accept-Language";
  static const RequestHeader REQ_AUTHORIZATION =       "Authorization";
  static const RequestHeader REQ_EXPECT =              "Expect";
  static const RequestHeader REQ_FROM =                "From";
  static const RequestHeader REQ_HOST =                "Host";
  static const RequestHeader REQ_IF_MATCH =            "If-Match";
  static const RequestHeader REQ_IF_MODIFIED_SINCE =   "If-Modified-Since";
  static const RequestHeader REQ_IF_NONE_MATCH =       "If-None-Match";
  static const RequestHeader REQ_IF_RANGE =            "If-Range";
  static const RequestHeader REQ_IF_UNMODIFIED_SINCE = "If-Unmodified-Since";
  static const RequestHeader REQ_MAX_FORWARS =         "Max-Forwards";
  static const RequestHeader REQ_PROXY_AUTHORIZATION = "Proxy-Authorization";
  static const RequestHeader REQ_RANGE =               "Range";
  static const RequestHeader REQ_REFERER =             "Referer";
  static const RequestHeader REQ_TE =                  "TE";
  static const RequestHeader REQ_USER_AGENT =          "User-Agent";
  static const RequestHeader REQ_CONTENT_TYPE =        "Content-type"; // netscape sends 'Content-type' instead of 'Content-Type' ... :/

  static const EntityHeader ENT_ALLOW =            "Allow";
  static const EntityHeader ENT_CONTENT_ENCODING = "Content-Encoding";
  static const EntityHeader ENT_CONTENT_LENGTH =   "Content-Length";
  static const EntityHeader ENT_CONTENT_LOCATION = "Content-Location";
  static const EntityHeader ENT_CONTENT_MD5 =      "Content-MD5";
  static const EntityHeader ENT_CONTENT_RANGE =    "Content-Range";
  static const EntityHeader ENT_CONTENT_TYPE =     "Content-Type";
  static const EntityHeader ENT_ACCEPT_RANGES =    "Accept-Ranges";
  static const EntityHeader ENT_EXPIRES =          "Expires";
  static const EntityHeader ENT_LAST_MODIFIED =    "Last-Modified";

  static const ResponseHeader RESP_ACCEPT_RANGES =      "Accept-Ranges";
  static const ResponseHeader RESP_AGE =                "Age";
  static const ResponseHeader RESP_ETAG =               "ETag";
  static const ResponseHeader RESP_LOCATION =           "Location";
  static const ResponseHeader RESP_PROXY_AUTHENTICATE = "Proxy-Authenticate";
  static const ResponseHeader RESP_RETRY_AFTER =        "Retry-After";
  static const ResponseHeader RESP_SERVER =             "Server";
  static const ResponseHeader RESP_VARY =               "Vary";
  static const ResponseHeader RESP_WWW_AUTHENTICATE =   "WWW-Authenticate";
  
  static const FieldValue FV_CONNECTION_CLOSE      = "close";
  static const FieldValue FV_CONNECTION_KEEPALIVE  = "Keep-Alive";
  static const FieldValue FV_TE_CHUNKED            = "chunked";
  static const FieldValue FV_BYTES                 = "bytes";

  const HTTPString& httpProtocolVersionToString( httpprotversion_type version );

  /**
   * converts a field value describing connection type into 
   * httpresponseconnectiontype_type.<br>
   */
  httpresponseconnectiontype_type responseConnectionStringToType( const FieldValue& responseConnection );

  std::ostream& httpendl( std::ostream& out );

};
#endif
