/***************************************************************************
                           httpserverport.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: httpserverport.hpp,v 1.9 2002/05/16 17:50:23 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HTTPSERVERPORT_HPP
#define HTTPSERVERPORT_HPP


#include "pollable.hpp"
#include "poller.hpp"
#include "socket.hpp"
#include "httpd.hpp"
#include "config.h"

namespace httpd
{

class HTTPd;

class HTTPServerPort : protected TCPPort, public Pollable
{
public:
  HTTPServerPort( HTTPd* m_server,
		  Poller* poller,
		  const InetHostAddress& bind, port_type port );

  virtual ~HTTPServerPort()
  {}

  virtual const File* getPollableFile() const
  { return this; }

protected:
  /**
   * @see Pollable#onInput
   */
  virtual void onInput();
  /**
   * @see Pollable#onOutput
   */
  virtual void onOutput();
  /**
   * @see Pollable#Error
   */
  virtual void onError();
  /**
   * @see Pollable#onHangup
   */
  virtual void onHangup();
  /**
   * @see Pollable#onRemoved
   */
  virtual void onRemoved();

  virtual void onConnected( int sockFD,
			    const sockaddr_in& from,
			    socklen_t fromLen );

private:
  HTTPd* const  m_server;
  Poller* const m_poller;

  virtual File::len_type read( File::data_reference, File::len_type len )
  { return File::eof; }
  virtual File::len_type write( const File::data_reference, File::len_type len )
  { return File::eof; }
};

}//EON http
#endif
