/***************************************************************************
                           httpstring.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: httpstring.hpp,v 1.9 2002/05/25 16:25:27 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HTTPSTRING_HPP
#define HTTPSTRING_HPP


#include <string>
#include "httptraits.hpp"

#ifdef __GNUC__
#define DIRTY_GNU_STD_STRING_WORKAROUND
#endif

namespace http
{

class HTTPString : public std::basic_string<unsigned char>
{
public:
  typedef std::basic_string<unsigned char> string_type;
  typedef string_type::traits_type traits_type;
  typedef string_type::value_type value_type;
  typedef string_type::size_type size_type;
  typedef string_type::difference_type difference_type;
  typedef string_type::reference reference;
  typedef string_type::const_reference const_reference;
  typedef string_type::pointer pointer;
  typedef string_type::const_pointer const_pointer;
  typedef string_type::iterator iterator;
  typedef string_type::const_iterator const_iterator;
  typedef string_type::reverse_iterator reverse_iterator;
  typedef string_type::const_reverse_iterator const_reverse_iterator;

  HTTPString()
    : string_type(),
       m_cstrBuffer( NULL )
  {}  
  HTTPString( const char* msg )
    : string_type( (const_pointer) msg ),
      m_cstrBuffer( NULL )
  {}
  HTTPString( const char* msg, size_type len )
    : string_type( (const_pointer)msg, len ),
      m_cstrBuffer( NULL )      
  {}
  HTTPString( const unsigned char* msg )
    : string_type( msg ),
      m_cstrBuffer( NULL )
  {}
  HTTPString( const unsigned char* msg, size_type len )
    : string_type( msg, len ),
      m_cstrBuffer( NULL )
  {}
  HTTPString( const HTTPString& str )
    : string_type( str ),
      m_cstrBuffer( NULL )
  {}
  HTTPString( const std::basic_string<unsigned char>& str )
    : string_type( str ),
      m_cstrBuffer( NULL )
  {}
  HTTPString( const std::string& str )
    : string_type( (const_pointer)str.c_str(), str.length() ),
      m_cstrBuffer( NULL )
  {}

  /**
   * does the same as atoi() does. 
   * Warning: this function does not cache results. Means:
   * repetitive calling of operator int() will make 
   * this class compute the same on and on
   */
  // this function is about 10 times faster than the gnu std c
  // atoi function. hossa!
  operator unsigned int() const
  {
    int                  value   = 0;
    const_iterator       iter    = string_type::begin();
    const const_iterator iterEnd = string_type::end();

    // skip last non digit characters
    while( iter != iterEnd )
    {
      if( HTTPTraits::isDigitChar( *iter ) )
	break;
      ++iter;
    }

    while( iter != iterEnd ) {
      if( !HTTPTraits::isDigitChar( *iter ) )
	break;
      value *= 10;
      value += HTTPTraits::digitCharToInt( *iter );
      ++iter;
    }

    return value;
  }

  bool endsWith( const HTTPString& str ) const
  { return string_type::rfind( str ) == string_type::length() - str.length(); }

#ifdef DIRTY_GNU_STD_STRING_WORKAROUND
  virtual ~HTTPString()
  {
    clearBuffer();
  }

  const char* c_str() const
  { 
    clearBuffer();

    size_type len = string_type::length();
    m_cstrBuffer = new value_type[len + 1];
    traits_type::copy( m_cstrBuffer, string_type::data(), len );
    m_cstrBuffer[len] = '\0';

    return (const char*)m_cstrBuffer;
  }
    
private:
  mutable value_type* m_cstrBuffer; 

  void clearBuffer() const
  {
     delete[] m_cstrBuffer;
     m_cstrBuffer = NULL;
  }

#endif //DIRTY_GNU_STD_STRING_WORKAROUND
};

std::ostream& operator<<( std::ostream& out, const HTTPString& str );



} // EON http
#endif
