/***************************************************************************
                           httptraits.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: httptraits.hpp,v 1.5 2002/05/16 17:50:23 d3cline Exp $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HTTPTRAITS_HPP
#define HTTPTRAITS_HPP


#include <iostream>

class HTTPTraits
{
public:
  typedef int            int_type;
  typedef unsigned short chartype_type;
  typedef unsigned char  char_type;

  static const char_type WS    = ' ';
  static const char_type CR    = 13;
  static const char_type LF    = 10;
  static const char_type DDOTS = ':';

  inline static bool isTokenChar( char_type c )
  { return CHARTYPE_ATTRS[c] & TOKEN_C; }
  inline static bool isSeparatorChar( char_type c )
  { return CHARTYPE_ATTRS[c] & SEPARA_C; }
  inline static bool isUpperChar( char_type c )
  { return CHARTYPE_ATTRS[c] & UPALPHA_C; }
  inline static bool isLowerChar( char_type c )
  { return CHARTYPE_ATTRS[c] & LOALPHA_C; }
  inline static bool isDigitChar( char_type c )
  { return CHARTYPE_ATTRS[c] & DIGIT_C; }
  inline static bool isCntrlChar( char_type c )
  { return CHARTYPE_ATTRS[c] & CTL_C; }
  inline static bool isCRChar( char_type c )
  { return CHARTYPE_ATTRS[c] & CR_C; }
  inline static bool isLFChar( char_type c )
  { return CHARTYPE_ATTRS[c] & LF_C; }
  inline static bool isSpaceChar( char_type c )
  { return CHARTYPE_ATTRS[c] & SP_C; }
  inline static bool isHTabChar( char_type c )
  { return CHARTYPE_ATTRS[c] & HT_C; }
  inline static bool isHexChar( char_type c )
  { return CHARTYPE_ATTRS[c] & HEX_C; }
  inline static bool isDQuoteChar( char_type c ) // `"' 
  { return CHARTYPE_ATTRS[c] & DQUOTE_C; }
  inline static bool isSQuoteChar( char_type c ) // `''
  { return CHARTYPE_ATTRS[c] & SQUOTE_C; }
  inline static bool isTextChar( char_type c )
  { return CHARTYPE_ATTRS[c] & TEXT_C; }

  /**
   * assumes isDigitChar( digit ) == true
   */
  inline static int digitCharToInt( char_type digit )
  { return digit - '0'; }

  /**
   * assumes isHexChar( digit ) == true
   */
  inline static int hexCharToInt( char_type digit )
  {
    if( digit < ':' )
      return digitCharToInt( digit );
    else if( digit < 'G' )
      return digit - 'A' + 10;
    else
      return digit - 'a' + 10;
  }

private:
  HTTPTraits();
  HTTPTraits( const HTTPTraits& );

  const HTTPTraits& operator=( const HTTPTraits& );

  static const chartype_type TOKEN_C     = 0x0001;
  static const chartype_type SEPARA_C    = 0x0002;
  static const chartype_type UPALPHA_C   = 0x0004;
  static const chartype_type LOALPHA_C   = 0x0008;
  static const chartype_type DIGIT_C     = 0x0010;
  static const chartype_type CTL_C       = 0x0020;
  static const chartype_type CR_C        = 0x0040;
  static const chartype_type LF_C        = 0x0080;

  static const chartype_type SP_C        = 0x0100;
  static const chartype_type HT_C        = 0x0200;
  static const chartype_type HEX_C       = 0x0400;
  static const chartype_type DQUOTE_C    = 0x0800;
  static const chartype_type SQUOTE_C    = 0x1000;
  static const chartype_type TEXT_C      = 0x2000;

  static const chartype_type CHARTYPE_ATTRS[];
};

#endif
