/***************************************************************************
                           ircclient.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: ircclient.hpp,v 1.7 2002/03/09 12:49:34 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _IRCCLIENT_HPP
#define _IRCCLIENT_HPP

#include "irctoken.hpp"
#include "config.h"

using namespace std;

// FIXME: usage of IRCToken instead of string
class IRCClient {
protected:
  IRCToken       m_nickname;
  const IRCToken m_username;
  const IRCToken m_address;
  const IRCToken m_realname;

public:
  IRCClient( const IRCToken& nick,
	     const IRCToken& username,
	     const IRCToken& address,
	     const IRCToken& realname );
  virtual ~IRCClient();

  /**
   * @return nickname of irc client
   */
  inline const IRCToken& getNickname() const
  { return m_nickname; };

  /**
   * @return username / ident of irc client
   */
  inline const IRCToken& getUsername() const
  { return m_username; };

  /**
   * @return inet address of irc client
   */
  inline const IRCToken& getAddress() const
  { return m_address; };

  /**
   * @return realname of irc client
   */
  inline const IRCToken& getRealname() const
  { return m_realname; };

  /**
   * compares this' nickname with <tt>cNick</tt>
   *
   * @return nicks are equal
   */
  inline bool operator==(const IRCToken &cNick) const 
  { return m_nickname==cNick; };

  inline bool operator==( const IRCClient &c ) const 
  { return m_nickname == c.m_nickname; };

  inline bool operator<(const IRCToken &cNick) const 
  { return m_nickname < cNick; };

  inline bool operator<( const IRCClient &c) const
  { return m_nickname < c.m_nickname; };

  /**
   * comparses cNick and stored nickname
   * - used to provide basic support for sets
   */
  inline bool operator>(const IRCToken &cNick) const
  { return m_nickname > cNick; };

  inline bool operator>( const IRCClient &c) const
  { return m_nickname > c.m_nickname; };

  inline void setNickname(const IRCToken &newNick);
};

#endif
