/***************************************************************************
                           ircclientconnection.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: ircclientconnection.hpp,v 1.15 2002/05/29 19:43:39 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef IRCCLIENTCONNECTION_HPP
#define IRCCLIENTCONNECTION_HPP

#include <iostream>
#include "pollerclient.hpp"
#include "poller.hpp"
#include "tcpclientconnection.hpp"
#include "ircmessage.hpp"
#include "ircfileinbuf.hpp"
#include "config.h"

class IRCClientConnection : public TCPClientConnection, public Pollable
{
public:
  /**
   * <b>before the runPollSyscall() function gets called
   * 'connect' should be called before</b>
   *
   */
  IRCClientConnection( Poller* poller, 
		       len_type inBufLen,
		       len_type outBufLen );

  virtual ~IRCClientConnection()
  { /* m_pollerClient will unregister itselft */ }

  virtual const File* getPollableFile() const
  { return this; }

  const PollerClient& getPollerClient()
  { return m_pollerClient; }

  bool registerAt( Poller* poller )
  { return m_pollerClient.registerClient( poller ); }

  std::ostream& tcp()
  { return m_outstream; }

  len_type flushOutput()
  { return m_outBuf.doFlush(); }

protected:
  virtual void onConnected();

  virtual void onClosing();
  
  virtual void onInput();

  virtual void onOutput();

  virtual void onError()
  {}

  virtual void onHangup()
  {}

  /*
   * will never get called due to PollerClient
   */
  virtual void onRemoved()
  {}

  /**
   * this class' hook. called after a complete message has been received
   * and processed
   */
  virtual void onMessage( const IRCServerMessage& )
  {}

  /**
   * this class' hook. called after a tcp connection just has been established
   * and output stream is ready to write data
   */
  virtual void onDoLogin()
  {}
  
  IRCFileInBuf& getInputBuffer()
  { return m_inBuf; }
  
private:
  typedef enum { DO_LOGIN, DID_LOGIN } loginstate_type;

  IRCFileInBuf    m_inBuf;
  FileOutBuf      m_outBuf;
  std::ostream    m_outstream;

protected:
  PollerClient    m_pollerClient;

private:
  loginstate_type m_loginState;
};


#endif
