/***************************************************************************
                           ircfileinbuf.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: ircfileinbuf.hpp,v 1.6 2002/03/09 12:49:34 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef IRCFILEINBUF_HPP
#define IRCFILEINBUF_HPP

#include "filebuf.hpp"
#include "ircmessagetokenizer.hpp"
#include "ircmessage.hpp"

class IRCFileInBuf : public FileInBuf
{
public:
  typedef FileInBuf::len_type       len_type;
  typedef FileInBuf::data_type      data_type;
  typedef FileInBuf::data_reference data_reference;
  typedef int                       int_type;
  
  // the maximum number of irc tokens to be read
  const static int_type MAX_TOKENS = 15;
  const static len_type MAX_MSGLEN = 512; // -> rfc

  /**
   *
   * @param maxMsgArgs - the miximum number of tokens which 
   *                     should be parsed out of the last irc token
   */
  IRCFileInBuf( File&f, len_type buflen, int maxMsgArgs);

  /**
   * @return <tt>NULL</tt>: no complete message has been retreived
             else IRCServerMessage has been parsed successfully
   */
  const IRCServerMessage* read( data_reference = NULL, len_type = 0 );

private:
  const IRCFileInBuf& operator= ( const IRCFileInBuf& );
  IRCFileInBuf( const IRCFileInBuf& );
  IRCFileInBuf();

  IRCMessageTokenizer m_tokenizer;
  data_reference      m_maxFilled; // signalized end of fill state
};

#endif
