/***************************************************************************
                           ircmessage.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: ircmessage.hpp,v 1.8 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef IRCMESSAGE_HPP
#define IRCMESSAGE_HPP

#include "irctoken.hpp"

class IRCMessageTokenizer;

/**
 * unmodifieable irc message
 *
 * - gets used by ircclientconnection
 * 
 */
class IRCServerMessage
{
public:
  typedef int          int_type;
  typedef unsigned int ircoptidx_t; // for backward compatibility

  friend class IRCMessageTokenizer;

  const static ircoptidx_t IDX_UNSET=~0;
  const static ircoptidx_t IDX_HOSTMASK=0;
  const static ircoptidx_t IDX_CMD=1;
  const static ircoptidx_t IDX_TARGET=2;
  const static ircoptidx_t IDX_OPT1=3;
  const static ircoptidx_t IDX_OPT2=4;
  const static ircoptidx_t IDX_OPT3=5;
  const static ircoptidx_t IDX_OPT4=6;
  const static ircoptidx_t IDX_OPT5=7;
  const static ircoptidx_t IDX_OPT6=8;
  const static ircoptidx_t IDX_OPT7=9;
  const static ircoptidx_t IDX_OPT8=10;
  const static ircoptidx_t IDX_OPT9=11;
  const static ircoptidx_t IDX_OPT10=12;
  const static ircoptidx_t IDX_OPT11=13;
  const static ircoptidx_t IDX_OPT12=14;
  const static ircoptidx_t IDX_OPT13=15;
  const static ircoptidx_t IDX_MSG=16;

  IRCServerMessage( const IRCToken* ircTokens, 
		    int_type ircTokLen, 
		    const IRCToken* msgTokens, 
		    int_type msgTokLen,
		    bool clientMessage = false )
  : m_ircTokens( ircTokens ),
    m_ircTokenNum( ircTokLen ),
    m_msgTokens( msgTokens ),
    m_msgTokenNum( msgTokLen ),
    m_clientMessage( clientMessage )
  {}
  
  const IRCToken* getToken() const
  { return m_ircTokens; }

  const IRCToken& getToken( ircoptidx_t idx ) const
  { 
    assert( idx < m_ircTokenNum );
    return m_ircTokens[idx];
  }

  const IRCToken& getSender() const
  { assert( m_clientMessage ); return m_ircTokens[IDX_HOSTMASK]; }
    
  const IRCToken& getCommand() const
  { return m_clientMessage ? m_ircTokens[IDX_CMD] : *m_ircTokens; }

  const IRCToken& getTarget() const
  { return m_clientMessage ? m_ircTokens[IDX_TARGET] : m_ircTokens[IDX_CMD]; }

  const IRCToken& getMessage() const
  { return m_msgTokens[m_msgTokenNum-1]; }

  const IRCToken* getMessageToken() const
  { return m_msgTokens; }

  const IRCToken& getMessageToken( ircoptidx_t idx ) const
  {
    assert ( idx < m_msgTokenNum );
    return m_msgTokens[idx];
  }
  
  ircoptidx_t end() const
  { return m_ircTokenNum; }
  ircoptidx_t begin() const
  { return 0; }
  
  int_type length() const
  { return m_ircTokenNum; }

  int_type messageTokenNum() const
  { return m_msgTokenNum; }

  bool isClientMessage() const
  { return m_clientMessage; }

protected:
  IRCServerMessage()
    : m_ircTokens( NULL ),
      m_ircTokenNum( 0 ),
      m_msgTokens( NULL ),
      m_msgTokenNum( 0 )
  {}
  
  void assign( const IRCToken* ircTokens, 
	       ircoptidx_t ircTokenNum,
	       const IRCToken* msgTokens,
	       ircoptidx_t msgTokenNum,
	       bool clientMessage )
  {
    m_ircTokens     = ircTokens;
    m_ircTokenNum   = ircTokenNum;
    m_msgTokens     = msgTokens;
    m_msgTokenNum   = msgTokenNum;
    m_clientMessage = clientMessage;
  }
  
private:
  IRCServerMessage( const IRCServerMessage& msg )
    : m_ircTokens( msg.m_ircTokens ),
      m_ircTokenNum( msg.m_ircTokenNum ),
      m_msgTokens( msg.m_msgTokens ),
      m_msgTokenNum( msg.m_msgTokenNum )
  {}
  const IRCServerMessage& operator=( const IRCServerMessage& );

  const IRCToken* m_ircTokens;
  ircoptidx_t     m_ircTokenNum;
  const IRCToken* m_msgTokens;
  ircoptidx_t     m_msgTokenNum;
  bool            m_clientMessage;
};

#endif
