/***************************************************************************
                           irctokenizer.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: irctokenizer.hpp,v 1.6 2002/03/09 12:49:34 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef IRCTOKENIZER_HPP
#define IRCTOKENIZER_HPP

#include "irctoken.hpp"
#include "tokenizer.hpp"

/**
 * IRCTokenizer makes token of a given string:-)
 * IRCTokenizer is never authorative for the given string.
 * More the tokens point to the buffer. Therefore
 * Whenever IRCTokenizer gets used make sure the buffer is
 * always 'there'.
 */
class IRCTokenizer : protected Tokenizer {
public:
  typedef IRCToken::reference_type reference_type;
  typedef IRCToken::size_type size_type;
  typedef IRCToken::data_type data_type;
  typedef int                 int_type;
  typedef unsigned int        len_type;


  /**
   * the maximum number of tokens a valid ircmessage may carry
   */
  static const int_type  TOK_NUM ;
  
  /**
   * the default token delimiter ( ' ' )
   */
  static const data_type TOK_DELIM;

  /**
   * the default ircmessage delimiter ( '\n' )
   */
  static const data_type MSG_DELIM;

  /**
   * the default long / last irctoken delimiter ( ':' )
   */
  static const data_type LTOK_DELIM;

  IRCTokenizer();
  
  /**
   * tokenizes 'buf'
   * @return the range which has been tokenized
   *
   */
  len_type update( reference_type buf, size_type length );

private:
  IRCTokenizer( const IRCTokenizer& );
  const IRCTokenizer& operator=( const IRCTokenizer& );
};

#endif
