#ifndef MESSAGEHEADER_HPP
#define MESSAGEHEADER_HPP

/***************************************************************************
                           httpclientconnection.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.nets

    $Id: messageheader.hpp,v 1.10 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <utility>
#include <iostream>
#include "http_common.hpp"

namespace http
{
  /**
   * A 'message-header' as defined in RFC 2616
   * which is part of a request after 'first-line'
   * 
   */
  class MessageHeader
  {
  public:
    typedef HTTPString FieldName;
    typedef HTTPString FieldValue;

    static const MessageHeader ChunkedTransferEncoding;
    static const MessageHeader ConnectionClose;
    static const MessageHeader ConnectionKeepAlive;
    static const MessageHeader Server;
    static const MessageHeader AcceptRanges;
    static const MessageHeader PlainTextContentType;


    MessageHeader()
      : m_fieldName(),
	m_fieldValue(),
	m_valueSet( false )
    {}

    MessageHeader( const FieldName& key, 
		   const FieldValue& value )
      : m_fieldName( key ),
	m_fieldValue( value ),
	m_valueSet( true )
    {}

    MessageHeader( const MessageHeader& header )
      : m_fieldName( header.m_fieldName ),
	m_fieldValue( header.m_fieldValue ),
	m_valueSet( header.m_valueSet )
    {}

    const MessageHeader& operator=( const MessageHeader& header )
    {
      if( &header != this ) {
	m_fieldName  = header.m_fieldName;
	m_fieldValue = header.m_fieldValue;
	m_valueSet   = header.m_valueSet; 
      }
      return *this;
    }

    const FieldName& getFieldName() const
    { return m_fieldName; }

    const FieldValue& getFieldValue() const
    { return m_fieldValue; }

    void assign( const FieldName& key, const FieldValue& value )
    {
      m_fieldName     = key;
      m_fieldValue    = value;
      m_valueSet = true;
    }
    void assign( const FieldName& key )
    { 
      m_fieldName = key;
      m_valueSet  = false;
    }

    void reset()
    { 
      m_valueSet = false; 
      m_fieldName.erase(); 
      m_fieldValue.erase();  
    }

    bool fieldValueSet() const
    { return m_valueSet; }

  private:
    FieldName  m_fieldName;
    FieldValue m_fieldValue;

    bool m_valueSet;    
  };

  inline std::ostream& operator<<( std::ostream& out, const MessageHeader& header )
  {
    out << header.getFieldName() << HTTPTraits::DDOTS << HTTPTraits::WS;
    if( header.fieldValueSet() )
      out << header.getFieldValue();
    return out;
  }

  inline const MessageHeader& httpconnectionTypeToHeader( http::httpresponseconnectiontype_type connection )
  {
    using namespace http;

    switch( connection )
    {
    case HTTPCONN_CLOSE:
      return MessageHeader::ConnectionClose;
    case HTTPCONN_KEEPALIVE:
      return MessageHeader::ConnectionKeepAlive;
    case HTTPCONN_CHUNKED:
      return MessageHeader::ConnectionKeepAlive;
    case HTTPCONN_UNKNOWN:
      return MessageHeader::ConnectionClose;
    }
    return MessageHeader::ConnectionClose;
  }


}// EON http


#endif
