/***************************************************************************
                           pollerfd.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: pollerfd.hpp,v 1.7 2002/04/26 21:25:22 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef POLLERFD_HPP
#define POLLERFD_HPP

extern "C" {
#include <poll.h>

}
#include "pollable.hpp"

class PollerFD : public pollfd {
public:
  typedef unsigned short event_type;

  static const event_type EVT_IN   = POLLIN;
  static const event_type EVT_PRI  = POLLPRI;
  static const event_type EVT_OUT  = POLLOUT;
  static const event_type EVT_ERR  = POLLERR;
  static const event_type EVT_HUP  = POLLHUP;
    
  PollerFD()
  { 
    events = 0;
    fd = -1;
  }

  PollerFD( const Pollable *s )
  { 
    events = 0;
    fd = s->getPollableFile()->getFD(); 
  }

  short getREvents() const
  { return revents; }
  short getEvents() const
  { return events; }

  void registerEvent( event_type type ) 
  { events |= type; }
  
  void unregisterEvent( event_type type )
  { events &= ~type; }

  void unregisterAllEvents()
  { events = 0; }

  const PollerFD& operator=( Pollable* sock )
  {
    fd = sock->getPollableFile()->getFD();
    return *this;
  }
};

#endif
