/***************************************************************************
                           requestfilebuffer.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: requestfilebuffer.hpp,v 1.6 2002/05/16 17:50:24 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef REQUESTPARSER_HPP
#define REQUESTPARSER_HPP


#include "file.hpp"
#include "filebuf.hpp"
#include "requeststartline.hpp"
#include "messageheader.hpp"
#include "http_common.hpp"
#include "httptraits.hpp"

namespace httpd
{
  class RequestFileBuffer : public FileInBuf
  {
  public:
    typedef int return_type;

    RequestFileBuffer( File& f, len_type buflen );
    
    /**
     * signalizes that a read error must have happened
     */
    static const return_type ERR_BUFFEREMPTY = -2;
    static const return_type READ_OK         =  1;
    static const return_type ERR_MALFORMED   =  0;
    /**
     * this will <b>only</b> be returned by
     * 'read( MessageHeader& )'
     * - signalizes that a header has been completely read
     */
    static const return_type HEADER_TERMINATE = 2;
    
    /**
     * - buffer will not check whether EOF has been read before
     *
     * @return whether the RequestFileBuffer was able to read a start-line
     */ 
    return_type read( http::RequestStartline& startLine );
    /**
     * @return whether the RequestFileBuffer was able to read a message-header
     */
    return_type read( http::MessageHeader& messageHeader );
    
  protected:
    /**
     * reads a 'token' as defined in rfc2616 from the _buffer_
     * - the function will not fill the buffer
     * - the function will only use operator+= on target
     */
    typedef bool (genericChecker)( HTTPTraits::char_type );
    static FileInBuf::data_reference readToken( http::HTTPString& target,
						data_reference start,
						data_reference end );
    static FileInBuf::data_reference readText( http::HTTPString& target,
					       data_reference start,
					       data_reference end );
    static FileInBuf::data_reference readGeneric( http::HTTPString& target,
						  genericChecker* checkFunc,
						  data_reference start,
						  data_reference end );
    
  };
}
#endif
