/***************************************************************************
                           requeststartline.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: requeststartline.hpp,v 1.6 2002/05/21 19:55:27 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef REQUESTSTARTLINE_HPP
#define REQUESTSTARTLINE_HPP


#include "uri.hpp"
#include "http_common.hpp"

namespace http
{
  /**
   * A 'Request-line' as defined in RFC 2616
   * - provides conversion from strings into more
   *   native datatypes.
   */
  class RequestStartline
  {
  public:
    RequestStartline();
    RequestStartline( const HTTPString& method,
		      const HTTPString& uri );
    RequestStartline( const HTTPString& method,
		      const HTTPString& uri,
		      const HTTPString& version );
    RequestStartline( httpmethod_type method,
		      const uri::URL uri,
		      httpprotversion_type version );
    RequestStartline( const RequestStartline& );

    const RequestStartline& operator=( const RequestStartline& );


    void update( const HTTPString& method,
		 const HTTPString& uri );
    void update( const HTTPString& method,
		 const HTTPString& uri,
		 const HTTPString& version );

    void reset();

    httpmethod_type getMethod() const
    { return m_method; }

    const uri::URL& getURL() const
    { return m_url; }

    httpprotversion_type getProtocolVersion() const
    { return m_protocolVersion; }

    static void setDefaultProtocolVersion( httpprotversion_type protocol )
    { m_defaultProtVersion = protocol; }
    
    static httpprotversion_type getDefaultProtocolVersion()
    { return m_defaultProtVersion; }

  protected:
    httpmethod_type      m_method;
    uri::URL             m_url;
    httpprotversion_type m_protocolVersion;

    static httpprotversion_type stringToProtVersion( const HTTPString& version );
    static httpmethod_type stringToMethod( const HTTPString& method );

  private:
    struct StringProt
    {
      const HTTPString           m_protString;
      const httpprotversion_type m_protVal;
    };
    struct StringMethod
    {
      const HTTPString      m_methodString;
      const httpmethod_type m_methodVal;
    };

    static const StringMethod   STRING_TO_METHOD_TAB[];
    static const StringProt     STRING_TO_PROT_TAB[];
    static httpprotversion_type m_defaultProtVersion;
  };
}

#endif
