/***************************************************************************
                           response.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: response.hpp,v 1.9 2002/04/26 21:25:22 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RESPONSE_HPP
#define RESPONSE_HPP

#include <iostream>
#include <vector>
#include "http_common.hpp"
#include "responsestatusline.hpp"
#include "messageheader.hpp"
#include "responsebody.hpp"
#include "config.h"

namespace httpd
{  
  class Request;

  class Response
  {
  public:
    typedef vector<http::MessageHeader> container_type;
    typedef http::httpresponseconnectiontype_type connection_type;
    typedef http::httpresponseconnectiontype_type bodyencoding_type;


    /**
     * creates a new Response instance<br>
     * ctor's default responseEncoding setting is: HTTPCONN_CLOSE
     * the connection type is taken from the request.
     */
    Response( http::ResponseStatusline::responsecode_type responseCode,
	      http::httpprotversion_type protocolVersion,
	      const Request* const response );

    virtual ~Response()
    {
#ifdef VERBOSE_MEMORY
      cout << "Response] destroyed" << endl;
#endif
    }

    const container_type& getMessageHeaders() const
    { return m_messageHeaders; }

    const http::ResponseStatusline& getStatusLine() const
    { return m_statusLine; }

    void appendMessageHeader( const http::MessageHeader& header )
    { m_messageHeaders.push_back( header ); }


    /**
     * @return NULL - no ResponseBody available
     */
    ResponseBody* getResponseBody() const
    { return m_responseBody; }

    const Request* getRequest() const
    { return m_request; }

    /**
     * if not other defined body encoding
     * defaults to 'HTTPCONN_UNKNOWN'
     */
    bodyencoding_type getResponseBodyEncoding() const
    { return m_bodyEncoding; }

    connection_type getConnectionType() const
    { return m_connectionType; }

    void setResponseBody( ResponseBody* responseBody )
    { m_responseBody = responseBody; }

    void setResponseBodyEncoding( bodyencoding_type type )
    { m_bodyEncoding = type; }

    void setConnectionType( connection_type type )
    { m_connectionType = type; }

  private:
    Response( const Response& );
    const Response& operator=( const Response& );

    static const int DEF_HEADERNUM = 5;

    container_type        m_messageHeaders;
    const http::ResponseStatusline m_statusLine;
    mutable ResponseBody* m_responseBody;
    const Request* const  m_request;
    bodyencoding_type     m_bodyEncoding;
    connection_type       m_connectionType;
  };

  Response* make_response( http::ResponseStatusline::responsecode_type responseCode,
			   http::httpprotversion_type protocolVersion,
			   const Request* request );

} // EON httpd

#endif
