/***************************************************************************
                           responsefilebuffer.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: responsefilebuffer.hpp,v 1.3 2002/04/09 19:32:55 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef RESPONSEFILEBUFFER_HPP
#define RESPONSEFILEBUFFER_HPP

#include "http_common.hpp"
#include "filebuf.hpp"

namespace httpd
{
/**
 * will only output data<br>
 * <b>Note</b>: this class actes on a given buffer.
 *              therefore the buffer may _not_ get REALLOCATED<br><br>
 * in HTTPCONN_CHUNKED mode this buffer will reserve a block of fixed
 * size at the beginning of the buffer to write chunk information.
 *<br><br>
 * I have noticed this class to be very slow compared with FileOutBuf
 * ( about 3 times more lazy :/ )
 */
class ResponseFileBuffer : public FileOutBuf
{
public:
  typedef http::httpresponseconnectiontype_type buffer_type;

  ResponseFileBuffer( File& f, 
		      len_type buflen = 2060,
		      buffer_type type = http::HTTPCONN_CLOSE );

  virtual ~ResponseFileBuffer();

  void setBodyTransportEncoding( buffer_type type );

protected:
  virtual len_type flushBuffer();

private:
  buffer_type      m_bufferType;
  data_type* const m_bufferBase;
  const len_type   m_bufferLen;
  int              m_reservedBytes;
  char*            m_chunkHeaderBuffer;

  void setupBuffer();
  /**
   * @return the number of unused characters of the reserved buffer area
   */
  data_reference addChunkHeader();

};

}//EON httpd
#endif
