/***************************************************************************
                           responsestatusline.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: responsestatusline.hpp,v 1.3 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef RESPONSESTATUSLINE_HPP
#define RESPONSESTATUSLINE_HPP

#include "http_common.hpp"

namespace http
{
  class ResponseStatusline 
  {
  public:
    typedef short responsecode_type;
    
    static const responsecode_type RESP_CONTINUE                   =  0;
    static const responsecode_type RESP_SWITCH_PROTCLS             =  1;
    static const responsecode_type RESP_OK                         =  2;
    static const responsecode_type RESP_CREATED                    =  3;
    static const responsecode_type RESP_ACCEPTED                   =  4;
    static const responsecode_type RESP_NON_AUTHORATIVE            =  5;
    static const responsecode_type RESP_NO_CONTENT                 =  6;
    static const responsecode_type RESP_RESET_CONTENT              =  7;
    static const responsecode_type RESP_PARTIAL_CONTENT            =  8;
    static const responsecode_type RESP_MULTIPLE_CHOICES           =  9;
    static const responsecode_type RESP_MOVED_PERMANENTLY          = 10;
    static const responsecode_type RESP_FOUND                      = 11;
    static const responsecode_type RESP_SEE_OTHER                  = 12;
    static const responsecode_type RESP_NOT_MODIFIED               = 13;
    static const responsecode_type RESP_USE_PROXY                  = 14;
    static const responsecode_type RESP_TEMPORARY_REDIRECT         = 15;
    static const responsecode_type RESP_BAD_REQUEST                = 16;
    static const responsecode_type RESP_UNAUTHORIZED               = 17;
    static const responsecode_type RESP_PAYMENT_REQUIRED           = 18; // :-)
    static const responsecode_type RESP_FORBIDDEN                  = 19;
    static const responsecode_type RESP_NOT_FOUND                  = 20;
    static const responsecode_type RESP_METHOD_NOT_ALLOWED         = 21;
    static const responsecode_type RESP_NOT_ACCEPTABLE             = 22;
    static const responsecode_type RESP_PROXY_AUTH_REQU            = 23;
    static const responsecode_type RESP_REQUEST_TIMEOUT            = 24;
    static const responsecode_type RESP_CONFLICT                   = 25;
    static const responsecode_type RESP_GONE                       = 26;
    static const responsecode_type RESP_LENGTH_REQUIRED            = 27;
    static const responsecode_type RESP_PRECONDITION_FAILED        = 28;
    static const responsecode_type RESP_REQUEST_ENTITY_TOO_LARGE   = 29;
    static const responsecode_type RESP_URI_TOO_LARGE              = 30;
    static const responsecode_type RESP_UNSUPP_MEDIA_TYPE          = 31;
    static const responsecode_type RESP_REQU_RANGE_NOT_SATISFIABLE = 32;
    static const responsecode_type RESP_EXPECTATION_FAILED         = 33;
    static const responsecode_type RESP_INTERNAL_SERVER_ERROR      = 34;
    static const responsecode_type RESP_NOT_IMPLEMENTED            = 35;
    static const responsecode_type RESP_BAD_GATEWAY                = 36;
    static const responsecode_type RESP_SERVICE_UNAVAILABLE        = 37;
    static const responsecode_type RESP_GATEWAY_TIMEOUT            = 38;
    static const responsecode_type RESP_HTTP_VERSION_UNSUPP        = 39;
    
    ResponseStatusline( httpprotversion_type version,
		responsecode_type respCode )
      : m_responseCode( RESPONSE_TAB[respCode].statusCode ),
	m_protocolVersion( version ),
	m_reason( RESPONSE_TAB[respCode].reasonPhrase )
    {}
    
    const HTTPString& getReason() const
    { return m_reason; }
    const httpprotversion_type getProtocolVersion() const
    { return m_protocolVersion; }
    const responsecode_type getResponseCode() const
    { return m_responseCode; }

  protected:
    static const struct ResponseValueString {
      const responsecode_type statusCode;
      const HTTPString        reasonPhrase;
    } RESPONSE_TAB[];

  private:
    ResponseStatusline();
    ResponseStatusline( const ResponseStatusline& );

    const responsecode_type    m_responseCode;
    const httpprotversion_type m_protocolVersion;
    const HTTPString&          m_reason;
  };

  std::ostream& operator<<( std::ostream& out,  const ResponseStatusline& statusLine );


}//EON http

#endif
