/***************************************************************************
                           socket.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: socket.hpp,v 1.7 2002/05/23 15:02:17 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef SOCKET_HPP
#define SOCKET_HPP

extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
}
#include "inethostaddress.hpp"
#include "file.hpp"

class Socket: public File {
public:
  Socket()
  {}

protected:
  virtual int createSocket() = 0;
  Socket( int f ) 
    : File( f )
  {}

private:
  Socket( const Socket &);
  const Socket& operator=( const Socket& );
};

class TCPSocket : public Socket {
public:
  typedef unsigned short port_type;
  
  TCPSocket();

  virtual void close()
  { ::shutdown( fd, SHUT_RDWR ); fd = eof; }

protected:
  TCPSocket( int f )
    : Socket( f )
  {}

  virtual int createSocket();

private:
  TCPSocket( const TCPSocket &);
  const TCPSocket& operator=( const TCPSocket& );
};

class TCPPort : public TCPSocket {
public:
  static int DEF_BACKLOG;

  TCPPort( const InetHostAddress &bind, port_type port, 
	   int backlog = DEF_BACKLOG );
  
  /**
   * waits for incoming connections.
   * calls onConnect() after a client has connected the port
   */
  void listen( );

  static void setDefaultBacklog( int newBacklog )
  { DEF_BACKLOG = newBacklog; };

protected:
  /**
   * signalizes that a client has connected the tcp port
   * @param fd File descriptor of the connected client
   * @param from adresse from where the client has been connected
   * @param fromLen size of 'from'
   */
  virtual void onConnected( int sockFD, 
			    const sockaddr_in &from, 
			    socklen_t fromLen)
  {}

private:
  TCPPort();
  TCPPort( const TCPPort &);

  void setupSocket( const hostent * const he, 
		    port_type port,
		    int backlog );
};


#endif
