/***************************************************************************
                           tcpclientconnection.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: tcpclientconnection.hpp,v 1.13 2002/04/26 21:25:22 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef TCPCLIENTCONNECTION_HPP
#define TCPCLIENTCONNECTION_HPP

#include "socket.hpp"
#include "fcntl.h"

class TCPClientConnection : public TCPSocket
{
public:
  typedef File::len_type       len_type;
  typedef File::data_reference data_reference;
  typedef const data_reference const_data_reference;

  TCPClientConnection()
    : TCPSocket()
  {
  }

  virtual ~TCPClientConnection()
  {}
  
  virtual void close();

  /** 
   * establishes a connection with a remote host
   *
   * @param addr remote host to connect
   * @param port a port at 'addr'
   * @return in case of an error: File::eof;
   *         else 1
   */
  len_type connect( const InetHostAddress& addr,
		    port_type port );

  /**
   * this function is reserved for the buffers
   */
  virtual len_type read( data_reference,
			 len_type len );

  /**
   * this function is reserved for the buffers
   */
  virtual len_type write( const_data_reference,
			  len_type len );

protected:
  TCPClientConnection( TCPSocket::fd_type Fd )
    : TCPSocket( Fd )
  {}
  
  // hooks
  virtual void onConnecting()
  {}

  virtual void onConnected()
  {}

  virtual void onClosing()
  {}

  virtual void onClosed()
  {}

private:
  TCPClientConnection( const TCPClientConnection& );
  const TCPClientConnection& operator=( const TCPClientConnection& );
};

#endif
