/***************************************************************************
                           tokenizer.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: tokenizer.hpp,v 1.4 2002/03/09 12:49:34 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef TOKENIZER_HPP
#define TOKENIZER_HPP

#include "irctoken.hpp"

class Tokenizer
{
public:
  typedef IRCToken::data_type      data_type;
  typedef IRCToken::reference_type reference_type;
  typedef IRCToken::size_type      len_type;
  typedef int                      int_type;

  Tokenizer( int tokNum, 
	     data_type delim1 = ' ', 
	     data_type delim2 = '\n' );
  virtual ~Tokenizer();

  int_type update( reference_type buf, len_type len );
  const IRCToken* getToken() const
  { return m_token; }
  data_type getDelim1() const
  { return m_delim1; }
  data_type getDelim2() const
  { return m_delim2; }
  int getMaxTokenNum() const
  { return m_maxTokenNum; }
  int getTokenNum() const
  { return m_tokenNum; }

protected:
  const int       m_maxTokenNum;
  const data_type m_delim1; // token delimiter
  const data_type m_delim2; // end delimiter 
  IRCToken* const m_token; // array of parsed token
  int             m_tokenNum; // actual number of parsed token

private:
  Tokenizer();
  const Tokenizer& operator=( const Tokenizer& );
  Tokenizer( const Tokenizer& );
};

#endif
