/***************************************************************************
                           uri.hpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: uri.hpp,v 1.5 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef URI_HPP
#define URI_HPP

#include <iostream>
#include "socket.hpp" // for TCPSocket::port_type
#include "httpstring.hpp"

namespace uri
{
  class URI
  {
  public:
    URI();
    URI( const http::HTTPString& HTTPString );
    URI( const URI& );
    const URI& operator=( const URI& );

    const URI& operator=( const http::HTTPString& urlHTTPString )
    { parse( urlHTTPString); return *this; }

    void parse( const http::HTTPString& urlHTTPString );

    const http::HTTPString& getProtocol() const
    { return m_protocol; }
    /**
     * maybe we should return an inethostaddress
     * object here but as InetHostAddress seems to
     * be such buggy ...
     */
    const http::HTTPString& getHost() const
    { return m_host; }
    TCPSocket::port_type getPort() const
    { return m_port; }
    const http::HTTPString& getDocument() const
    { return m_document; }
    const http::HTTPString& getQuery() const
    { return m_query; }

    bool protocolSet() const
    { return !m_protocol.empty(); }
    bool hostSet() const
    { return !m_host.empty(); }
    bool portSet() const
    { return m_portSet; }
    bool documentSet() const
    { return !m_document.empty(); }
    bool querySet() const
    { return !m_query.empty(); }

    // isMalformed / isIncomplete
    bool isIncomplete() const
    { return m_invalidURI; }

  private:
    static const http::HTTPString PROT_DIV;
    static const http::HTTPString PORT_DIV;
    static const http::HTTPString DOC_DIV;
    static const http::HTTPString QUERY_DIV;

  protected:
    http::HTTPString     m_protocol;
    http::HTTPString     m_host;
    TCPSocket::port_type m_port;
    http::HTTPString     m_document;
    http::HTTPString     m_query;

    bool m_portSet;
    /**
     * gets falsed by 'reset()' and maybe( after an successfull parsing )
     * trued by parse()
     */
    bool m_invalidURI;

    void reset();
  };

  typedef URI URL;

  std::ostream& operator<<( std::ostream& out, const URI& uri );
}
#endif
