/***************************************************************************
                           vhost.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: vhost.hpp,v 1.14 2002/04/09 19:32:55 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _VHOST_HPP
#define _VHOST_HPP

#include <string>
#include <set>
#include <iostream>
#include "dereferencecomparator.hpp"
class VHost;
#include "hsclient.hpp"
#include "config.h"
#include "irctoken.hpp"

class VHost {  
private:
  static bool defaultPublicPolicy;

public:
  typedef set<const HSClient*, DereferenceLess<const HSClient*> > ClientSet;
  
  VHost( const string &address, bool vhostPublic = defaultPublicPolicy );
  VHost( const string &ident, const string &address, bool vhostPublic = defaultPublicPolicy );
  VHost( const IRCToken& address, bool vhostPublic = defaultPublicPolicy );
  VHost( const IRCToken& ident, const IRCToken& address, bool vhostPublic = defaultPublicPolicy );
  virtual ~VHost();

  void removeReference( const HSClient *c );
  void addReference( const HSClient * c );
  bool addressOnly() const
  { return m_addressOnly; };
  const ClientSet* getReferences() const
  { return references; };

  /**
   * @return the number of HSClients which link to this vhost
   */
  int referenceCnt() const
  { return references->size(); };

  /**
   * @return username
   */
  const IRCToken &getIdent() const
  { return *ident; };
  /**
   * @return hostname
   */
  const IRCToken &getAddress() const
  { return *address; };
  /**
   * @return is a public vhost
   */
  bool isPublic() const
  { return m_public; };

  void setPublic( bool publicVHost)
  { m_public = publicVHost; };

  static void setDefaultPublicPolicy( bool policy )
  { defaultPublicPolicy = policy; };
  
  static bool getDefaultPublicPoliyc()
  { return defaultPublicPolicy; };
    
  bool operator == (const IRCToken &cAddress ) const
  { return *address == cAddress; };

  bool operator < ( const IRCToken &cAddress ) const
  { return *address < cAddress; };

  bool operator > ( const IRCToken &cAddress ) const
  { return *address > cAddress; };

  bool operator==( const VHost &h ) const
  { return *address == *h.address; };

  bool operator < ( const VHost &h ) const
  { return *address < *h.address; };

  bool operator > ( const VHost &h ) const
  { return *address > *h.address; };

  static const IRCToken::size_type MAX_HOSTLEN;
  static const IRCToken::size_type MAX_USERLEN;

private:
  IRCToken*  ident;
  IRCToken*  address;
  ClientSet* references;
  bool       m_public;
  bool       m_addressOnly;

  void conformToRFC();
  
  VHost();
  VHost( const VHost& vh );
  const VHost & operator = ( const VHost & );
};

ostream & operator << ( ostream &out, const VHost &vhost );

#endif
