/***************************************************************************
                           vhostlist.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: vhostlist.hpp,v 1.17 2002/05/19 17:54:03 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _VHOSTLIST_HPP
#define _VHOSTLIST_HPP

#include <map>
#include <vector>
#include <list>
#include "vhost.hpp"
#include "dereferencecomparator.hpp"
#include "config.h"
#include "irctoken.hpp"

class VHostList : protected virtual map<const VHost*, unsigned int, DereferenceLess<const VHost*> >
{
public:
  typedef map<const VHost*, unsigned int, DereferenceLess<const VHost*> > map_type;
  typedef vector<map_type::key_type> vector_type;
  typedef map_type::const_iterator const_iterator;
  typedef map_type::value_type storage_type;
  typedef unsigned int id_type;
  typedef list<id_type> list_type;

  static const id_type npos = ~0;

  using map_type::size;

  VHostList( int initialCapacity = 100 );
  virtual ~VHostList();

  const_iterator begin() const
  { return map_type::begin(); }

  const_iterator end() const
  { return map_type::end(); }

  bool empty()
  { return m_nextId == 0 || m_nextId == m_unusedIds.size(); }

  /**
   * @return insertion successfully completed
   */ 
  bool insertVHost( const VHost *v );
  
  bool eraseVHost( const VHost *v );

  const VHost * getVHost( const IRCToken &address ) const;

  const VHost* getVHost( id_type id ) const;

  id_type getVHostID( const VHost* ) const;
  
  void clear();

  void optimize();

private:
  VHostList(const VHostList &);
  const VHostList & operator = (const VHostList &);

  id_type     m_nextId;
  vector_type m_idxVector;
  list_type   m_unusedIds;

  class VHostAddressPredicate {
    const IRCToken& m_refName;
    
  public:
    VHostAddressPredicate( const IRCToken &refName )
      : m_refName( refName )
    {}
    bool operator() ( const VHostList::storage_type& st ) const
    { return *st.first == m_refName; }
  };
}; 


#endif
