#include "cgi.hpp"
#include "method.hpp"
#include "request.hpp"
#include "requeststartline.hpp"
#include "hsconfiguration.hpp"
#include "hsconfigfile.hpp"
#include "hostserv.hpp"
#include "config.h"

namespace httpd
{

using std::string;
using http::HTTPString;


class HSConfigurationCGIScriptHandler : public CGIScriptHandler
{
   public:
      typedef void (HSConfiguration::*configfunc_type)( const string& str );

      HSConfigurationCGIScriptHandler( configfunc_type func )
	 : m_handler( func )
      {}

      virtual ~HSConfigurationCGIScriptHandler()
      {}

      virtual void execute( const http::HTTPString& key, const http::HTTPString& value )
      { (HSConfiguration::getConfiguration().*m_handler)( string( (char*)value.data(), value.length() ) ); }
      // ok this calls are _extremely_ inefficient

   private:
      configfunc_type const m_handler;
};

const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_SERVERNAME =               HSConfigFile::SERVER_NAME;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_SERVERDESCRIPTION =        HSConfigFile::SERVER_DESCRIPTION;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_REMOTEHOST =               HSConfigFile::REMOTE_SERVERNAME;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_REMOTEPASSWORD =           HSConfigFile::REMOTE_PASSWORD;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_HOSTSERVNICKNAME =         HSConfigFile::HOSTSERV_NICK;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_HOSTSERVUSERNAME =         HSConfigFile::HOSTSERV_USERNAME;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_HOSTSERVADDRESS =          HSConfigFile::HOSTSERV_ADDRESS;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_HOSTSERVREALNAME =         HSConfigFile::HOSTSERV_REALNAME;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_HOSTSERVMODES =            HSConfigFile::HOSTSERV_MODES;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_HOSTSERVCHANNEL =          HSConfigFile::HOSTSERV_CHANNEL;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_CLIENTDEFAULTPERMISSIONS = HSConfigFile::HSCLIENT_DEFAULTPERMISSIONS;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_VHOSTPUBLICPOLICY  =       HSConfigFile::VHOST_PUBLICPOLICY;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_HOSTSERVCONFIGFILENAME =   HSConfigFile::HOSTSERV_CONFIGFILENAME;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_VHOSTSCONFIGFILENAME =     HSConfigFile::VHOSTS_CONFIGFILENAME;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_USERSCONFIGFILENAME =      HSConfigFile::USERS_CONFIGFILENAME;
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_RELOAD =                   "Application.Reload";
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_RECONNECT =                "Application.Reconnect";
const HTTPString POSTCGIScriptHandlerKit::DIRECTIVE_SHUTDOWN =                 "Application.Shutdown";

static HSConfigurationCGIScriptHandler SH_SERVERNAME( &HSConfiguration::setServerName );
static HSConfigurationCGIScriptHandler SH_SERVERDESCRIPTION( &HSConfiguration::setServerDescription );
static HSConfigurationCGIScriptHandler SH_REMOTEHOST( &HSConfiguration::setRemoteHost );
static HSConfigurationCGIScriptHandler SH_REMOTEPASSWORD( &HSConfiguration::setRemotePassword );
static HSConfigurationCGIScriptHandler SH_HOSTSERVNICKNAME( &HSConfiguration::setHostservNickname );
static HSConfigurationCGIScriptHandler SH_HOSTSERVUSERNAME( &HSConfiguration::setHostservUsername );
static HSConfigurationCGIScriptHandler SH_HOSTSERVADDRESS( &HSConfiguration::setHostservAddress );
static HSConfigurationCGIScriptHandler SH_HOSTSERVREALNAME( &HSConfiguration::setHostservRealname );
static HSConfigurationCGIScriptHandler SH_HOSTSERVMODES( &HSConfiguration::setHostservModes );
static HSConfigurationCGIScriptHandler SH_HOSTSERVCHANNEL( &HSConfiguration::setHostservChannel );
static HSConfigurationCGIScriptHandler SH_CLIENTDEFAULTPERMISSIONS( &HSConfiguration::setClientDefaultPermissions );
static HSConfigurationCGIScriptHandler SH_VHOSTPUBLICPOLICY( &HSConfiguration::setVHostPublicpolicy );
static HSConfigurationCGIScriptHandler SH_HOSTSERVCONFIGFILENAME( &HSConfiguration::setHostservConfigfilename );
static HSConfigurationCGIScriptHandler SH_VHOSTSCONFIGFILENAME( &HSConfiguration::setVHostsConfigfilename );
static HSConfigurationCGIScriptHandler SH_USERSCONFIGFILENAME( &HSConfiguration::setUsersConfigfilename );


CGIScriptHandlerKit* POSTCGIScriptHandlerKit::m_staticKit = NULL;

POSTCGIScriptHandlerKit::POSTCGIScriptHandlerKit( HostServ* hostserv )
   : m_hostserv( hostserv ),
     m_reloadHandler( hostserv, &HostServ::reload ),
     m_reconnectHandler( hostserv, &HostServ::reconnect ),
     m_shutdownHandler( hostserv, &HostServ::stop )
{
   initStaticKit();
   initKit();
}

void
POSTCGIScriptHandlerKit::initStaticKit()
{
   if( m_staticKit == NULL )
   {
      m_staticKit = new CGIScriptHandlerKit();
      m_staticKit->registerScriptHandler( DIRECTIVE_SERVERNAME,               &SH_SERVERNAME );
      m_staticKit->registerScriptHandler( DIRECTIVE_SERVERDESCRIPTION,        &SH_SERVERDESCRIPTION );
      m_staticKit->registerScriptHandler( DIRECTIVE_REMOTEHOST,               &SH_REMOTEHOST );
      m_staticKit->registerScriptHandler( DIRECTIVE_REMOTEPASSWORD,           &SH_REMOTEPASSWORD );
      m_staticKit->registerScriptHandler( DIRECTIVE_HOSTSERVNICKNAME,         &SH_HOSTSERVNICKNAME );
      m_staticKit->registerScriptHandler( DIRECTIVE_HOSTSERVUSERNAME,          &SH_HOSTSERVUSERNAME );
      m_staticKit->registerScriptHandler( DIRECTIVE_HOSTSERVADDRESS,          &SH_HOSTSERVADDRESS );
      m_staticKit->registerScriptHandler( DIRECTIVE_HOSTSERVREALNAME,         &SH_HOSTSERVREALNAME );
      m_staticKit->registerScriptHandler( DIRECTIVE_HOSTSERVMODES,            &SH_HOSTSERVMODES );
      m_staticKit->registerScriptHandler( DIRECTIVE_HOSTSERVCHANNEL,          &SH_HOSTSERVCHANNEL );
      m_staticKit->registerScriptHandler( DIRECTIVE_CLIENTDEFAULTPERMISSIONS, &SH_CLIENTDEFAULTPERMISSIONS );
      m_staticKit->registerScriptHandler( DIRECTIVE_VHOSTPUBLICPOLICY,        &SH_VHOSTPUBLICPOLICY );
      m_staticKit->registerScriptHandler( DIRECTIVE_HOSTSERVCONFIGFILENAME,   &SH_HOSTSERVCONFIGFILENAME );
      m_staticKit->registerScriptHandler( DIRECTIVE_VHOSTSCONFIGFILENAME,     &SH_VHOSTSCONFIGFILENAME );
      m_staticKit->registerScriptHandler( DIRECTIVE_USERSCONFIGFILENAME,      &SH_USERSCONFIGFILENAME );
   }
}

void
POSTCGIScriptHandlerKit::initKit()
{
   this->registerScriptHandler( DIRECTIVE_RELOAD,    &m_reloadHandler );
   this->registerScriptHandler( DIRECTIVE_RECONNECT, &m_reconnectHandler );
   this->registerScriptHandler( DIRECTIVE_SHUTDOWN,  &m_shutdownHandler );
}

SimpleCGIScript::SimpleCGIScript( const Request* req, CGIScriptHandlerKit& kit )
   : m_request( req ),
     m_kit( kit ),
     m_getRequest( NULL )
{
}

SimpleCGIScript::~SimpleCGIScript()
{
   delete m_getRequest;
}

const Response* 
SimpleCGIScript::makeResponse( const execenv_type& env )
{
   using http::RequestStartline;

   /*
    * for each key in env we will call handler function
    */
   
   // of course we could optimize this code segment heavily...
   // but what we want first is oo ( which hostserv lacks of currently )

   execenv_type::const_iterator iter = env.begin(), end = env.end();

   for( ; iter != end; ++iter )
      m_kit.execute( iter->first, iter->second );

   /*
    * create GETMethod and process requested document as if we had a GET request
    */

   m_getRequest = new GETRequest( this->m_request->getFrom() );

   const RequestStartline& getStartline = m_request->getRequestStartline();

   m_getRequest->setStartline( getStartline );
   m_getRequest->setReferer( m_request->getReferer() );
   m_getRequest->setHost( m_request->getHost() );
   m_getRequest->setUserAgent( m_request->getUserAgent() );
   m_getRequest->setContentType( m_request->getContentType() );
   m_getRequest->setConnectionType( m_request->getConnectionType() );

   m_getRequest->getMethod()->processRequest();
   
   return m_getRequest->getMethod()->getResponse();
}

}//EON httpd
