/***************************************************************************
                           configfile.cpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: configfile.cpp,v 1.6 2002/03/09 12:49:34 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <configfile.hpp>
#include <iostream>
#include <fstream>

const string ConfigFile::STD_DIV="=";

ConfigFile::ConfigFile() :
  ConfigFileParser(STD_DIV)
{
#ifdef VERBOSE_MEMORY
  cout << "ConfigFile] created" << endl;
#endif 
}

ConfigFile::ConfigFile(const string &Delim, char QuoteChar)
  : ConfigFileParser(Delim, &QuoteChar), 
    filename(), quoteChar(QuoteChar)
{
#ifdef VERBOSE_MEMORY
  cout << "ConfigFile] created" << endl;
#endif
}

ConfigFile::~ConfigFile()
{
#ifdef VERBOSE_MEMORY
  cout << "configfile] destroyed" << endl;
#endif    
}

void ConfigFile::process(const string &Filename)
{
  filename=Filename;
  cout << "Opening configfile " << Filename << " ... ";;
  ifstream input(Filename.c_str());
  if( input.good() ) {
    cout << "done" << endl;
    parse(input.rdbuf());
  }
  else cout << "error" << endl;
}

void ConfigFile::serialize( )
{
  ofstream o( filename.c_str(), ios::out | ios::trunc );
  onSerialize( o.rdbuf() );
}

void ConfigFile::onParsedLine(const string &l, const string &r)
{
  string minl, minr;
  minimize(minl, l);
  minimize(minr, r);
  onParsedEntry(minl, minr);
}

bool ConfigFile::isOutQuoted(const string &row) const
{
  string::size_type beg=row.find_first_not_of(' ');

  if( beg == string::npos ) return true;

  return row[beg]==quoteChar; 
}

void ConfigFile::minimize(string &min, const string &max) const {
    char c;
    bool check=true;
    for(string::size_type it=(string::size_type)0; it<max.length(); ++it) {
      c=max[it];
      if(c=='"' && it>0 && max[it-1]!='\\') check=!check;
      else if(!check || c!=' ') min+=c;
#ifdef DEBUG
      else cout << "ConfigFile] Omitting character: '" << c << '\'' << endl;
#endif
    }
}

