/***************************************************************************
                           fileresponsesetting.hpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: fileresponsesetting.cpp,v 1.2 2002/04/30 18:19:48 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fileresponsesetting.hpp"

namespace httpd
{

using namespace http;

struct _FRSPair
{
      const HTTPString          suffix;
      const FileResponseSetting setting;
};

static const _FRSPair _HTTPUNKNOWN_SETTINGS[] =
{
   // suffix                                    Mime-type     Connection-type  Response-Encoding
   //   |                                           |               |                 |
   //   v                                           v               v                 v
   { ".html", FileResponseSetting( HTTPString( "text/html" ), HTTPCONN_CLOSE,  HTTPCONN_CLOSE ) },
   { ".htm",  FileResponseSetting( HTTPString( "text/html" ), HTTPCONN_CLOSE,  HTTPCONN_CLOSE ) },
   { ".hss",  FileResponseSetting( HTTPString( "text/html" ), HTTPCONN_CLOSE,  HTTPCONN_CLOSE ) }, 
   { ".gif",  FileResponseSetting( HTTPString( "image/gif" ), HTTPCONN_CLOSE,  HTTPCONN_CLOSE ) },
   { ".jpg",  FileResponseSetting( HTTPString( "image/jpeg" ), HTTPCONN_CLOSE, HTTPCONN_CLOSE ) },
   { ".png",  FileResponseSetting( HTTPString( "image/png" ), HTTPCONN_CLOSE,  HTTPCONN_CLOSE ) },
   { "",      FileResponseSetting( HTTPString( "text/plain" ), HTTPCONN_CLOSE, HTTPCONN_CLOSE ) }
};

static const _FRSPair _HTTP100_SETTINGS[] = 
{
   { ".htm",  FileResponseSetting( HTTPString( "text/html" ), HTTPCONN_KEEPALIVE,  HTTPCONN_CLOSE ) },
   { ".html", FileResponseSetting( HTTPString( "text/html" ), HTTPCONN_KEEPALIVE,  HTTPCONN_CLOSE ) },
   { ".hss",  FileResponseSetting( HTTPString( "text/html" ), HTTPCONN_CLOSE,      HTTPCONN_CLOSE, true ) },
   { ".gif",  FileResponseSetting( HTTPString( "image/gif" ), HTTPCONN_KEEPALIVE,  HTTPCONN_CLOSE ) },
   { ".jpg",  FileResponseSetting( HTTPString( "image/jpeg" ), HTTPCONN_KEEPALIVE, HTTPCONN_CLOSE ) },
   { ".png",  FileResponseSetting( HTTPString( "image/png" ), HTTPCONN_KEEPALIVE,  HTTPCONN_CLOSE ) },
   { "",      FileResponseSetting( HTTPString( "text/plain" ), HTTPCONN_KEEPALIVE, HTTPCONN_CLOSE ) }
};

static const _FRSPair _HTTP110_SETTINGS[] = 
{
   { ".htm",  FileResponseSetting( HTTPString( "text/html" ), HTTPCONN_KEEPALIVE,  HTTPCONN_CLOSE ) },
   { ".html", FileResponseSetting( HTTPString( "text/html" ), HTTPCONN_KEEPALIVE,  HTTPCONN_CLOSE ) },
   { ".hss",  FileResponseSetting( HTTPString( "text/html" ), HTTPCONN_KEEPALIVE,  HTTPCONN_CHUNKED, true ) },
   { ".gif",  FileResponseSetting( HTTPString( "image/gif" ), HTTPCONN_KEEPALIVE,  HTTPCONN_CLOSE ) },
   { ".jpg",  FileResponseSetting( HTTPString( "image/jpeg" ), HTTPCONN_KEEPALIVE, HTTPCONN_CLOSE ) },
   { ".png",  FileResponseSetting( HTTPString( "image/png" ), HTTPCONN_KEEPALIVE,  HTTPCONN_CLOSE ) },
   { "",      FileResponseSetting( HTTPString( "text/plain" ), HTTPCONN_KEEPALIVE, HTTPCONN_CLOSE ) }
};


const FileResponseSetting& processTab( const _FRSPair* start,
				       const http::HTTPString& requestDocument,
				       bool& defaultUsed )
{
   const _FRSPair* iter = start;

   while( true )
   {
      if( iter->suffix.length() == 0 )
      {
	 defaultUsed = true;
	 return iter->setting;
      }

      if( requestDocument.endsWith( iter->suffix ) )
      {
	 defaultUsed = false;
	 return iter->setting;
      }
      
      ++iter;
   }
}

const FileResponseSetting& FileResponseSetting::getFileResponseSetting( const http::HTTPString& requestDocument,
									http::httpprotversion_type protocolVersion,
									bool& defaultUsed )
{
   if( protocolVersion == HTTPVER_100 )
      return processTab( &_HTTP100_SETTINGS[0], requestDocument, defaultUsed );

   else if( protocolVersion == HTTPVER_110 )
      return processTab( &_HTTP110_SETTINGS[0], requestDocument, defaultUsed );

   else
      return processTab( &_HTTPUNKNOWN_SETTINGS[0], requestDocument, defaultUsed );
}

}//EON httpd
