/***************************************************************************
                           hsclient.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsclient.cpp,v 1.16 2002/03/19 10:58:10 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <hsclient.hpp>
#include <iostream>

HSClient::flag_type HSClient::defaultFlags = flagUpdateOnAuth | flagChangeVHost;


HSClient::HSClient( const string &name,
		    const string &password,
		    VHost *Vhost,
		    flag_type Flags )
   : loginName( name ), 
     loginPassword( password ),
     flags( Flags ), 
     vhost( Vhost )
{
   if(vhost) vhost->addReference( this );

   // if user is admin apply implicit admin flags
   if( isAdmin() ) 
     flags |= IMPLICIT_ADMIN_FLAGS;

#ifdef VERBOSE_MEMORY
   cout << "HSClient created: '" << name << "'->'" << password 
	<< "' flags: " << createFlagsString() << endl;
#endif
  
}

HSClient::HSClient( const IRCToken& name ,
		    const IRCToken& password,
		    VHost *Vhost,
		    flag_type Flags )
  : loginName( name ), 
    loginPassword( password ),
    flags( Flags ), 
    vhost( Vhost )
{
   if(vhost) vhost->addReference( this );
   
   // if user is admin apply implicit admin flags
   if( isAdmin() ) 
     flags |= IMPLICIT_ADMIN_FLAGS;

#ifdef VERBOSE_MEMORY
   cout << "HSClient created: '" << name << "'->'" << password 
	<< "' flags: " << createFlagsString() << endl;
#endif
  
}

HSClient::~HSClient()
{
   if( vhost ) vhost->removeReference( this );
   
#ifdef VERBOSE_MEMORY
   cout << "HSClient] destroyed" << endl;
#endif
}

HSClient::flag_type HSClient::parseFlagsString( const string &s )
{
   flag_type maxIter = 1 << MAX_FLAGS;

   if( s.length() < MAX_FLAGS )
   {
      cout << "HSClient] parseFlagsstring(): warning: flags string too short"
	   << endl;
      maxIter = 1 << s.length();
   }

   string::const_iterator it = s.begin();
   flag_type parsedFlags = 0;

   for( flag_type flagIter = 1 ; flagIter != maxIter; ++it )
   {
      if( *it == '1' ) parsedFlags |= flagIter;
      flagIter <<= 1;
   }

#ifdef DEBUG
   if( s.length() > 3 ) 
   {
      cout << "HSClient] parseFlagsstring(): ignoring "
	   << s.length() - 3
	   << "characters" << endl;
   }
#endif

   return parsedFlags;
}

void HSClient::setVHost( VHost *newVHost ) 
{
   if( vhost )
      vhost->removeReference( this );

   if( newVHost )
      newVHost->addReference( this );

   vhost = newVHost;
}

/*
 * warning: this function needs HSClient::flag_type to be unsigned
 */
string HSClient::createFlagsString() const
{
   flag_type flagsIter = 1;   
   flag_type iterMax   = 1 << MAX_FLAGS;
   string s;
   s.reserve( static_cast< string::size_type > ( MAX_FLAGS ) );

   for( ; flagsIter != iterMax ; flagsIter <<= 1 )
      s += ( ( flags & flagsIter ) > 0 ) ? '1' : '0';

   return s;
}

ostream& operator<<( ostream& out, const HSClient& client )
{
  return out << "username=" << client.getUsername() << " password=" << client.getPassword() << endl;
}
