
/***************************************************************************
                           hsclientconfigfile.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsclientconfigfile.cpp,v 1.10 2002/03/19 10:58:10 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <hsclientconfigfile.hpp>
#include <algorithm>

const string HSClientConfigFile::DEF_DELIM            = ":";
const string HSClientConfigFile::FLAGS_VHOST_DELIM    = "->";
const string HSClientConfigFile::PASSWORD_FLAGS_DELIM = "@";

HSClientConfigFile::HSClientConfigFile( HSClientList *l, 
					const VHostList *v )
  : ConfigFile( DEF_DELIM, '#' ),
    m_clients( l ),
    m_vhosts( v )
{
#ifdef VERBOSE_MEMORY
  cout << "HSClientConfigFile] created" << endl;
#endif
}

HSClientConfigFile::~HSClientConfigFile()
{
#ifdef VERBOSE_MEMORY
  cout << "HSClientConfigFile] destroyed" << endl;
#endif
}

void
HSClientConfigFile::onParsedEntry( const string &username,
				   const string &r )
{
  // r = password@flags[->[vhostID]]
  
  string::size_type passwordEnd = r.find( PASSWORD_FLAGS_DELIM );
  if( passwordEnd != string::npos ) {
    const string password = r.substr( static_cast< string::size_type >( 0 ),
				      passwordEnd );

    passwordEnd += PASSWORD_FLAGS_DELIM.length();
    string::size_type flagsEnd = r.find( FLAGS_VHOST_DELIM, passwordEnd );

    const string flagsString = r.substr( passwordEnd, flagsEnd-passwordEnd );
    int flags = HSClient::parseFlagsString( flagsString );
    const VHost *vhost = NULL;
    
    if( flagsEnd != string::npos && ( flagsEnd+= FLAGS_VHOST_DELIM.length() ) < r.length() ) {
      string vhostId = r.substr( flagsEnd );
      vhost = m_vhosts->getVHost( atoi( vhostId.c_str() ) );
      if( !vhost ) {
	cout << "Warning: vhost for user " << username << " not found: " <<
	  vhostId << endl;
      }
    }

    HSClient* client = new HSClient( username, password,
				     const_cast<VHost*>( vhost ),
				     flags );

    if( ! m_clients->insertClient( client ) )
    {
       cout << "UUUUUUH. Database is corrupt. Duplicate username found: " << username << endl;
       cout << "Omitting this client." << endl;

       delete client;
    }
  }
  else cout << "Ignoring malformed entry for client: " << username << endl;

}

void
HSClientConfigFile::onSerialize( streambuf *buf )
{
   ostream o( buf );
   writeClients( o, m_clients, m_vhosts );
		 
}

void
HSClientConfigFile::writeClients( ostream &o, 
				  HSClientList *newClients,
				  const VHostList* vhosts )
{
   for_each( newClients->begin(), newClients->end(), 
	     Writer( o, vhosts ) );
}

HSClientConfigFile::Writer::Writer( ostream &o,
				    const VHostList* const vhosts  )
  : m_out( o ),
    m_vhosts( vhosts )
{
}

void
HSClientConfigFile::Writer::operator() ( const HSClient* h )
{
   const VHost *v = h->getVHost();
   m_out << h->getUsername() << DEF_DELIM
	 << h->getPassword() << PASSWORD_FLAGS_DELIM
	 << h->createFlagsString();
   if( v )
   {
      VHostList::id_type id = m_vhosts->getVHostID( v );
      m_out << FLAGS_VHOST_DELIM << id;
   }

   m_out << endl;
}

