/***************************************************************************
                           hsclientlist.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsclientlist.cpp,v 1.13 2002/03/19 21:09:27 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <hsclientlist.hpp>
#include <iostream>
#include <algorithm>
#include <config.h>

HSClientList::HSClientList() 
  : set<HSClient*, DereferenceLess<HSClient*> >()
{
#ifdef VERBOSE_MEMORY
  cout << "HSClientList] created" << endl;
#endif
}

HSClientList::~HSClientList()
{
  clear();

#ifdef VERBOSE_MEMORY
  cout << "HSClientList] destroyed" << endl;
#endif  
}

bool
HSClientList::insertClient( HSClient *c )
{
  bool ret = false;
  if( c ) {
    pair< iterator, bool> p = insert( c );
    ret = p.second;
  }
  else cout << "HSClientList] ::insertClient(): arg == NULL" << endl;

  return ret;
}

bool
HSClientList::eraseClient( HSClient *c )
{
  bool ret = false;
  
  const_iterator it = find( c );
  if( it != end() ) {
    erase( it );
    ret = true;
  }

  return ret;
}

inline
void
my_delete( const HSClient* t )
{ 
  delete t;
}

void
HSClientList::clear()
{
  if( empty() ) return;

  for_each( begin(), end(), &my_delete );
  set<HSClient*, DereferenceLess<HSClient*> >::clear();
}

HSClient*
HSClientList::getClient( const IRCToken &nickname ) const
{
  HSClient *ret = NULL;
  const_iterator it = find_if( begin(), end(), HSClientListPredicate( nickname ) );
  if( it != end() ) ret = *it;
  return ret;
}

HSClientListPredicate::HSClientListPredicate( const IRCToken &nickname )
  : refName( nickname )
{
}

bool
HSClientListPredicate::operator() ( const HSClient *c )
{
  return (*c) == refName;
}
