#include <algorithm>
#include "hscommands.hpp"
#include "hostserv.hpp"
#include "irc.hpp" // for match()
#include "config.h"

namespace hostserv{

namespace commands{

using std::cout;
using std::endl;

/*
 * toplevel command names
 */
const IRCToken HSAdminCommandSet::SUPER_CMD           = "admin";
const IRCToken HSNetCommandSet::SUPER_CMD             = "net";
const IRCToken HSMeCommandSet::SUPER_CMD              = "me";
const IRCToken HSClientCommandSet::SUPER_CMD          = "client";
const IRCToken HSVHostCommandSet::SUPER_CMD           = "vhost";
const IRCToken HSAuthedCommands::CMD_DEAUTH           = "deauth";
const IRCToken HSUnauthedCommands::CMD_AUTH           = "auth";

const IRCToken HSAdminCommandSet::CMD_SHUTDOWN        = "shutdown";
const IRCToken HSAdminCommandSet::CMD_RECONNECT       = "reconnect";
const IRCToken HSAdminCommandSet::CMD_RELOAD          = "reload";

const IRCToken HSNetCommandSet::CMD_SETHOST           = "sethost";
const IRCToken HSNetCommandSet::CMD_SETIDENT          = "setident";

const IRCToken HSMeCommandSet::CMD_UPDATE             = "update";
const IRCToken HSMeCommandSet::CMD_SETVHOST           = "setvhost";
const IRCToken HSMeCommandSet::CMD_LISTPUBLIC         = "listpublic";
const IRCToken HSMeCommandSet::CMD_SETPASSWORD        = "setpassword";
const IRCToken HSMeCommandSet::CMD_DETAILS            = "details";
const IRCToken HSMeCommandSet::CMD_RESTORE            = "restore";
const IRCToken HSMeCommandSet::CMD_SETUPDATEONAUTH    = "setupdateonauth";
const IRCToken HSMeCommandSet::CMD_SETAFFECTUSERNAME  = "setaffectusername";

const IRCToken HSClientCommandSet::CMD_ADD            = "add";
const IRCToken HSClientCommandSet::CMD_DEL            = "del";
const IRCToken HSClientCommandSet::CMD_SETPASSWORD    = "setpassword";
const IRCToken HSClientCommandSet::CMD_DISABLE        = "disable";
const IRCToken HSClientCommandSet::CMD_UPDATEONAUTH   = "updateonauth";
const IRCToken HSClientCommandSet::CMD_UPDATEUSERNAME = "updateusername";
const IRCToken HSClientCommandSet::CMD_ADMIN          = "admin";
const IRCToken HSClientCommandSet::CMD_VHOSTADMIN     = "vhostadmin";
const IRCToken HSClientCommandSet::CMD_NETWORKADMIN   = "networkadmin";
const IRCToken HSClientCommandSet::CMD_USERADMIN      = "useradmin";
const IRCToken HSClientCommandSet::CMD_SETVHOST       = "setvhost";
const IRCToken HSClientCommandSet::CMD_LIST           = "list";
const IRCToken HSClientCommandSet::CMD_DETAILS        = "details";

const IRCToken HSVHostCommandSet::CMD_ADD             = "add";
const IRCToken HSVHostCommandSet::CMD_LIST            = "list";
const IRCToken HSVHostCommandSet::CMD_PUBLIC          = "public";
const IRCToken HSVHostCommandSet::CMD_REFERENCES      = "references";
const IRCToken HSVHostCommandSet::CMD_DEL             = "del";

void
HSAdminCommandSet::initSet()
{
   registerCommand( CMD_RELOAD ,   HSAdminReloadCommand::getCommand() );
   registerCommand( CMD_SHUTDOWN,  HSAdminShutdownCommand::getCommand() );
   registerCommand( CMD_RECONNECT, HSAdminReconnectCommand::getCommand() );
}

void
HSMeCommandSet::initSet()
{
   registerCommand( CMD_SETVHOST,          HSMeSetVHostCommand::getCommand() );
   registerCommand( CMD_UPDATE,            HSMeUpdateCommand::getCommand() );
   registerCommand( CMD_LISTPUBLIC,        HSMeListpublicCommand::getCommand() );
   registerCommand( CMD_SETPASSWORD,       HSMeSetPasswordCommand::getCommand() );
   registerCommand( CMD_DETAILS,           HSMeDetailsCommand::getCommand() );
   registerCommand( CMD_RESTORE,           HSMeRestoreCommand::getCommand() );
   registerCommand( CMD_SETUPDATEONAUTH,   HSMeSetUpdateonauthCommand::getCommand() );
   registerCommand( CMD_SETAFFECTUSERNAME, HSMeSetAffectusernameCommand::getCommand() );
}

void
HSNetCommandSet::initSet()
{
   registerCommand( CMD_SETHOST,  HSNetSethostCommand::getCommand() );
   registerCommand( CMD_SETIDENT, HSNetSetidentCommand::getCommand() );
}

void
HSClientCommandSet::initSet()
{
   registerCommand( CMD_ADD,            HSClientAddCommand::getCommand() );
   registerCommand( CMD_DEL,            HSClientDelCommand::getCommand() );
   registerCommand( CMD_DISABLE,        HSClientDisableCommand::getCommand() );
   registerCommand( CMD_UPDATEONAUTH,   HSClientUpdateonauthCommand::getCommand() );
   registerCommand( CMD_UPDATEUSERNAME, HSClientUpdateusernameCommand::getCommand() );
   registerCommand( CMD_SETPASSWORD,    HSClientSetPasswordCommand::getCommand() );
   registerCommand( CMD_ADMIN,          HSClientAdminCommand::getCommand() );
   registerCommand( CMD_VHOSTADMIN,     HSClientVHostadminCommand::getCommand() );
   registerCommand( CMD_USERADMIN,      HSClientUseradminCommand::getCommand() );
   registerCommand( CMD_NETWORKADMIN,   HSClientNetworkadminCommand::getCommand() );
   registerCommand( CMD_SETVHOST,       HSClientSetvhostCommand::getCommand() );
   registerCommand( CMD_LIST,           HSClientListCommand::getCommand() );
   registerCommand( CMD_DETAILS,        HSClientDetailsCommand::getCommand() );
}

void
HSVHostCommandSet::initSet()
{
   registerCommand( CMD_ADD,            HSVHostAddCommand::getCommand() );
   registerCommand( CMD_LIST,           HSVHostListCommand::getCommand() );
   registerCommand( CMD_PUBLIC,         HSVHostPublicCommand::getCommand() );
   registerCommand( CMD_REFERENCES,     HSVHostReferencesCommand::getCommand() );
   registerCommand( CMD_DEL,            HSVHostDelCommand::getCommand() );
}

void
HSAuthedCommands::initSet()
{
   registerCommand( HSMeCommandSet::getName(),     HSMeCommandSet::getSet() );
   registerCommand( HSAdminCommandSet::getName(),  HSAdminCommandSet::getSet() );
   registerCommand( HSNetCommandSet::getName(),    HSNetCommandSet::getSet() );
   registerCommand( HSClientCommandSet::getName(), HSClientCommandSet::getSet() );
   registerCommand( HSVHostCommandSet::getName(),  HSVHostCommandSet::getSet() );

   registerCommand( CMD_DEAUTH,                    HSDeauthCommand::getCommand() );
}

void
HSUnauthedCommands::initSet()
{
   registerCommand( CMD_AUTH, HSAuthCommand::getCommand() );
}



HSAdminCommandSet*
HSAdminCommandSet::getSet()
{
   static HSAdminCommandSet* set = new HSAdminCommandSet();
   
   return set;
}

HSNetCommandSet*
HSNetCommandSet::getSet()
{
   static HSNetCommandSet* commandSet = new HSNetCommandSet();

   return commandSet; 
}

HSMeCommandSet*
HSMeCommandSet::getSet()
{
   static HSMeCommandSet* commandSet = new HSMeCommandSet();

   return commandSet;
}

HSClientCommandSet*
HSClientCommandSet::getSet()
{
   static HSClientCommandSet* set = new HSClientCommandSet();
   
   return set;
}

HSVHostCommandSet*
HSVHostCommandSet::getSet()
{
   static HSVHostCommandSet* set = new HSVHostCommandSet();
   
   return set;
}

void
CommandSetHSCommandImpl::printClientNotFound( HSIRCServerConnection& conn,
					      const HSIRCClient& target,
					      const IRCToken& clientName )
{
   const static char* MSG1 = "Client not found: %s";

   conn.printNotice( conn.getHostservNick(), target.getNickname(), MSG1, &clientName );
}

void
CommandSetHSCommandImpl::printSuccessfullOperation( HSIRCServerConnection& conn,
						    const HSIRCClient& target )
{
   const static IRCToken MSG1( "Success." );

   conn.printNotice( conn.getHostservNick(), target.getNickname(), MSG1 );
}

void
CommandSetHSCommandImpl::printSuccessfullOperation( HSIRCServerConnection& conn,
						    const HSIRCClient& target,
						    const IRCToken& msg )
{
   const static char* MSG1 = "Success: %s";
   
   conn.printNotice( conn.getHostservNick(), target.getNickname(), MSG1, &msg );
}

void
CommandSetHSCommandImpl::printVHostNotFound( HSIRCServerConnection& conn,
					     const HSIRCClient& client,
					     const IRCToken& vhostName )
{
   const static char* MSG1 = "VHost not found: %U%s%U";

   conn.printNotice( conn.getHostservNick(), client.getNickname(), MSG1, &vhostName );
}

void
CommandSetHSCommandImpl::printHSIRCClientDetails( HSIRCServerConnection& conn,
						  const HSIRCClient& sender,
						  const HSIRCClient* client,
						  bool printRealHostname )
{
   const IRCToken& target = sender.getNickname();
   const HSClient* c = client->getHSClient();

   assert( c );

   conn.printNotice( conn.getHostservNick(), target,  "%s logedin as: %U%B%s%B%U",
		     &client->getNickname(), &c->getUsername() );
   if( printRealHostname )
      conn.printNotice( conn.getHostservNick(), target, "real hostname: %U%B%s%B%U",
			&client->getAddress() );
}

void
CommandSetHSCommandImpl::printHSClientDetails( HSIRCServerConnection& conn,
					       const HSIRCClient& sender,
					       const HSClient* c )
{
   const IRCToken& target       = sender.getNickname();
   ostream&        out          = conn.tcp();
   const IRCToken& hostservNick = conn.getHostservNick();
   
   out << ':' << hostservNick
       << ' ' << IRCServerConnection::CMD_NOTICE << ' '
       << target << " :"
       << bold << "vhost" << bold
       << ": ";
   
   if( c->getVHost() ) 
      out << *c->getVHost() << endl;
   else
      out << "none assigned" << endl;
   
   conn.printNotice( hostservNick, target, "is an admin:          %b", c->isAdmin() );
   conn.printNotice( hostservNick, target, "is online:            %b", c->isOnline() );
   conn.printNotice( hostservNick, target, "is disabled:          %b", c->isDisabled() );
   conn.printNotice( hostservNick, target, "update on auth up:    %b", c->updateOnAuth() );
   conn.printNotice( hostservNick, target, "username updateable:  %b", c->updateUsername() );
   conn.printNotice( hostservNick, target, "may change his vhost: %b", c->mayChangeVHost() );
   conn.printNotice( hostservNick, target, "is a user db admin:   %b", c->isUserDBAdmin() );
   conn.printNotice( hostservNick, target, "is a vhost db admin:  %b", c->isVHostDBAdmin() );
   conn.printNotice( hostservNick, target, "is a network admin:   %b", c->isNetworkAdmin() );
}

HSClient*
CommandSetHSCommandImpl::lookupHSClient( HSIRCServerConnection& conn,
					 const IRCToken& username )
{
   HSClient *client = NULL;
   
   if( username[0] == '#' )
   {
      const IRCToken::size_type begin = static_cast<IRCToken::size_type>(1);
      const HSIRCClient* hsC = 
	 conn.getHSIRCClients()->getClient( username.substr( begin ) );
      if( hsC ) 
	 client = const_cast<HSClient*>( hsC->getHSClient() );
   }
   else
      client = conn.getHSClients()->getClient( username );

   return client;
}


AbstractHSCommandState*
CommandStateMap::resume( AbstractHSCommandState * state )
			 
{
   assert( state == this );


   list_type::iterator       iter = m_list.begin();
   const list_type::iterator end  = m_list.end();

   for( ; iter != end; ++iter )
   {
      AbstractHSCommand* const      command = iter->m_command;
      AbstractHSCommandState* const result  = command->resume( iter->m_state );
      
      if( result == NULL )
	 // execution finished -> remove Command from queue
	 m_list.erase( iter );
      else
	 // command not finished yet. update state therefore
	 iter->m_state = result;
   }

   return m_list.empty() ? NULL : state;
}

AbstractHSCommandState*
CommandSet::execute( HSIRCServerConnection& conn,
		     HSIRCClient& client,
		     const IRCToken& command, 
		     const IRCToken* args,
		     int argNum )
{
#ifdef DEBUG
   cout << "CommandSet::execute: command='" << command << "' args: ";
   for( int i = 0; i < argNum; ++i )
      cout << args[i] << ' ';

   cout << endl;
#endif

   CommandStateMap*                map  = NULL;
   commandset_type::iterator       iter = m_commands.begin();
   const commandset_type::iterator end  = m_commands.end();
   int commandsExecuted = 0;
   
   for( ; iter != end; ++iter )
   {
      if( iter->first != command )
	 continue;

      AbstractHSCommand*      const hsCommand = iter->second;
      AbstractHSCommandState* const state     = hsCommand->execute( conn, client, command, args, argNum );

      
       if( state != NULL )
       {
 	 /*
 	  * command has not finished with execution yet
 	  */
 	 if( map == NULL )
 	    map = CommandStateMap::makeMap();

 	 map->addCommand( hsCommand, state );
       }

      ++commandsExecuted;
   }

   if( commandsExecuted == 0 )
   {
#ifdef DEBUG
      cout << "CommandSet::execute(): no such command found: " << command << endl;
#endif      
      onCommandNotFound( conn,
			 client,
			 command,
			 args,
			 argNum );
   }
   return map;
}

AbstractHSCommandState*
CommandSet::resume( AbstractHSCommandState* state )
{
//    CommandStateMap* map = dynamic_cast<CommandStateMap*>( state );
   CommandStateMap* map = (CommandStateMap*) state;
   
   assert( map );

   AbstractHSCommandState* const mapState = map->resume( state );

   if( mapState == NULL )
      delete map;

   return mapState;
}

void
CommandSet::unregisterCommand( const IRCToken& name )
{
   const commandset_type::iterator end  = m_commands.end();
   const commandset_type::iterator newEnd = 
      std::remove_if( m_commands.begin(),
		      end,
		      CSetPredicate( name ) );

   m_commands.erase( newEnd, end );
}

int
CommandSet::unregisterCommand( const AbstractCommandSetHSCommand* command )
{
   commandset_type::iterator       iter = m_commands.begin();
   const commandset_type::iterator end  = m_commands.end();

   int numRemoved = 0;

   for( ; iter != end; ++iter )
   {
      if( iter->second == command )
      {
	 m_commands.erase( iter );
	 ++numRemoved;
      }
   }

   return numRemoved;
}

AbstractHSCommandState*
HSAdminCommandSet::onHelp( HSIRCServerConnection& conn,
			   HSIRCClient& client,
			   const IRCToken& command, 
			   const IRCToken* args, 
			   int argNum )
{
   static const char* MSGS[] = {
      "%UUsage%U: %Badmin%B <command> [<arguments>]",
      " ",
      "%UAvailable subcommands%U:",
      "%Bshutdown%B   - makes hostserv shutdown",
      "%Breconnect%B  - makes hostserv reconnect current ircserver",
      "%Breload%B     - makes hostserv reload configurationfile",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );


   return NULL;
}

AbstractHSCommandState* onHelp( HSIRCServerConnection& conn,
				HSIRCClient& client,
				const IRCToken& command,
				const IRCToken* args,
				int argNum )
{
   static const char* MSGS[] = {
      "%UUsage%U: %Badmin%B <command> [<arguments>]",
      " ",
      "%UAvailable subcommands%U:",
      "%Bshutdown%B   - makes hostserv shutdown",
      "%Breconnect%B  - makes hostserv reconnect current ircserver",
      "%Breload%B     - makes hostserv reload configurationfile",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS ); 

   return NULL;
}

// AbstractHSCommandState*
// CommandSetComposition::execute( HSIRCServerConnection& conn,
// 				  HSIRCClient& client,
// 				  const IRCToken& command, 
// 				  const IRCToken* args, 
// 				  int argNum )
// {
//    commandsetvector_type::iterator       iter = m_commandSets.begin();
//    const commandsetvector_type::iterator end  = m_commandSets.end();
   
//    CommandStateMap* map = NULL;

//    for( ; iter != end; ++iter )
//    {
//       CommandSet* const           set   = *iter;
//       AbstractHSCommandState* const state = set->execute( conn, 
// 							  client, 
// 							  command, 
// 							  args, 
// 							  argNum );

//       if( state != NULL )
//       {
// 	 if( map == NULL )
// 	    map = CommandStateMap::makeMap();

// 	 map->addCommand( set, state );
//       }
//    }

//    return map;
// }

AbstractHSCommandState*
HSAdminReloadCommand::onExecute( HSIRCServerConnection& conn, 
				 HSIRCClient& client,
				 const IRCToken& command,
				 const IRCToken* args,
				 int argNum )
{
   const IRCToken& option = *args;

   if( option != "yes" )
      return onHelp( conn, client, command, args, argNum );
   


   return NULL;
}

AbstractHSCommandState*
HSAdminReloadCommand::onHelp( HSIRCServerConnection& conn,
			      HSIRCClient& client,
			      const IRCToken& command,
			      const IRCToken* args,
			      int argNum )
{
   static const char* MSGS[] = {
      "Used to make hostserv reload its configurationfile.",
      "HostServ will disconnect the current irc server, serialize user and",
      "vhost lists, reload hostserv configurationfile and startup.",
      "$USyntax%U: %Breload%B yes",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**) MSGS );



   return NULL;
}

AbstractHSCommandState*
HSAdminReconnectCommand::onExecute( HSIRCServerConnection& conn,
				 HSIRCClient& client,
				 const IRCToken& command,
				 const IRCToken* args,
				 int argNum )
{
   const IRCToken& option = *args;

   if( option != "yes" )
      return onHelp( conn, client, command, args, argNum );

   const IRCToken& initiator = client.getNickname();

   
   const static unsigned char* msg = (const unsigned char*) "Reconnect requested by: %s";
   conn.printQuit( conn.getHostservNick(), msg,
		   &initiator );
   conn.printSquit( conn.getServername(), msg,
		    &initiator );
   
   conn.flushOutput();
				       
   conn.getMother()->reconnect();


   return NULL;
}

AbstractHSCommandState*
HSAdminReconnectCommand::onHelp( HSIRCServerConnection& conn,
				 HSIRCClient& client,
				 const IRCToken& command,
				 const IRCToken* args,
				 int argNum )
{
   static const char* MSGS[] = {
      "Used to amek hostserv reconnect the current irc server.",
      "VHost and user databases will get stored on disk.",
      "Hostserv configurationfile will not get reloaded from disk.",
      "%USyntax%U: %Breconnect%B yes",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**) MSGS );

   return NULL;
}

AbstractHSCommandState*
HSAdminShutdownCommand::onExecute( HSIRCServerConnection& conn,
				   HSIRCClient& client,
				   const IRCToken& command,
				   const IRCToken* args,
				   int argNum )
{
   const IRCToken& option = *args;

   if( option != "yes" )
      return onHelp( conn, client, command, args, argNum );

   const IRCToken& initiator = client.getNickname();

   const static unsigned char *msg = (const unsigned char*)"Shutdown requested by: %s";
   conn.printQuit( conn.getHostservNick(), msg,
		   &initiator );
   conn.printSquit( conn.getServername(), msg,
		    &initiator );

   conn.flushOutput();


   return NULL;
}

AbstractHSCommandState*
HSAdminShutdownCommand::onHelp( HSIRCServerConnection& conn,
				HSIRCClient& client,
				const IRCToken& command,
				const IRCToken* args,
				int argNum )
{
   static const char* MSGS[] = {
      "Used to make hostserv disconnect from irc server and shutdown.",
      "VHost and user lists will get stored on disk.",
      "%USyntax%U: %Bshutdown%B yes",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );
   
   return NULL;
}


AbstractHSCommandState* 
HSNetCommandSet::onHelp( HSIRCServerConnection& conn,
			 HSIRCClient& client,
			 const IRCToken& command, 
			 const IRCToken* args, 
			 int argNum )
{
   static const char* MSGS[] = {
      "%UUsage%U: %Bnet%B <command> [<arguments>]",
      " ",
      "%UAvailable subcommands%U:",
      "%Bsethost%B  <nick> <new hostname> - frontend for command 'CHGHOST'",
      "%Bsetident%B <nick> <new username> - frontend for command 'CHGIDENT'",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );


   return NULL;
}

AbstractHSCommandState*
HSNetSethostCommand::onExecute( HSIRCServerConnection& conn,
				HSIRCClient& client,
				const IRCToken& command, 
				const IRCToken* args, 
				int argNum )
{
   const IRCToken& victim      = args[0];
   const IRCToken& newHostname = args[1];
   
   if( victim.equalsIgnoreCase( conn.getHostservNick() ) )
      return NULL;

   HSIRCClient* victimClient = conn.getHSIRCClients()->getClient( victim );

   if( !victimClient )
   { 
      printClientNotFound( conn,
			   client,
			   victim );
      return NULL;
   }

   conn.changeHost( victimClient->getNickname(), newHostname );


   return NULL;
}

AbstractHSCommandState*
HSNetSethostCommand::onHelp( HSIRCServerConnection& conn,
			     HSIRCClient& client,
			     const IRCToken& command, 
			     const IRCToken* args, 
			     int argNum )
{
   static const char* MSGS[] = {
      "Used to manipulate the hostname of a certain irc client.",
      "Does never affect any VHost setting. This command is just",
      "a 'frontend' to the server-to-server only command 'CHGHOST'.",
      "%USyntax%U: %Bsethost%B <username> <new hostname>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}


AbstractHSCommandState*
HSNetSetidentCommand::onExecute( HSIRCServerConnection& conn,
				 HSIRCClient& client,
				 const IRCToken& command, 
				 const IRCToken* args, 
				 int argNum )
{
   const IRCToken& victim   = args[0];
   const IRCToken& newIdent = args[1];

   if( victim.equalsIgnoreCase( conn.getHostservNick() ) )
      return NULL;

   HSIRCClient* victimClient = conn.getHSIRCClients()->getClient( victim );

   if( !victimClient )
   { 
      printClientNotFound( conn,
			   client,
			   victim );
      return NULL;
   }

   conn.changeUsername( victimClient->getNickname(), newIdent );


   return NULL;
}

AbstractHSCommandState*
HSNetSetidentCommand::onHelp( HSIRCServerConnection& conn,
			      HSIRCClient& client,
			      const IRCToken& command, 
			      const IRCToken* args, 
			      int argNum )
{
   static const char* MSGS[] = {
      "Used to manipulate the hostname of a certain irc client.",
      "Does never affect any VHost setting. This command is just",
      "a 'frontend' to the server-to-server only command 'CHGHOST'.",
      "%USyntax%U: %Bsethost%B <username> <new hostname>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );


   return NULL;
}

AbstractHSCommandState*
HSClientPermissionChangingCommand::onExecute( HSIRCServerConnection& conn,
					      HSIRCClient& client,
					      const IRCToken& command, 
					      const IRCToken* args, 
					      int argNum )
{
   const bool      val = *args == "1" ? true : false;
   HSClient* const cl  = const_cast<HSClient*>( client.getHSClient() );

   updatePermission( cl, val );

   printSuccessfullOperation( conn, client );

   return NULL;
}

AbstractHSCommandState*
HSClientLookingUpPermissionChangingCommand::onExecute( HSIRCServerConnection& conn,
						       HSIRCClient& client,
						       const IRCToken& command, 
						       const IRCToken* args, 
						       int argNum )
{
   const bool      val      = *(args+1) == "1" ? true : false;
   const IRCToken& username = *args;
   HSClient* const cl       = CommandSetHSCommandImpl::lookupHSClient( conn, username );

   if( !cl )
   {
      printClientNotFound( conn, client, username );
      return NULL;
   }

   updatePermission( cl, val );

   printSuccessfullOperation( conn, client );

   return NULL;
}

AbstractHSCommandState* 
HSMeCommandSet::onHelp( HSIRCServerConnection& conn,
			HSIRCClient& sender,
			const IRCToken& command, 
			const IRCToken* args, 
			int argNum )
{
   static const char* ME_COMMON[] = {
      "%UUsage%U: %Bme%B <command> [<arguments>]",
      " ",
      "%Bupdate%B",
      " - applies your assigned vhost",
      " ",
      "%Bsetpassword%B",
      " - updates your current password",
      " ",
      "%Bsetupdateonauth%B <1|0>",
      " - sets your updateonauth flag",
      " ",
      "%Bsetaffectusername%B <1|0>",
      " - sets your affectusername flag",
      " ",
      "%Bdetails%B",
      " - prints your settings",
      " ",
      "%Brestore%B",
      " - restores your old hostname and username",
      " ",
      NULL
   };
   static const char* ME_MAYCHANGEVHOST[] = {
      "%Blistpublic%B [<criteria>]",
      " - list public vhosts.",
      "   If criteria is given only vhosts are printed which match",
      "   the given criteria. Criteria may contain wildcards such",
      "   as '?' or '*'.",
      " ",
      "%Bsetvhost%B <public vhost>",
      "                        - assign a public vhost to you.",
      NULL
   };

   conn.printHelpArray( sender.getNickname(), (const unsigned char**)ME_COMMON );

   if( sender.getHSClient()->mayChangeVHost() )
      conn.printHelpArray( sender.getNickname(), (const unsigned char**)ME_MAYCHANGEVHOST );

   return NULL;
}


AbstractHSCommandState* 
HSMeSetVHostCommand::onExecute( HSIRCServerConnection& conn,
				HSIRCClient& client,
				const IRCToken& command, 
				const IRCToken* args, 
				int argNum )
{
   const IRCToken& vhostName = args[0];
   const IRCToken& target    = client.getNickname();

   VHost* const vhost = const_cast<VHost*>( conn.getVHosts()->getVHost( vhostName ) );

   if( vhost == NULL )
   {
      printVHostNotFound( conn, client, vhostName );

      return NULL;
   }

   if( !vhost->isPublic() )
   {
      static const char* MSG = "VHost is not public: %B%s%B";
      conn.printNotice( conn.getHostservNick(), target, MSG, &vhostName );

      return NULL;
   }

   HSClient* hsClient = const_cast<HSClient*>( client.getHSClient() );
   hsClient->setVHost( vhost );

   if( hsClient->updateOnAuth() )
      conn.applyVHost( client, vhost, true );
   else 
   {
      printSuccessfullOperation( conn, client );

      conn.printNotice( conn.getHostservNick(), target,
			"Your host- and username are untouched" );
   }

   return NULL;
}


AbstractHSCommandState* 
HSMeSetVHostCommand::onHelp( HSIRCServerConnection& conn,
			     HSIRCClient& target,
			     const IRCToken& command, 
			     const IRCToken* args, 
			     int argNum )
{
   static const char* MSGS[] = {
      "Used to set your vhost dynamically.",
      "You can choose a vhost from a list of public vhosts.",
      "%USyntax%U: %Bsetvhost%B <vhostname>",
      NULL
   };
   conn.printHelpArray( target.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}


AbstractHSCommandState* 
HSMeUpdateCommand::onExecute( HSIRCServerConnection& conn,
			       HSIRCClient& client,
			       const IRCToken& command, 
			       const IRCToken* args, 
			       int argNum )
{
   const HSClient* const hsC = client.getHSClient();
   const VHost*    const v   = hsC->getVHost();

   if( v )
   {
      // only update hostname if not already updated by HostServ
      // -> reduce traffic
      if( !hsC->updateOnAuth() && !client.vhostApplied() )
      {
	 conn.applyVHost( client, v, true );
	 client.setVHostApplied( true );
      }
   }
   else
   {
      static const IRCToken MSG( "No vhost has been assigned to you." );

      conn.printNotice( conn.getHostservNick(), client.getNickname(), MSG );
   }
   
   return NULL;
}


AbstractHSCommandState* 
HSMeUpdateCommand::onHelp( HSIRCServerConnection& conn,
			   HSIRCClient& client,
			   const IRCToken& command, 
			   const IRCToken* args, 
			   int argNum )
{
   static const char *MSGS[] = {
      "Used to update your hostname (and username).",
      "%USyntax:%U %Bupdate%B",
      "Notice: this command will only affect your hostname ",
      "if your hostname has not been changed by HostServ already.",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}


AbstractHSCommandState* 
HSMeListpublicCommand::onExecute( HSIRCServerConnection& conn,
				  HSIRCClient& client,
				  const IRCToken& command, 
				  const IRCToken* args, 
				  int argNum )
{

   if( argNum == 0 )
      return resume( new CommandState( conn, client, IRCToken( "*" ), conn.getVHosts()->begin() ) );
   else
      return resume( new CommandState( conn, client, *args, conn.getVHosts()->begin() ) );
}

AbstractHSCommandState*
HSMeListpublicCommand::resume( AbstractHSCommandState* s )
{
   CommandState* const state = static_cast<CommandState*>( s );
   
   // we will increment state->m_numCalled below
//    if( state->m_numPrinted >= CommandState::MAX_NUM_CALLED )
//    {
//       delete state;
//       return NULL;
//    }
   

   HSIRCServerConnection&          conn      = state->m_conn;
   HSIRCClient&                    client    = state->m_client;
   const VHostList::const_iterator end       = conn.getVHosts()->end();
   VHostList::const_iterator&      iter      = state->m_pos;
   const IRCToken&                 criteria  = state->m_criteria; // vhostname criteria
   const IRCToken&                 target    = state->m_client.getNickname(); // to whom we will print


   // print message command without longarg
   conn.tcp() << ':' << conn.getHostservNick() << ' ' << IRCServerConnection::CMD_NOTICE
	      << ' ' << target << " :";

   /*
    * we will print junks of 10 vhosts and then return with a AbstractHSCommandState instance
    */
   int numPrinted = 0;
   for( ; numPrinted < CommandState::MAX_VHOSTS_PER_CALL && iter != end; ++iter )
   {
      if( onPrintVHost( conn, client, iter->first, criteria ) )
	 ++numPrinted;
   }

   if( iter != end )
   {
      conn.tcp() << endl;
      delete state;

      return NULL;
   }


   state->m_numPrinted += numPrinted;
   if( state->m_numPrinted == 0 )
      conn.tcp() << "no matching vhost found for criteria: " << criteria << endl;

   
   conn.tcp() << endl;

   delete state;
   return NULL;
}

bool
HSMeListpublicCommand::onPrintVHost( HSIRCServerConnection& conn,
				     HSIRCClient& client,
				     const VHost* vhost,
				     const IRCToken& criteria )
{
   if( vhost->isPublic() && match( criteria, vhost->getAddress() ) )
   {
      conn.tcp() << *vhost << ' ';
      return true;
   }

   return false;
}
				  

AbstractHSCommandState* 
HSMeListpublicCommand::onHelp( HSIRCServerConnection& conn,
			       HSIRCClient& client,
			       const IRCToken& command, 
			       const IRCToken* args, 
			       int argNum )
{
   static const char* MSGS[] = {
      "%Used to print public vhosts.",
      "%USyntax%U: %me%B listpublic [criteria]",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**) MSGS );
   
   return NULL;
}

IRCToken::size_type
HSMeSetPasswordCommand::changePassword( HSClient* hsClient,
					const IRCToken& passwordParam )
{
   IRCToken newPassword( passwordParam );
   const IRCToken::size_type s = toValidHSClientString( newPassword );

   if( s > 0 )
      return s;
   
   hsClient->setPassword( newPassword );

   return s;
}

AbstractHSCommandState* 
HSMeSetPasswordCommand::onExecute( HSIRCServerConnection& conn,
				   HSIRCClient& sender,
				   const IRCToken& command,
				   const IRCToken* args, 
				   int argNum )
{
   const IRCToken& currentPassword = *args;
   const IRCToken& realPassword    = sender.getHSClient()->getPassword();

   if( currentPassword != realPassword )
   {
      conn.printNotice( conn.getHostservNick(), sender.getNickname(),
			"Your current password does not match '%s'",
			&realPassword );
      return NULL;
   }

   const IRCToken&           newPassword = args[1];
   const IRCToken::size_type size        = changePassword( const_cast<HSClient*>( sender.getHSClient() ), 
							   newPassword );
   if( size > 0 )
   {   
      conn.printNotice( conn.getHostservNick(), sender.getNickname(),
			"Your password contains %i illegal character(s)." );

      return NULL;
   }

   conn.printNotice( conn.getHostservNick(), sender.getNickname(),
		     "Your new password is: %B%s%B",
		     &newPassword );
   
   return NULL;
}


AbstractHSCommandState* 
HSMeSetPasswordCommand::onHelp( HSIRCServerConnection& conn,
				HSIRCClient& client,
				const IRCToken& command,
				const IRCToken* args,
				int argNum )
{
   const static char* MSGS[] = {
      "Update your current password.",
      "%USyntax:%U %me setpassword%B <current password> <new password>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState* 
HSMeDetailsCommand::onExecute( HSIRCServerConnection& conn,
			       HSIRCClient& sender,
			       const IRCToken& command, 
			       const IRCToken* args, 
			       int argNum )
{
   printHSIRCClientDetails( conn, sender, &sender, true );
   printHSClientDetails( conn, sender, sender.getHSClient() );

   return NULL;
}

AbstractHSCommandState* 
HSMeRestoreCommand::onExecute( HSIRCServerConnection& conn,
			       HSIRCClient& sender,
			       const IRCToken& command, 
			       const IRCToken* args, 
			       int argNum )
{
   // quick and dirty
   if( sender.vhostApplied() )
   {
      VHost v( sender.getUsername(), sender.getAddress() );
      conn.applyVHost( sender, &v, false );
      // ok this is dirty .. but a must be
      const_cast<HSIRCClient*>( &sender )->setVHostApplied( false );

      printSuccessfullOperation( conn, sender );
   }
   else
   {
      conn.printNotice( conn.getHostservNick(), sender.getNickname(),
		   (const unsigned char*)"Hostname and username are up-to-date",
		   &conn.getHostservNick() );
   }

   return NULL;
}

AbstractHSCommandState*
HSClientCommandSet::onHelp( HSIRCServerConnection& conn,
			    HSIRCClient& client,
			    const IRCToken& command,
			    const IRCToken* args,
			    int argNum )
{
   static const char* MSGS[] = {
      "%UUsage%U: %Bclient%B <command> [<arguments>]",
      " ", 
      "%UAvailable subcommands are%U:",
      "%Badd%B <username> <password> <flags>",
      " - stores another client into HostServ's user db",
      "   <flags> is a IRCToken consisting of '0' and '1' which set the flags of the client.",
      "     1. character: admin flag",
      "     2. character: is disabled (is not permitted to use hostserv)",
      "     3. character: hostname (and username) should be updated on auth up",
      "     4. character: hostserv may update username",
      "     5. character: may change his vhost",
      "     6. character: is vhost database admin",
      "     7. character: is user database admin",
      "     8. character: user is a network admin",
      " ",
      "%Bdel%B <username>",
      " - deletes client <username> from HostServ's user db",
      " ",
      "%Bdisable%B <username> <1|0>",
      " - sets the disabled flag for user <username>",
      " ",
      "%Bupdateonauth%B <username> <1|0>",
      " - sets the update on auth flag for user <username>.",
      "   This flag defines the host update behaviour when a client",
      "   has authenticated successfull at hostserv.",
      " ",
      "%Bupdateusername%B <username> <1|0>",
      " - sets the update username flag for user <username>.",
      "   With this flag set HostServ is permitted to change the username",
      "   according to vhost's defaults.",
      " ",
      "%Bvhostadmin%B <username> <1|0>",
      " - sets the vhost db admin flag.",
      "   With this flag set clients may commit changes to the",
      "   vhost database.",
      " - Only admins are permitted to set this flag.",
      " ",
      "%Buserdbadmin%B <username> <1|0>",
      " - sets the user db admin flag.",
      "   With this flag set clients may commit changes to the",
      "   user database.",
      " - Only admins are permitted to set this flag.",
      " ",
      "%Bnetworkadmin%B <username> <1|0>",
      " - sets the network admin flag.",
      "   With this flag set clients may use commands 'sethost' and 'setident'",
      " - Only admins are permitted to set this flag.",
      " ",
      "%Badmin%B <username> <1|0>",
      " - sets the admin flag for user <username>.",
      "   With this flag set other flags are turned on such as",
      "   'networkadmin', 'useradmin' and 'vhostadmin'.",
      " - Only admins are permitted to set this flag.",
      " ",
      "%Bsetvhost%B <username> <vhostname>",
      " - sets vhost <vhostname> for client <username>",
      " ",
      "%Bsetpassword%B <username> <new password>",
      "  - updates password for <username>",
      " ",
      "%Blist%B [<criteria>]",
      " - list available loginnames",
      " ",
      "%Bdetails%B <username>",
      " - list some details for user <username>",
      "   Netadmins are permitted to see the real hostname of the client", 
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );
   

   return NULL;
}

AbstractHSCommandState* 
HSMeSetUpdateonauthCommand::onHelp( HSIRCServerConnection& conn,
				    HSIRCClient& client,
				    const IRCToken& command, 
				    const IRCToken* args, 
				    int argNum )
{
   static const char* MSGS[] = {
      "Sets your updateusername flag.",
      "This flag defines hostserv's username update behaviour.",
      "Updateusername set will make hostserv change your username too",
      "if a username is associated with your vhost",
      "%USyntax%U: %Bme setaffectusername%B <1|0>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState* 
HSMeSetAffectusernameCommand::onHelp( HSIRCServerConnection& conn,
				      HSIRCClient& client,
				      const IRCToken& command, 
				      const IRCToken* args, 
				      int argNum )
{
  static const char* MSGS[] = {
      "Sets your updateonauth flag.",
      "This flag defines hostserv's behaviour when you have authenticated at HostServ.",
      "Updateonauth set means HostServ will update your hostname / username according to",
      "your VHost whenever you auth up",
      "%USyntax%U: %Bme setupdateonauth%B <1|0>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState*
HSClientDetailsCommand::onExecute( HSIRCServerConnection& conn,
				HSIRCClient& sender,
				const IRCToken& command, 
				const IRCToken* args, 
				int argNum )
{
   const IRCToken&       username = *args;
   const HSClient* const client   = lookupHSClient( conn, username );


   if( !client )
   {
      printClientNotFound( conn, sender, username );
      return NULL;
   }
   
   
   printHSClientDetails( conn, sender, client );

   return NULL;
}

AbstractHSCommandState*
HSClientDetailsCommand::onHelp( HSIRCServerConnection& conn,
				HSIRCClient& client,
				const IRCToken& command, 
				const IRCToken* args, 
				int argNum )
{
   static const char* MSGS[] = {
      "Used to print details for a HostServ user",
      "%USyntax%U: %Bclient details%B <username>",
      "If <username> starts with '#' in the list of persistent clients will be searched",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );
   

   return NULL;
}

AbstractHSCommandState*
HSClientListCommand::resume( AbstractHSCommandState* s )
{
   cout << "HSClientListCommand::resume: called" << endl;

   CommandState* const state = static_cast<CommandState*>( s );
   
   HSIRCServerConnection&             conn      = state->m_conn;
   HSIRCClient&                       client    = state->m_client;
   const HSClientList::const_iterator end       = conn.getHSClients()->end();
   HSClientList::const_iterator&      iter      = state->m_pos;
   const IRCToken&                    criteria  = state->m_criteria;
   const IRCToken&                    target    = state->m_client.getNickname();

   int totalClientsPrinted = 0;

   for( int i = 0; i < CommandState::MAX_ROWS_PER_CALL; ++i )
   {
      // print message command without longarg
      conn.tcp() << ':' << conn.getHostservNick() << ' ' << IRCServerConnection::CMD_NOTICE
		    << ' ' << target << " :";

      /*
       * we will print junks of 10 clients and then return with a AbstractHSCommandState instance
       */
      int clientsPrinted = 0;
      for( ; clientsPrinted < CommandState::MAX_CLIENTS_PER_ROW && iter != end; ++iter )
      {
	 HSClient* const hsclient = *iter;
	 
	 if( onPrintClient( conn, client, hsclient, criteria ) )
	    ++clientsPrinted;
      }
      totalClientsPrinted += clientsPrinted;
      

      if( iter == end )
      {
	 if( totalClientsPrinted == 0 )
	    conn.tcp() << "no matching client found for: " << criteria;
	 
	 conn.tcp() << endl;

	 delete state;
	 return NULL;
      }
      else if( ++state->m_numCalled >= CommandState::MAX_NUM_CALLED )
      {
	 conn.tcp() << endl;

	 delete state;
	 return NULL;
      }

      conn.tcp() << endl;
   }

   return state;
}

bool
HSClientListCommand::onPrintClient( HSIRCServerConnection& conn,
				    HSIRCClient& client,
				    const HSClient* hsclient,
				    const IRCToken& criteria )
{
   if( ::match( criteria, hsclient->getUsername() ) )
   {
      conn.tcp() << hsclient->getUsername() << ' ';
      return true;
   }
   
   return false;
}

AbstractHSCommandState*
HSClientSetvhostCommand::onExecute( HSIRCServerConnection& conn,
				    HSIRCClient& sender,
				    const IRCToken& command, 
				    const IRCToken* args, 
				    int argNum )
{
   const IRCToken& username = *args;
   const IRCToken& newVHost = *(args+1);

   HSClient* const client = lookupHSClient( conn, *args );

   if( !client )
   {
      printClientNotFound( conn, sender, username );
      return NULL;
   }

   const VHost* const vhost = conn.getVHosts()->getVHost( newVHost );

   if( !vhost )
   {
      printVHostNotFound( conn, sender, newVHost );
      return NULL;
   }
   
   client->setVHost( const_cast<VHost*>( vhost ) );

   
   printSuccessfullOperation( conn, sender );

   return NULL;
}

AbstractHSCommandState*
HSClientSetvhostCommand::onHelp( HSIRCServerConnection& conn,
				 HSIRCClient& client,
				 const IRCToken& command, 
				 const IRCToken* args, 
				 int argNum )
{
   const static char* MSGS[] = {
      "Used to set the 'setownvhost' flag for a certain vhost",
      "With this flag set an unpriviledged user can assign",
      "a public vhost to him / her."
      "%USyntax:%U %Bvhost public%B <vhost> <1|0>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS);
   
   return NULL;
}

AbstractHSCommandState* 
HSClientNetworkadminCommand::onHelp( HSIRCServerConnection& conn,
				     HSIRCClient& client,
				     const IRCToken& command, 
				     const IRCToken* args, 
				     int argNum )
{
    const static char* MSGS[] = {
      "Used to set the networkadmin flag for a certain user",
      "With this flag set a user may use commands '%Bsethost%B'",
      " and '%Bsetident%B'.",
      "%USyntax:%U %Bclient networkadmin%B <username> <1|0>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );
  
   return NULL;
}

AbstractHSCommandState* 
HSClientUseradminCommand::onHelp( HSIRCServerConnection& conn,
				  HSIRCClient& client,
				  const IRCToken& command, 
				  const IRCToken* args, 
				  int argNum )
{
   const static char* MSGS[] = {
      "Used to set the clientadmin flag for a certain user",
      "With this flag set a user may change the user database.",
      "%USyntax:%U %Bclient useradmin%B <username> <1|0>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );


   return NULL;
}

AbstractHSCommandState* 
HSClientVHostadminCommand::onHelp( HSIRCServerConnection& conn,
				   HSIRCClient& client,
				   const IRCToken& command, 
				   const IRCToken* args, 
				   int argNum )
{
   const static char* MSGS[] = {
      "Used to set the vhostadmin flag for a certain user",
      "With this flag set a user may change the vhost database.",
      "%USyntax:%U %Bclient vhostadmin%B <username> <1|0>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );


   return NULL;
}

AbstractHSCommandState* 
HSClientAdminCommand::onHelp( HSIRCServerConnection& conn,
			      HSIRCClient& client,
			      const IRCToken& command, 
			      const IRCToken* args, 
			      int argNum )
{
   const static char* MSGS[] = {
      "Used to set the updateusername flag for a certain user",
      "This flag sets the behaviour when a vhost is applied to",
      " a user. If this flag is set the client's username will",
      " be overwritten when a vhost gets applied on the client.",
      "%USyntax:%U %Bclient updateusername%B <username> <1|0>",
      "%UNotice:%U This flag does not affect flag 'updateonauth'",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );


   return NULL;
}

AbstractHSCommandState* 
HSClientUpdateonauthCommand::onHelp( HSIRCServerConnection& conn,
				     HSIRCClient& client,
				     const IRCToken& command, 
				     const IRCToken* args, 
				     int argNum )
{
   const static char* MSGS[] = {
      "Used to set the updateonauth flag for a certain user",
      "%USyntax:%U %Bclient updateonauth%B <username> <1|0>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );


   return NULL;
}

AbstractHSCommandState* 
HSClientUpdateusernameCommand::onHelp( HSIRCServerConnection& conn,
				     HSIRCClient& client,
				     const IRCToken& command, 
				     const IRCToken* args, 
				     int argNum )
{
    const static char* MSGS[] = {
      "Used to set the updateusername flag for a certain user",
      "This flag sets the behaviour when a vhost is applied to",
      " a user. If this flag is set the client's username will",
      " be overwritten when a vhost gets applied on the client.",
      "%USyntax:%U %Bclient updateusername%B <username> <1|0>",
      "%UNotice:%U This flag does not affect flag 'updateonauth'",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState* 
HSClientDisableCommand::onHelp( HSIRCServerConnection& conn,
				HSIRCClient& client,
				const IRCToken& command, 
				const IRCToken* args, 
				int argNum )
{
   static const char* MSGS[] = {
      "Used to disable HostServ clients dynamically.",
      "%USyntax%U: %Bclient disable%B <username> <1|0>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );


   return NULL;
}

AbstractHSCommandState* 
HSClientDelCommand::onExecute( HSIRCServerConnection& conn,
			       HSIRCClient& sender,
			       const IRCToken& command, 
			       const IRCToken* args, 
			       int argNum )
{
   const IRCToken&       target   = sender.getNickname();
   const IRCToken&       username = *args;
   const HSClient* const c        = conn.getHSClients()->getClient( username );

   if( !c )
   {
      printClientNotFound( conn, sender, username );
      return NULL; 
   }
   
   if( c->isAdmin() && !sender.getHSClient()->isAdmin() )
   {
      static const IRCToken MSG1( "You do not have to appropriate privileges." );
      conn.printNotice( conn.getHostservNick(), target, MSG1 );

      return NULL;
   }

   HSIRCClient* const hsC = conn.getHSIRCClients()->getClient( c );

   if( hsC && hsC->isAuthed() )
      HSDeauthCommand::logOut( conn, sender );


   conn.getHSClients()->eraseClient( const_cast<HSClient*>( c ) );
   delete c;

   printSuccessfullOperation( conn, sender );

   return NULL;   
}

AbstractHSCommandState* 
HSClientDelCommand::onHelp( HSIRCServerConnection& conn,
			    HSIRCClient& client,
			    const IRCToken& command, 
			    const IRCToken* args, 
			    int argNum )
{
   static const char* MSGS[] = {
      "Used to delete HostServ clients dynamically.",
      "%USyntax%U: %Bclient del%B <username>",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState* 
HSClientAddCommand::onExecute( HSIRCServerConnection& conn,
			       HSIRCClient& client,
			       const IRCToken& command, 
			       const IRCToken* args, 
			       int argNum )
{
   const IRCToken& target        = client.getNickname();
   IRCToken        username      = *args;
   IRCToken        Password      = args[1];

   // check username
   IRCToken::size_type invalidCharacters = toValidHSClientString( username );
   if( invalidCharacters == username.length() )
   {
      conn.printNotice( conn.getHostservNick(), target, 
			"Username consists of invalid characters only." );
      return NULL;
   }
   else if( invalidCharacters > 0 )
   {
      conn.printNotice( conn.getHostservNick(), target, 
			"Username has %i invalid characters. Using '%s' instead.",
			invalidCharacters, &username );
   }

   // check password
   invalidCharacters = toValidHSClientString( Password );
   if( invalidCharacters == Password.length() )
   {
      conn.printNotice( conn.getHostservNick(), target, 
			"Password consists of invalid characters only." );
      return NULL;
   }
   else if( invalidCharacters > 0 )
   {
      conn.printNotice( conn.getHostservNick(), target, 
			"Password has %i invalid characters. Using '%s' instead.",
			invalidCharacters, &Password );
   }
  
   HSClient *cl = NULL;
   if( argNum > 2 ) 
   {
      const IRCToken& flagsIRCToken = args[2];

      cl = new HSClient( username, 
			 Password, 
			 NULL,
			 HSClient::parseFlagsString( string( ( (const char*)flagsIRCToken.c_str() ),
							     args[2].length() ) ) );
   }
   else
      cl = new HSClient( username, Password, NULL );
   
   if( !conn.getHSClients()->insertClient( cl ) )
   {
      delete cl;
      
      static const char* MSG1 = "Client exists: %B%s%B";

      conn.printNotice( conn.getHostservNick(), target, MSG1, &username );
   }
   else
      printSuccessfullOperation( conn, client );

   return NULL;
}

AbstractHSCommandState* 
HSClientAddCommand::onHelp( HSIRCServerConnection& conn,
			    HSIRCClient& sender,
			    const IRCToken& command, 
			    const IRCToken* args, 
			    int argNum )
{
   static const char* MSGS[] = {
      "Used to add clients dynamically into HostServ's database.",
      "%USyntax%U: %Bclient add%B <username> <password> [<flags IRCToken>]",
      "If no flags IRCToken is given the default usermodes will be used.",
      NULL
   };
   conn.printHelpArray( sender.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState*
HSClientSetPasswordCommand::onExecute( HSIRCServerConnection& conn,
				       HSIRCClient& sender,
				       const IRCToken& command,
				       const IRCToken* args,
				       int argNum )
{
   HSClient* const hsClient = lookupHSClient( conn, *args );

   if( !hsClient )
   {
      printClientNotFound( conn, sender, *args );
      return NULL;
   }

   if( hsClient->isAdmin() && !sender.getHSClient()->isAdmin() )
   {
      static const IRCToken MSG1( "You do not have the appropriate permissions to delete an hostserv administrator");
      conn.printNotice( conn.getHostservNick(), sender.getNickname(), MSG1 );
      return NULL;
   }

   const IRCToken&           passwordParam = args[1];
   IRCToken                  newPassword( passwordParam );
   const IRCToken::size_type s             = toValidHSClientString( newPassword );

   if( s > 0 )
   {
      const static char* MSG1 = "Your password contains at least %i illegal character(s). A valid password is e.g.: %U%s%U";
      conn.printNotice( conn.getHostservNick(), sender.getNickname(), MSG1, &newPassword );
      return NULL;
   }

   hsClient->setPassword( args[1] );

   printSuccessfullOperation( conn, sender );

   return NULL;
}

AbstractHSCommandState*
HSClientSetPasswordCommand::onHelp( HSIRCServerConnection& conn,
				    HSIRCClient& sender,
				    const IRCToken& command,
				    const IRCToken* args,
				    int argNum )
{
   static const char* MSGS[] = {
      "Used to set the password of a certain hostserv client.",
      "%USyntax%U: %Bclient setpassword%B <username> <password>",
      "If the username starts with a `#' hostserv will lookup",
      "<username> in the list of irc clients",
      NULL
   };
   conn.printHelpArray( sender.getNickname(), (const unsigned char**)MSGS );
   
   return NULL;
}

AbstractHSCommandState* 
HSAuthCommand::onExecute( HSIRCServerConnection& conn,
			  HSIRCClient& client,
			  const IRCToken& command, 
			  const IRCToken* args, 
			  int argNum )
{
   const IRCToken& username = *args;
   const IRCToken& Password = *(args+1);

   assert( !client.isAuthed() );

//    if( client.isAuthed() )
//    {
//       static const IRCToken MSG1( "You are already authed at hostserv. Deauth first." );
//       conn.printNotice( conn.getHostservNick(), client.getNickname(),
// 			MSG1 );

//       return NULL;
//    }

   HSClient *c = conn.getHSClients()->getClient( username );

   if( c == NULL || c->getPassword() != Password )
   {
      static const IRCToken MSG1( "Login failed. Password / username do not match." );
      conn.printNotice( conn.getHostservNick(), client.getNickname(),
			MSG1 );

      return NULL;
   }
   
   if( c->isDisabled() )
   {
      static const IRCToken MSG1( "Your account has been disabled" );

      conn.printNotice( conn.getHostservNick(), client.getNickname(),
			MSG1 );

      return NULL;
   }

   if( c->isOnline() )
   {
      static const IRCToken MSG1( "Your account is already in use" );
      
      conn.printNotice( conn.getHostservNick(), client.getNickname(),
			MSG1 );

      return NULL;
   }

   client.login( c );

   const VHost *v = c->getVHost();
   if( c->updateOnAuth() && v != NULL )
      conn.applyVHost( client, v, true );

   c->setOnline( true );


   static const IRCToken MSG1( "Login successfull" );
   conn.printNotice( conn.getHostservNick(), client.getNickname(),
		     MSG1 );

   
   return NULL;
}

AbstractHSCommandState* 
HSAuthCommand::onHelp( HSIRCServerConnection& conn,
		       HSIRCClient& sender,
		       const IRCToken& command, 
		       const IRCToken* args, 
		       int argNum )
{
    static const char* MSGS [] = { 
      "Authenticate at HostServ.",
      " ",
      "%USyntax%U: %Bauth%B <username> <password>",
      NULL
   };
   conn.printHelpArray( sender.getNickname(), (const unsigned char**)MSGS );
  
   return NULL;
}

AbstractHSCommandState* 
HSVHostAddCommand::onExecute( HSIRCServerConnection& conn,
			      HSIRCClient& sender,
			      const IRCToken& command, 
			      const IRCToken* args, 
			      int argNum )
{
   const VHost*    v      = NULL;
   const IRCToken& target = sender.getNickname();
   
   // -> we want backward compatibilty
   if( ( *args == "-" && argNum > 2 ) || argNum == 1 ) {
      v= new VHost( *args );
   }
   else {
      v = new VHost( *args, args[1] );
   }

   if( conn.getVHosts()->insertVHost( v ) ) {
      printSuccessfullOperation( conn,
				 sender );
   }
   else {
      static const IRCToken MSG1( "VHost exists." );
      conn.printNotice( conn.getHostservNick(), target, MSG1 );
      delete v;
   }

   return NULL;
}

AbstractHSCommandState* 
HSVHostAddCommand::onHelp( HSIRCServerConnection& conn,
			   HSIRCClient& sender,
			   const IRCToken& command, 
			   const IRCToken* args, 
			   int argNum )
{
   static const char* MSGS [] = {
      "Used to add vhosts dynamically into HostServ's intern vhost database.",
      "%USyntax%U: %Bvhost add%B <username> <hostname>",
      " - Inserts a vhost with a associated usernmae / ident into the database",
      "%USyntax%U: %Bvhost add%B <username>",
      " - Inserts an address-only vhost into the database",
      NULL
   };
   conn.printHelpArray( sender.getNickname(), (const unsigned char**)MSGS );
   
   return NULL;
}

AbstractHSCommandState* 
HSVHostListCommand::onExecute( HSIRCServerConnection& conn,
			       HSIRCClient& sender,
			       const IRCToken& command, 
			       const IRCToken* args, 
			       int argNum )
{
   const IRCToken& hostservNick = conn.getHostservNick();

   if( conn.getVHosts()->empty() )
   {
      conn.printNotice( hostservNick, sender.getNickname(), "no vhost stored" );
      return NULL;
   }

   conn.printNotice( hostservNick, sender.getNickname(), 
		     "public vhosts are marked with a trailing '%B*%B'" );
   
   return HSMeListpublicCommand::onExecute( conn, sender, command, args, argNum );
}

AbstractHSCommandState* 
HSVHostListCommand::onHelp( HSIRCServerConnection& conn,
			    HSIRCClient& sender,
			    const IRCToken& command, 
			    const IRCToken* args, 
			    int argNum )
{
   return NULL;
}

bool
HSVHostListCommand::onPrintVHost( HSIRCServerConnection& conn,
				  HSIRCClient& client,
				  const VHost* vhost,
				  const IRCToken& criteria )
{
   ostream& out = conn.tcp();

   if( ::match( criteria, vhost->getAddress() ) )
   {
      out << *vhost;
      if( vhost->isPublic() )
	 out << bold << '*' << bold;
      out << ' ';
      return true;
   }
   
   return false;
}

AbstractHSCommandState* 
HSVHostPublicCommand::onExecute( HSIRCServerConnection& conn,
				 HSIRCClient& sender,
				 const IRCToken& command, 
				 const IRCToken* args, 
				 int argNum )
{
   VHost* const    vhost  = 
      const_cast<VHost*>( conn.getVHosts()->getVHost( *args ) );

   if( !vhost )
   {
      printVHostNotFound( conn, sender, *args );
      return NULL;
   }

   vhost->setPublic( args[1] == "1" ? true : false );

   printSuccessfullOperation( conn, sender );

   return NULL;
}

AbstractHSCommandState* 
HSVHostPublicCommand::onHelp( HSIRCServerConnection& conn,
			      HSIRCClient& sender,
			      const IRCToken& command, 
			      const IRCToken* args, 
			      int argNum )
{
   static const char* MSGS[] = {
      "Used to set the public flag for a certain vhost.",
      "With this flag set any client can assign this vhost to themself",
      "",
      "%USyntax%U: %Bvhost public%B <vhostname> <1|0>",
      NULL
   };
   conn.printHelpArray( sender.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState* 
HSVHostReferencesCommand::onExecute( HSIRCServerConnection& conn,
				     HSIRCClient& sender,
				     const IRCToken& command, 
				     const IRCToken* args, 
				     int argNum )
{
   const VHost*    vhost  = conn.getVHosts()->getVHost( *args );
   const IRCToken& target = sender.getNickname();

   if( !vhost )
   {
      printVHostNotFound( conn, sender, *args );
      return NULL;
   }

   const VHost::ClientSet * const s = vhost->getReferences();
  
   conn.tcp() << ':' << conn.getHostservNick() 
	      << ' ' << IRCServerConnection::CMD_NOTICE << ' ' 
	      << target << " :" ;

   if( !s->empty() ) 
   {
      for( VHost::ClientSet::const_iterator it = s->begin(); ; ++it ) 
      {
	 conn.tcp() << (*it)->getUsername();
	 if( ++it != s->end() ) conn.tcp() << ", ";
	 else break;
      }
   }
   else
      conn.tcp() << "no vhost available";

  
   conn.tcp() << endl;

   return NULL;
}

AbstractHSCommandState* 
HSVHostReferencesCommand::onHelp( HSIRCServerConnection& conn,
				  HSIRCClient& sender,
				  const IRCToken& command, 
				  const IRCToken* args, 
				  int argNum )
{
   static const char* MSGS [] = {
      "Prints the login names of the clients which reference to a given vhost.",
      "%USyntax%U: %Bvhost references%B <vhost>",
      NULL
   };
   conn.printHelpArray( sender.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState* 
HSVHostDelCommand::onExecute( HSIRCServerConnection& conn,
			      HSIRCClient& sender,
			      const IRCToken& command, 
			      const IRCToken* args, 
			      int argNum )
{
   const VHost*    v      = conn.getVHosts()->getVHost( *args );
   const IRCToken& target = sender.getNickname();

   if( v ) 
   {
      if( v->referenceCnt() == 0 )
      {
	 conn.getVHosts()->eraseVHost( v );
	 printSuccessfullOperation( conn, sender );
      }
      else 
      {
	 static const IRCToken MSG1( "Too many references left" );
	 conn.printNotice( conn.getHostservNick(), target, MSG1 );
      }
   }
   else
   {
      static const IRCToken MSG1( "VHost not found." );
      conn.printNotice( conn.getHostservNick(), target, MSG1 );
   }


   return NULL;
}

AbstractHSCommandState* 
HSVHostDelCommand::onHelp( HSIRCServerConnection& conn,
			   HSIRCClient& sender,
			   const IRCToken& command, 
			   const IRCToken* args, 
			   int argNum )
{
   static const char* MSGS [] = {
      "Used to delete vhosts dynamically from HostServ's intern vhost database.",
      "%USyntax%U: %Bvhost del%B <vhost>",
      NULL
   };
   conn.printHelpArray( sender.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState*
HSVHostCommandSet::onHelp( HSIRCServerConnection& conn,
			   HSIRCClient& client,
			   const IRCToken& command,
			   const IRCToken* args,
			   int argNum )
{
   static const char* MSGS[] = {
      "%UUsage%U: %Bvhost%B <command> [<argument(s)>]",
      " ",
      "valid commands are:",
      "%Badd%B <ident> <address> / %Badd%B <address>",
      " - adds a new vhost into HostServ's intern db",
      " ",
      "%Bdel%B <vhost>",
      " - remove <vhost> from HostServ's intern db",
      " ",
      "%Bpublic%B <vhost> <1|0>",
      " - sets the public flag for a certain vhost",
      "   With this flag set clients without the vhostadmin flag may",
      "   assign this vhost to themself.",
      " ",
      "%Breferences%B <vhost>",
      " - print the name of clients which reference to vhost <vhost>",
      " ",
      "%Blist%B [<criteria>]",
      " - print every available vhost",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)MSGS );

   return NULL;
}

AbstractHSCommandState*
HSAuthedCommands::onHelp( HSIRCServerConnection& conn,
			  HSIRCClient& client,
			  const IRCToken& command,
			  const IRCToken* args,
			  int argNum )
{
   static const char* USERADMIN_MSGS[] = {
      "%Bclient%B         - manage HostServ's clients.",
      NULL
   };
   static const char* VHOSTADMIN_MSGS[] = {
      "%Bvhost%B          - manage HostServ's vhosts.",
      NULL
   };
   static const char* NETADMIN_MSGS[] = {
      "%Bnet%B            - change irc network data.",
      NULL
   };
   static const char* ADMIN_MSGS[] = {
      "%Badmin%B          - administrate HostServ.",
      NULL
   };
   static const char* LOGEDIN_COMMON_MSGS[] = {
      "%Bme%B             - administrate your account.",
      "%Bdeauth%B         - deauthenticate from hostserv.",
      NULL
   };
   static const char* COMMON_MSGS[] = {
      "%Bhelp%B [command] - print this help.",
      NULL
   };
   static const char* COMMON_HEADER[] = {
      "%UAvailable top-level commands%U:",
      " ",
      NULL
   };

   const IRCToken& target = client.getNickname();   

   conn.printHelpArray( target, (const unsigned char**)COMMON_HEADER );
   
   const HSClient* hsC = client.getHSClient();
   
   if( hsC->isUserDBAdmin() )
      conn.printHelpArray( target, (const unsigned char**)USERADMIN_MSGS );
   if( hsC->isVHostDBAdmin() )
      conn.printHelpArray( target, (const unsigned char**)VHOSTADMIN_MSGS );
   if( hsC->isNetworkAdmin() )
      conn.printHelpArray( target, (const unsigned char**)NETADMIN_MSGS );
   if( hsC->isAdmin() )
      conn.printHelpArray( target, (const unsigned char**)ADMIN_MSGS );

   conn.printHelpArray( target, (const unsigned char**)LOGEDIN_COMMON_MSGS );
   conn.printHelpArray( target, (const unsigned char**)COMMON_MSGS );
   
   return NULL;
}

AbstractHSCommandState* 
HSUnauthedCommands::onHelp( HSIRCServerConnection& conn,
			    HSIRCClient& client,
			    const IRCToken& command, 
			    const IRCToken* args, 
			    int argNum )
{
   static const char* UNAUTHED_MSGS[] = {
      "%Bauth%B           - authenticate yourself at hostserv",
      NULL
   };
   conn.printHelpArray( client.getNickname(), (const unsigned char**)UNAUTHED_MSGS );

   return NULL;
}

}//EON hostserv::commands

}//EON hostserv
