/***************************************************************************
                           hsconfiguration.hpp
                           --------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsconfiguration.cpp,v 1.20 2002/04/26 21:25:22 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "hostserv.hpp"
#include "vhostconfigfile.hpp"
#include "hsclientconfigfile.hpp"
extern "C" {
#include <sys/time.h>
}
#include "hsconfiguration.hpp"


const HSConfiguration::ConfigHandler HSConfiguration::HANDLERS[] =
{
   // 0 - servername
   { &HostServ::SA_SERVNAME,          &HostServ::LA_SERVNAME,          
     &HSConfigFile::SERVER_NAME,      &HSConfiguration::assignHostname },
   
   // 1 - server description
   { &HostServ::SA_SERVERDESCR,       &HostServ::LA_SERVERDESCR,
     &HSConfigFile::SERVER_DESCRIPTION,          &HSConfiguration::assignIgnoreEmpty },

   // 2 - remote host
   { &HostServ::SA_REMOTEHOST,        &HostServ::LA_REMOTEHOST,
     &HSConfigFile::REMOTE_SERVERNAME,           &HSConfiguration::assignIgnoreEmpty },

   // 3 - remote password
   { &HostServ::SA_REMOTEPASSWORD,    &HostServ::LA_REMOTEPASSWORD,
     &HSConfigFile::REMOTE_PASSWORD,             &HSConfiguration::assignIgnoreEmpty },

   // 4 - hostserv nickname
   { &HostServ::SA_HSNICKNAME,        &HostServ::LA_HSNICKNAME,
     &HSConfigFile::HOSTSERV_NICK,               &HSConfiguration::assignNickname },

   // 5 - hostserv username
   { &HostServ::SA_HSUSERNAME,        &HostServ::LA_HSUSERNAME,
     &HSConfigFile::HOSTSERV_USERNAME,           &HSConfiguration::assignUsername },

   // 6 - hostserv address
   { &HostServ::SA_HSADDRESS,         &HostServ::LA_HSADDRESS,
     &HSConfigFile::HOSTSERV_ADDRESS,            &HSConfiguration::assignHostname },

   // 7 - hostserv realname
   { &HostServ::SA_HSREALNAME,        &HostServ::LA_HSREALNAME,
     &HSConfigFile::HOSTSERV_REALNAME,           &HSConfiguration::assignIgnoreEmpty },

   // 8 - hostserv umodes
   { &HostServ::SA_HSMODES,           &HostServ::LA_HSMODES,
     &HSConfigFile::HOSTSERV_MODES,              &HSConfiguration::assignIgnoreEmpty },

   // 9 - hostserv channel
   { &HostServ::SA_HSCHANNEL,         &HostServ::LA_HSCHANNEL,
     &HSConfigFile::HOSTSERV_CHANNEL,            &HSConfiguration::assignChannel },

   // 10 - implicit default client permissions
   { &HostServ::SA_CLIENTDEFAULTPERM, &HostServ::LA_CLIENTDEFAULTPERM,
     &HSConfigFile::HSCLIENT_DEFAULTPERMISSIONS, &HSConfiguration::setDefaultClientPermission },

   // 11 - implicit public policy for vhosts
   { &HostServ::SA_VHOSTPUBLICPOLICY, &HostServ::LA_VHOSTPUBLICPOLICY,
     &HSConfigFile::VHOST_PUBLICPOLICY,          &HSConfiguration::setDefaultVHostPublicPolicy },

   // 12 - httpd default index document
   { &HostServ::SA_HTTPDDEFDOCUMENT,  &HostServ::LA_HTTPDDEFDOCUMENT,
     &HSConfigFile::HTTPD_DEFAULTDOCUMENT,       &HSConfiguration::assignIgnoreEmpty },

   // 13 - listening interface for httpd 
   { NULL,                            NULL,
     &HSConfigFile::HTTPD_IF,                    &HSConfiguration::assignIgnoreEmpty },

   // 14 - filename of vhost database
   { &HostServ::SA_VHOSTSFILENAME,    &HostServ::LA_VHOSTSFILENAME,
     &HSConfigFile::VHOSTS_CONFIGFILENAME,       &HSConfiguration::assignIgnoreEmpty },

   // 15 - filename of hostserv user database
   { &HostServ::SA_USERSFILENAME,     &HostServ::LA_USERSFILENAME,
     &HSConfigFile::USERS_CONFIGFILENAME,        &HSConfiguration::assignIgnoreEmpty },

   // 16 - filename of configfile
   { &HostServ::SA_HSCONFIGFILENAME,  &HostServ::LA_HSCONFIGFILENAME,
     &HSConfigFile::HOSTSERV_CONFIGFILENAME,     &HSConfiguration::assignIgnoreEmpty },

   // 17 - login name for explicit admin
   { &HostServ::SA_ADMINLOGIN,        &HostServ::LA_ADMINLOGIN,
     &HSConfigFile::ADMIN_LOGIN,                 &HSConfiguration::assignAdminLogin },

   // 18 - loing password for explicit admin
   { &HostServ::SA_ADMINPASSWORD,     &HostServ::LA_ADMINPASSWORD,
     &HSConfigFile::ADMIN_PASSWORD,              &HSConfiguration::assignAdminLogin },

   // 19 - vhost for explicit admin
   { &HostServ::SA_ADMINVHOST,        &HostServ::LA_ADMINVHOST,
     &HSConfigFile::ADMIN_VHOST,                 &HSConfiguration::assignIgnoreEmpty },

   // 20 - httpd's document root
   { &HostServ::SA_HTTPDDOCUMENTROOT, &HostServ::LA_HTTPDDOCUMENTROOT, 
     &HSConfigFile::HTTPD_DOCUMENTROOT,          &HSConfiguration::assignHTTPdDocumentRoot },

   /*
    * put options after this line which do _NOT_ depend on m_configuration
    */

   // 21
   { &HostServ::SA_REMOTEPORT,        &HostServ::LA_REMOTEPORT,
     &HSConfigFile::REMOTE_PORT,                 &HSConfiguration::assignPort },

   // 22
   { &HostServ::SA_FOREGROUND,        &HostServ::LA_FOREGROUND,
     NULL,                                       &HSConfiguration::assignForeground },

   // 23
   { &HostServ::SA_HTTPDPORT,         &HostServ::LA_HTTPDPORT,
     &HSConfigFile::HTTPD_PORT,                  &HSConfiguration::assignHTTPdPort },

   // 24
   { &HostServ::SA_HTTPDMAXCLIENTS,   &HostServ::LA_HTTPDMAXCLIENTS,
     &HSConfigFile::HTTPD_DOCUMENTROOT,          &HSConfiguration::assignHTTPdMaxClients },

   // 25
   { &HostServ::SA_HELP,              &HostServ::LA_HELP,
     NULL,                                       &HSConfiguration::assignHelp },

   { NULL,                            NULL,
     &HSConfigFile::HTTPD_MAX_CLIENT_TIMEOUT,    &HSConfiguration::assignHTTPdClientTimeout },
   
   // 27
   { NULL,                            NULL,
     NULL,                                       (ConfigHandler::cfgHandler)NULL } // keep this!
};

const HSConfiguration::int_type HSConfiguration::MANDATORY_DECLARATIONS[] =
{
   IDX_SERVNAME,
   IDX_SERVERDESCR,
   IDX_REMOTEHOST,
   IDX_REMOTEPASSWORD,
   IDX_ADMINLOGIN,
   IDX_ADMINPASSWORD,
   IDX_HTTPDDOCUMENTROOT,
   -1 // keep this
};

const string HSConfiguration::DEF_HSCONFIGFILENAME     = HSCONFIG_FILE; //  <- config.h
const string HSConfiguration::DEF_VHOSTSCONFIGFILENAME = VHOSTS_FILE;   //   <-^
const string HSConfiguration::DEF_USERSCONFIGFILENAME  = USERS_FILE;    // <-^
const string HSConfiguration::DEF_HSNICK               = "HostServ";
const string HSConfiguration::DEF_HSUSERNAME           = "hostserv";
const string HSConfiguration::DEF_HSADDRESS            = "service";
const string HSConfiguration::DEF_HSMODES              = "+io";
const string HSConfiguration::DEF_HSDESCRIPTION        = "HostServ "VERSION; // <- config.h
const string HSConfiguration::DEF_VHOSTPUBLICPOLICY    = "0";
const string HSConfiguration::DEF_CLIENTDEFAULTPERM    = "0";
const string HSConfiguration::DEF_HTTPDDEFDOCUMENT     = "/index.html";
const string HSConfiguration::DEF_HTTPDBINDIF          = "0.0.0.0";

HSConfiguration::HSConfiguration()
   : m_vhosts(),
     m_clients(),
     m_port( -1 ),
     m_foreground( false ),
     m_help( false ),
     m_maxHTTPdClientNum( 2 ),
     m_httpdPort( -1 ),
     m_maxHTTPdClientTimeout( 30 )
{
#ifdef VERBOSE_MEMORY
   cout << "HSConfiguration] created" << endl;
#endif

   fillWithDefault();
}

HSConfiguration::~HSConfiguration()
{
#ifdef VERBOSE_MEMORY
   cout << "HSConfiguration] destroyed" << endl;
#endif
}

void
HSConfiguration::processHSConfigfileMap( const ConfigfileMap& cfgMap )
{
   const ConfigHandler* const  hBase = &HANDLERS[0];

   ConfigfileMap::const_iterator iter          = cfgMap.begin();
   const ConfigfileMap::const_iterator iterMax = cfgMap.end();

   for( ; iter != iterMax; ++iter )
   {
      const string&        key   = iter->first;
      const string&        value = iter->second;
      const ConfigHandler* hIter = hBase;

      for( ; ; ++hIter )
      {
	 if( ! hIter->handler )
	 {
#ifdef DEBUG
	    cout << "Warning: unknown configuration directive: " << key << endl;
#endif

	    break;
	 }

	 if( hIter->cfgDirective && *hIter->cfgDirective == key )
	 {
	    int_type diff = hIter - hBase;

	    (this->*(hIter->handler))( diff, value );
#ifdef DEBUG
	    cout << "HSConfiguration::processHSConfigFileMap(): handeled '" << key << "'->'" << value << '\'' << endl;
#endif
	    break;
	 }
      }
   }
}

void
HSConfiguration::processCMDLineArgs( const CMDLineArgMap& cfgMap )
{
   const ConfigHandler* const  hBase = &HANDLERS[0];

   CMDLineArgMap::const_iterator iter          = cfgMap.begin();
   const CMDLineArgMap::const_iterator iterMax = cfgMap.end();

   for( ; iter != iterMax; ++iter )
   {
      const CMDLineKey&    key   = iter->first;
      const string&        value = iter->second;
      const ConfigHandler* hIter = hBase;
      
      for( ; ; ++hIter )
      {
	 // ok we could include this into the for loop for simplicity
	 // done we would have to do a branch below to make
	 // this debug output possible
	 if( !hIter->handler )
	 {
	    cout << "Warning: unknown argument: " << key << endl;

	    break;
	 }
	 
	 const string* arg = key.isGNUStyle() ? hIter->longArg : hIter->shortArg;
	 
	 if( arg && *arg == key )
	 {
	    int_type diff = hIter- hBase;

	    (this->*(hIter->handler))( diff, value );

#ifdef DEBUG
	    cout << "HSConfiguration::processCMDLineArgs(): inserted '" << key << "'->' " << value << '\'' << endl;
#endif
	    break;
	 }
      }
   }
}

void
HSConfiguration::processConfigfile()
{
   HSConfigFile config;

   config.process( getHostservConfigfilename() );
   
   processHSConfigfileMap( config );
}

void
HSConfiguration::processVHostsConfigfile()
{
   VHostConfigFile config( &m_vhosts );

   config.process( getVHostsConfigfilename() );

   // create vhost for admin account
   if( adminVHostNameSet() )
   {
      const VHost* v = new VHost( getAdminVHostName(), false );
      
      if( m_vhosts.insertVHost( v ) )
	 cout << "Created VHost '" << getAdminVHostName() << "' for admin" << endl;
      else
	delete v;
   }
}

void
HSConfiguration::processUsersConfigfile()
{
   HSClientConfigFile config( &m_clients, &m_vhosts );

   assert( adminLoginSet() );
   assert( adminPasswordSet() );

   config.process( getUsersConfigfilename() );
   

   /* insert explicit admin into list */
   const string& login    = getAdminLogin();
   const string& password = getAdminPassword();


   // find a user with an equal loginname
   HSClient* duplicate = m_clients.getClient( login );

   if( duplicate )
   {
      if( duplicate->getPassword() != password )
      {
	 duplicate->setPassword( password );
	 cout << "Warning: explicit admin's password has changed. Changed it back to the value in configurationfile." << endl;
      }

      // equal login exists
      if( !duplicate->isAdmin() )
      {
	 // we want at least 1 admin in user db
	 duplicate->setAdmin( true );
	 cout << "Warning: explicit admin is no admin according to user database. Set admin flag therefore." << endl;
      }
   }
   else
   {
      /*
       * here: admin is not in user database
       * typically this is the case whenever hostserv has been started
       * the first time :-)
       */
      VHost* adminVHost = NULL;

      // find admin vhost
      if( adminVHostNameSet() )
      {
	 adminVHost = const_cast<VHost*>( m_vhosts.getVHost( getAdminVHostName() ) );
	 
	 // admin vhost must have been set by processVHostsConfigfile()
	 assert( adminVHost );
      }

      HSClient* admin = new HSClient ( login,
				       password,
				       adminVHost,
				       HSClient::flagAdmin );
      m_clients.insertClient( admin );
   }
}

const string*
HSConfiguration::check()
{
   const int_type *iter = &MANDATORY_DECLARATIONS[0];
   
   for( ; *iter != -1; ++iter )
   {
      if( !m_configuration[*iter].set() )
	 return HANDLERS[*iter].cfgDirective;
   }

   if( !remotePortSet() )
     return &HSConfigFile::REMOTE_PORT;

   // check httpd related variables
   if( m_httpdPort != -1 )
   {
      if( !httpdDefaultDocumentSet() )
	 return &HSConfigFile::HTTPD_DEFAULTDOCUMENT;
   } 

   return NULL;
}

void
HSConfiguration::serialize()
{
   cout << "Serializing configuration..." << flush;

   struct timeval start;
   gettimeofday( &start, 0 );
   
   VHostConfigFile    vhostConfig( &m_vhosts );   
   HSClientConfigFile userConfig( &m_clients, &m_vhosts );

   // make index usage optimal / remove 'leaks' in index structure
   m_vhosts.optimize();
   
   vhostConfig.setFilename( getVHostsConfigfilename() );
   userConfig.setFilename( getUsersConfigfilename() );
   
   vhostConfig.serialize();
   userConfig.serialize();
   
   struct timeval end;
   gettimeofday( &end, 0 );
   
  cout << "done (" 
       << ( end.tv_sec - start.tv_sec ) << "s "
       << ( end.tv_usec - start.tv_usec ) << "ms)" << endl;
}

void
HSConfiguration::cleanUpLists()
{
   // !warning: the clients have to be cleared _before_ the vhosts can get cleared
   // as hsclient removes references from the associated vhost object!
   m_clients.clear();
   m_vhosts.clear();
}

void
HSConfiguration::setDefaultVHostPublicPolicy( int idx, 
					      const string& val )
{
   VHost::setDefaultPublicPolicy( val == "1" );
}

void
HSConfiguration::setDefaultClientPermission( int idx,
					     const string& val )
{
   HSClient::setDefaultFlags( HSClient::parseFlagsString( val ) );
}

void
HSConfiguration::assignHostname( int idx,
				 const string& val )
{
  string            parsedVal    = val;
  string::size_type charsSkipped = toValidHostname( parsedVal );

  if( charsSkipped == val.length() )
  {
     cout << "Error: you have entered an illegal hostname: " << val << endl;
     cout << "Skipping" << endl;
  }
  else 
  {
     if( charsSkipped > 0 )
     {
	cout << "Warning: hostname contains " << charsSkipped << " illegal character(s)." << endl;
	cout << "Using '" << parsedVal << "' as hostname name" << endl;
     }
     m_configuration[idx] = parsedVal;
  }
}

void
HSConfiguration::assignUsername( int idx,
				 const string& val )
{
  string            parsedVal    = val;
  string::size_type charsSkipped = toValidUsername( parsedVal );

  if( charsSkipped == val.length() )
  {
     cout << "Error: you have entered an illegal username: " << val << endl;
     cout << "Skipping" << endl;
  }
  else 
  {
     if( charsSkipped > 0 )
     {
	cout << "Warning: username contains " << charsSkipped << " illegal character(s)." << endl;
	cout << "Using '" << parsedVal << "' as username name" << endl;
     }
     m_configuration[idx] = parsedVal;
  }
}

void
HSConfiguration::assignNickname( int idx,
				 const string& val )
{
  string            parsedVal    = val;
  string::size_type charsSkipped = toValidNickname( parsedVal );

  if( charsSkipped == val.length() )
  {
     cout << "Error: you have entered an illegal nickname: " << val << endl;
     cout << "Skipping" << endl;
  }
  else 
  {
     if( charsSkipped > 0 )
     {
	cout << "Warning: nickname contains " << charsSkipped << " illegal character(s)." << endl;
	cout << "Using '" << parsedVal << "' as nickname" << endl;
     }
     m_configuration[idx] = parsedVal;
  }
}

void
HSConfiguration::assignChannel( int idx, 
				const string& val )
{
  string            parsedVal    = val;
  string::size_type charsSkipped = toValidChanname( parsedVal );

  if( charsSkipped == val.length() )
  {
     cout << "Error: you have entered an illegal channame: " << val << endl;
     cout << "Skipping" << endl;
  }
  else 
  {
     if( charsSkipped > 0 )
     {
	cout << "Warning: channame contains " << charsSkipped << " illegal character(s)." << endl;
	cout << "Using '" << parsedVal << "' as channel name" << endl;
     }
     m_configuration[idx] = parsedVal;
  }
}

void
HSConfiguration::assignPort( int idx,
			     const string& strPort )
{
   int port = atoi( strPort.c_str() );
   
   if( port > 0 )
      m_port = port;
   else
   {
      cout << "Warning: you have entered an illegal value for port: " << strPort << endl;
      cout << "Leaving value untouched" << endl;
   }
}

void
HSConfiguration::assignForeground( int idx,
				   const string& fg )
{
   if( fg == "1" )
      m_foreground = true;
   else if( fg == "0" )
      m_foreground = false;
}

void
HSConfiguration::assignHTTPdPort( int idx, const string& rawPort )
{
   int port = atoi( rawPort.c_str() );
   
   if( port > 0 )
      m_httpdPort = port;
   else
   {
      cout << "Warning: you have entered an illegal value for httpd port: " << rawPort << endl;
      cout << "Leaving value untouched" << endl;
   }
}

void
HSConfiguration::assignHTTPdMaxClients( int idx, const string& rawNum )
{
   m_maxHTTPdClientNum = atoi( rawNum.c_str() );
}

void
HSConfiguration::assignHTTPdDefaultDocument( int idx, const string& filename )
{
   if( filename.empty() )
      return;

   string parsedFilename;
   
   if( filename[0] != '/' )
   {
      parsedFilename =  "/";
      parsedFilename += filename;
   }
   else
      parsedFilename = filename;

   assignCommon( idx, parsedFilename );
}

void
HSConfiguration::assignHTTPdDocumentRoot( int idx, const string& filename )
{
   if( filename.empty() )
      return;

   string            parsedPath = filename;
   string::size_type pos;

   // removes trailing slashes
   pos = parsedPath.find_last_not_of( '/' );
   if( pos != string::npos )
      parsedPath.erase( pos+1 );

   if( parsedPath.empty() )
      m_configuration[idx] = "/";
   else
      m_configuration[idx] = parsedPath;
}

void
HSConfiguration::assignHTTPdClientTimeout( int idx, const string& strTimeout )
{
   if( strTimeout.empty() )
      return;
   
   int timeout = atoi( strTimeout.c_str() );

   if( timeout < 1 )
   {
      cout << "Error: you have entered an illegal value for httpd client timeout" << endl;
      cout << "Will use default value." << endl;
   }
   else
      m_maxHTTPdClientTimeout = timeout;
}

void
HSConfiguration::assignHelp( int idx,
			     const string& fg )
{
   m_help = true;
}

void
HSConfiguration::assignAdminLogin( int idx, 
				   const string& rawNick )
{
   if( rawNick.empty() )
      return;

   string adminLogin              = rawNick;   
   string::size_type removedChars = toValidHSClientString( adminLogin );

   if( removedChars == rawNick.length() )
   {
      cout << "Error: you have entered an admin login which is consisting only of illegal characters." << endl;
   }
   else
   {
      if( removedChars > 0 )
      {
	 cout << "Warning: admin login contains " << removedChars << " illegal characters" << endl; 
      }
      
      assignCommon( idx, adminLogin );
   }
  
}

void
HSConfiguration::assignAdminPassword( int idx,
				      const string& rawPass )
{
   if( rawPass.empty() )
      return;

   string adminPassword           = rawPass;
   string::size_type removedChars = toValidHSClientString( adminPassword );

   if( removedChars == rawPass.length() ) 
   {
      cout << "Error: you have entered an only of illegal characters consisting admin password." << endl;
   }
   else 
   {
      if( removedChars > 0 )
      {
	 cout << "Warning: admin password contains " << removedChars << " illegal characters." << endl;
	 cout << "         Using '" << adminPassword << " as admin password" << endl;
      }

      assignCommon( idx, adminPassword );
   }
}

void
HSConfiguration::fillWithDefault()
{
   m_configuration[IDX_SERVERDESCR]       = DEF_HSDESCRIPTION;
   m_configuration[IDX_HSNICKNAME]        = DEF_HSNICK;
   m_configuration[IDX_HSUSERNAME]        = DEF_HSUSERNAME;
   m_configuration[IDX_HSADDRESS]         = DEF_HSADDRESS;
   m_configuration[IDX_HSREALNAME]        = DEF_HSDESCRIPTION;
   m_configuration[IDX_HSMODES]           = DEF_HSMODES;
   m_configuration[IDX_HSCONFIGFILENAME]  = DEF_HSCONFIGFILENAME;
   m_configuration[IDX_VHOSTSFILENAME]    = DEF_VHOSTSCONFIGFILENAME;
   m_configuration[IDX_USERSFILENAME]     = DEF_USERSCONFIGFILENAME;
   m_configuration[IDX_CLIENTDEFAULTPERM] = DEF_CLIENTDEFAULTPERM;
   m_configuration[IDX_VHOSTPUBLICPOLICY] = DEF_VHOSTPUBLICPOLICY;
   m_configuration[IDX_HTTPDDEFDOCUMENT]  = DEF_HTTPDDEFDOCUMENT;
   m_configuration[IDX_HTTPDBINDIF]       = DEF_HTTPDBINDIF;
}
