#include "hsirctraits.hpp"

/************************************************************************
 *   IRC - Internet Relay Chat, src/match.c
 *   Copyright (C) 1990 Jarkko Oikarinen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Changes:
 * Thomas Helvey <tomh@inxpress.net> June 23, 1999
 * Const correctness changes
 * Cleanup of collapse and match
 * Moved static calls variable to match
 * Added asserts for null pointers
 * Martin Scholl <ms@deridiot.net> March 14, 2002
 * Added 'HSCLIENT_C'
 */

const unsigned int HSIRCTraits::CHAR_ATTRS[] = {
/* 0  */     CNTRL_C,
/* 1  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 2  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 3  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 4  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 5  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 6  */     CNTRL_C|CHAN_C|NONEOS_C,
/* 7 BEL */  CNTRL_C|NONEOS_C,
/* 8  \b */  CNTRL_C|CHAN_C|NONEOS_C,
/* 9  \t */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C,
/* 10 \n */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C|EOL_C,
/* 11 \v */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C,
/* 12 \f */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C,
/* 13 \r */  CNTRL_C|SPACE_C|CHAN_C|NONEOS_C|EOL_C,
/* 14 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 15 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 16 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 17 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 18 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 19 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 20 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 21 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 22 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 23 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 24 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 25 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 26 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 27 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 28 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 29 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 30 */     CNTRL_C|CHAN_C|NONEOS_C,
/* 31 */     CNTRL_C|CHAN_C|NONEOS_C,
/* SP */     PRINT_C|SPACE_C,
/* ! */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* " */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* # */      PRINT_C|CHANPFX_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* $ */      PRINT_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* % */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* & */      PRINT_C|CHANPFX_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* ' */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* ( */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* ) */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* * */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C|SERV_C|HSCLIENT_C,
/* + */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* , */      PRINT_C|NONEOS_C|HSCLIENT_C,
/* - */      PRINT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C,
/* . */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C|USER_C|HOST_C|SERV_C|HSCLIENT_C,
/* / */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* 0 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* 1 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* 2 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* 3 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* 4 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* 5 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* 6 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* 7 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* 8 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* 9 */      PRINT_C|DIGIT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* : */      PRINT_C|CHAN_C|NONEOS_C|HOST_C,
/* ; */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* < */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* = */      PRINT_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* > */      PRINT_C|CHAN_C|NONEOS_C,
/* ? */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C|HSCLIENT_C,
/* @ */      PRINT_C|KWILD_C|CHAN_C|NONEOS_C,
/* A */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* B */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* C */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* D */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* E */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* F */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* G */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* H */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* I */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* J */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* K */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* L */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* M */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* N */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* O */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* P */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* Q */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* R */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* S */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* T */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* U */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* V */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* W */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* X */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* Y */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* Z */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* [ */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* \ */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* ] */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* ^ */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* _ */      PRINT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* ` */      PRINT_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* a */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* b */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* c */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* d */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* e */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* f */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* g */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* h */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* i */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* j */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* k */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* l */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* m */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* n */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* o */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* p */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* q */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* r */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* s */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* t */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* u */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* v */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* w */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* x */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* y */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* z */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HOST_C|HSCLIENT_C,
/* { */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* | */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* } */      PRINT_C|ALPHA_C|NICK_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* ~ */      PRINT_C|ALPHA_C|CHAN_C|NONEOS_C|USER_C|HSCLIENT_C,
/* del  */   CHAN_C|NONEOS_C,
/* 0x80 */   CHAN_C|NONEOS_C,
/* 0x81 */   CHAN_C|NONEOS_C,
/* 0x82 */   CHAN_C|NONEOS_C,
/* 0x83 */   CHAN_C|NONEOS_C,
/* 0x84 */   CHAN_C|NONEOS_C,
/* 0x85 */   CHAN_C|NONEOS_C,
/* 0x86 */   CHAN_C|NONEOS_C,
/* 0x87 */   CHAN_C|NONEOS_C,
/* 0x88 */   CHAN_C|NONEOS_C,
/* 0x89 */   CHAN_C|NONEOS_C,
/* 0x8A */   CHAN_C|NONEOS_C,
/* 0x8B */   CHAN_C|NONEOS_C,
/* 0x8C */   CHAN_C|NONEOS_C,
/* 0x8D */   CHAN_C|NONEOS_C,
/* 0x8E */   CHAN_C|NONEOS_C,
/* 0x8F */   CHAN_C|NONEOS_C,
/* 0x90 */   CHAN_C|NONEOS_C,
/* 0x91 */   CHAN_C|NONEOS_C,
/* 0x92 */   CHAN_C|NONEOS_C,
/* 0x93 */   CHAN_C|NONEOS_C,
/* 0x94 */   CHAN_C|NONEOS_C,
/* 0x95 */   CHAN_C|NONEOS_C,
/* 0x96 */   CHAN_C|NONEOS_C,
/* 0x97 */   CHAN_C|NONEOS_C,
/* 0x98 */   CHAN_C|NONEOS_C,
/* 0x99 */   CHAN_C|NONEOS_C,
/* 0x9A */   CHAN_C|NONEOS_C,
/* 0x9B */   CHAN_C|NONEOS_C,
/* 0x9C */   CHAN_C|NONEOS_C,
/* 0x9D */   CHAN_C|NONEOS_C,
/* 0x9E */   CHAN_C|NONEOS_C,
/* 0x9F */   CHAN_C|NONEOS_C,
/* 0xA0 */   CHAN_C|NONEOS_C,
/* 0xA1 */   CHAN_C|NONEOS_C,
/* 0xA2 */   CHAN_C|NONEOS_C,
/* 0xA3 */   CHAN_C|NONEOS_C,
/* 0xA4 */   CHAN_C|NONEOS_C,
/* 0xA5 */   CHAN_C|NONEOS_C,
/* 0xA6 */   CHAN_C|NONEOS_C,
/* 0xA7 */   CHAN_C|NONEOS_C,
/* 0xA8 */   CHAN_C|NONEOS_C,
/* 0xA9 */   CHAN_C|NONEOS_C,
/* 0xAA */   CHAN_C|NONEOS_C,
/* 0xAB */   CHAN_C|NONEOS_C,
/* 0xAC */   CHAN_C|NONEOS_C,
/* 0xAD */   CHAN_C|NONEOS_C,
/* 0xAE */   CHAN_C|NONEOS_C,
/* 0xAF */   CHAN_C|NONEOS_C,
/* 0xB0 */   CHAN_C|NONEOS_C,
/* 0xB1 */   CHAN_C|NONEOS_C,
/* 0xB2 */   CHAN_C|NONEOS_C,
/* 0xB3 */   CHAN_C|NONEOS_C,
/* 0xB4 */   CHAN_C|NONEOS_C,
/* 0xB5 */   CHAN_C|NONEOS_C,
/* 0xB6 */   CHAN_C|NONEOS_C,
/* 0xB7 */   CHAN_C|NONEOS_C,
/* 0xB8 */   CHAN_C|NONEOS_C,
/* 0xB9 */   CHAN_C|NONEOS_C,
/* 0xBA */   CHAN_C|NONEOS_C,
/* 0xBB */   CHAN_C|NONEOS_C,
/* 0xBC */   CHAN_C|NONEOS_C,
/* 0xBD */   CHAN_C|NONEOS_C,
/* 0xBE */   CHAN_C|NONEOS_C,
/* 0xBF */   CHAN_C|NONEOS_C,
/* 0xC0 */   CHAN_C|NONEOS_C,
/* 0xC1 */   CHAN_C|NONEOS_C,
/* 0xC2 */   CHAN_C|NONEOS_C,
/* 0xC3 */   CHAN_C|NONEOS_C,
/* 0xC4 */   CHAN_C|NONEOS_C,
/* 0xC5 */   CHAN_C|NONEOS_C,
/* 0xC6 */   CHAN_C|NONEOS_C,
/* 0xC7 */   CHAN_C|NONEOS_C,
/* 0xC8 */   CHAN_C|NONEOS_C,
/* 0xC9 */   CHAN_C|NONEOS_C,
/* 0xCA */   CHAN_C|NONEOS_C,
/* 0xCB */   CHAN_C|NONEOS_C,
/* 0xCC */   CHAN_C|NONEOS_C,
/* 0xCD */   CHAN_C|NONEOS_C,
/* 0xCE */   CHAN_C|NONEOS_C,
/* 0xCF */   CHAN_C|NONEOS_C,
/* 0xD0 */   CHAN_C|NONEOS_C,
/* 0xD1 */   CHAN_C|NONEOS_C,
/* 0xD2 */   CHAN_C|NONEOS_C,
/* 0xD3 */   CHAN_C|NONEOS_C,
/* 0xD4 */   CHAN_C|NONEOS_C,
/* 0xD5 */   CHAN_C|NONEOS_C,
/* 0xD6 */   CHAN_C|NONEOS_C,
/* 0xD7 */   CHAN_C|NONEOS_C,
/* 0xD8 */   CHAN_C|NONEOS_C,
/* 0xD9 */   CHAN_C|NONEOS_C,
/* 0xDA */   CHAN_C|NONEOS_C,
/* 0xDB */   CHAN_C|NONEOS_C,
/* 0xDC */   CHAN_C|NONEOS_C,
/* 0xDD */   CHAN_C|NONEOS_C,
/* 0xDE */   CHAN_C|NONEOS_C,
/* 0xDF */   CHAN_C|NONEOS_C,
/* 0xE0 */   CHAN_C|NONEOS_C,
/* 0xE1 */   CHAN_C|NONEOS_C,
/* 0xE2 */   CHAN_C|NONEOS_C,
/* 0xE3 */   CHAN_C|NONEOS_C,
/* 0xE4 */   CHAN_C|NONEOS_C,
/* 0xE5 */   CHAN_C|NONEOS_C,
/* 0xE6 */   CHAN_C|NONEOS_C,
/* 0xE7 */   CHAN_C|NONEOS_C,
/* 0xE8 */   CHAN_C|NONEOS_C,
/* 0xE9 */   CHAN_C|NONEOS_C,
/* 0xEA */   CHAN_C|NONEOS_C,
/* 0xEB */   CHAN_C|NONEOS_C,
/* 0xEC */   CHAN_C|NONEOS_C,
/* 0xED */   CHAN_C|NONEOS_C,
/* 0xEE */   CHAN_C|NONEOS_C,
/* 0xEF */   CHAN_C|NONEOS_C,
/* 0xF0 */   CHAN_C|NONEOS_C,
/* 0xF1 */   CHAN_C|NONEOS_C,
/* 0xF2 */   CHAN_C|NONEOS_C,
/* 0xF3 */   CHAN_C|NONEOS_C,
/* 0xF4 */   CHAN_C|NONEOS_C,
/* 0xF5 */   CHAN_C|NONEOS_C,
/* 0xF6 */   CHAN_C|NONEOS_C,
/* 0xF7 */   CHAN_C|NONEOS_C,
/* 0xF8 */   CHAN_C|NONEOS_C,
/* 0xF9 */   CHAN_C|NONEOS_C,
/* 0xFA */   CHAN_C|NONEOS_C,
/* 0xFB */   CHAN_C|NONEOS_C,
/* 0xFC */   CHAN_C|NONEOS_C,
/* 0xFD */   CHAN_C|NONEOS_C,
/* 0xFE */   CHAN_C|NONEOS_C,
/* 0xFF */   CHAN_C|NONEOS_C
};



/***************************************************************************
                           hsirctraits.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: hsirctraits.cpp,v 1.2 2002/04/30 18:19:48 d3cline Exp $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

const unsigned char HSIRCTraits::TOLOWER_TAB[] = {
  /* 0  */     0,
  /* 1  */     1,
  /* 2  */     2,
  /* 3  */     3, 
  /* 4  */     4, 
  /* 5  */     5, 
  /* 6  */     6, 
  /* 7 BEL */  7, 
  /* 8  \b */  8,
  /* 9  \t */  9,
  /* 10 \n */  10,
  /* 11 \v */  11,
  /* 12 \f */  12,
  /* 13 \r */  13, 
  /* 14 */     14, 
  /* 15 */     15,
  /* 16 */     16,
  /* 17 */     17,
  /* 18 */     18, 
  /* 19 */     19, 
  /* 20 */     20, 
  /* 21 */     21, 
  /* 22 */     22, 
  /* 23 */     23, 
  /* 24 */     24,
  /* 25 */     25,
  /* 26 */     26,
  /* 27 */     27,
  /* 28 */     28,
  /* 29 */     29,
  /* 30 */     30,
  /* 31 */     31,
  /* SP */     32, 
  /* ! */      33, 
  /* " */      34, 
  /* # */      35,
  /* $ */      36,
  /* % */      37,
  /* & */      38,
  /* ' */      39,
  /* ( */      40,
  /* ) */      41,
  /* * */      42,
  /* + */      43,
  /* , */      44,
  /* - */      45,
  /* . */      46,
  /* / */      47,
  /* 0 */      48,
  /* 1 */      49,
  /* 2 */      50,
  /* 3 */      51,
  /* 4 */      52,
  /* 5 */      53,
  /* 6 */      54,
  /* 7 */      55,
  /* 8 */      56,
  /* 9 */      57,
  /* : */      58,
  /* ; */      59,
  /* < */      60,
  /* = */      61,
  /* > */      62,
  /* ? */      63,
  /* @ */      64,
  /* A */      'a', // 65
  /* B */      'b', // 66
  /* C */      'c', // 67
  /* D */      'd', // 68
  /* E */      'e', // 69
  /* F */      'f', // 70
  /* G */      'g', // 71
  /* H */      'h', // 72
  /* I */      'i', // 73
  /* J */      'j', // 74
  /* K */      'k', // 75
  /* L */      'l', // 76
  /* M */      'm', // 77
  /* N */      'n', // 78
  /* O */      'o', // 79
  /* P */      'p', // 80
  /* Q */      'q', // 81
  /* R */      'r', // 82
  /* S */      's', // 83
  /* T */      't', // 84
  /* U */      'u', // 85
  /* V */      'v', // 86
  /* W */      'w', // 87
  /* X */      'x', // 88
  /* Y */      'y', // 89
  /* Z */      'z', // 90
  /* [ */      '{', // 91
  /* \ */      '|', // 92
  /* ] */      '}', // 93
  /* ^ */      94,
  /* _ */      95,
  /* ` */      96,
  /* a */      97,
  /* b */      98,
  /* c */      99,
  /* d */      100,
  /* e */      101,
  /* f */      102,
  /* g */      103,
  /* h */      104,
  /* i */      105,
  /* j */      106,
  /* k */      107,
  /* l */      108,
  /* m */      109,
  /* n */      110,
  /* o */      111,
  /* p */      112, 
  /* q */      113, 
  /* r */      114,
  /* s */      115,
  /* t */      116,
  /* u */      117,
  /* v */      118,
  /* w */      119,
  /* x */      120,
  /* y */      121,
  /* z */      122,
  /* { */      123, 
  /* | */      124,
  /* } */      125,
  /* ~ */      126,
  /* del  */   127,
  /* 0x80 */   0x80,
  /* 0x81 */   0x81,
  /* 0x82 */   0x82,
  /* 0x83 */   0x83,
  /* 0x84 */   0x84,
  /* 0x85 */   0x85,
  /* 0x86 */   0x86,
  /* 0x87 */   0x87,
  /* 0x88 */   0x88,
  /* 0x89 */   0x89,
  /* 0x8A */   0x8A,
  /* 0x8B */   0x8B,
  /* 0x8C */   0x8C,
  /* 0x8D */   0x8D,
  /* 0x8E */   0x8E,
  /* 0x8F */   0x8F,
  /* 0x90 */   0x90,
  /* 0x91 */   0x91,
  /* 0x92 */   0x92,
  /* 0x93 */   0x93,
  /* 0x94 */   0x94,
  /* 0x95 */   0x95,
  /* 0x96 */   0x96,
  /* 0x97 */   0x97,
  /* 0x98 */   0x98,
  /* 0x99 */   0x99,
  /* 0x9A */   0x9A,
  /* 0x9B */   0x9B,
  /* 0x9C */   0x9C,
  /* 0x9D */   0x9D,
  /* 0x9E */   0x9E,
  /* 0x9F */   0x9F,
  /* 0xA0 */   0xA0,
  /* 0xA1 */   0xA1,
  /* 0xA2 */   0xA2,
  /* 0xA3 */   0xA3,
  /* 0xA4 */   0xA4,
  /* 0xA5 */   0xA5,
  /* 0xA6 */   0xA6,
  /* 0xA7 */   0xA7,
  /* 0xA8 */   0xA8,
  /* 0xA9 */   0xA9,
  /* 0xAA */   0xAA,
  /* 0xAB */   0xAB,
  /* 0xAC */   0xAC,
  /* 0xAD */   0xAD,
  /* 0xAE */   0xAE,
  /* 0xAF */   0xAF,
  /* 0xB0 */   0xB0,
  /* 0xB1 */   0xB1,
  /* 0xB2 */   0xB2,
  /* 0xB3 */   0xB3,
  /* 0xB4 */   0xB4,
  /* 0xB5 */   0xB5,
  /* 0xB6 */   0xB6,
  /* 0xB7 */   0xB7,
  /* 0xB8 */   0xB8,
  /* 0xB9 */   0xB9,
  /* 0xBA */   0xBA,
  /* 0xBB */   0xBB,
  /* 0xBC */   0xBC,
  /* 0xBD */   0xBD,
  /* 0xBE */   0xBE,
  /* 0xBF */   0xBF,
  /* 0xC0 */   0xC0,
  /* 0xC1 */   0xC1,
  /* 0xC2 */   0xC2,
  /* 0xC3 */   0xC3,
  /* 0xC4 */   0xC4,
  /* 0xC5 */   0xC5,
  /* 0xC6 */   0xC6,
  /* 0xC7 */   0xC7,
  /* 0xC8 */   0xC8,
  /* 0xC9 */   0xC9,
  /* 0xCA */   0xCA,
  /* 0xCB */   0xCB,
  /* 0xCC */   0xCC,
  /* 0xCD */   0xCD,
  /* 0xCE */   0xCE,
  /* 0xCF */   0xCF,
  /* 0xD0 */   0xD0,
  /* 0xD1 */   0xD1,
  /* 0xD2 */   0xD2,
  /* 0xD3 */   0xD3,
  /* 0xD4 */   0xD4,
  /* 0xD5 */   0xD5,
  /* 0xD6 */   0xD6,
  /* 0xD7 */   0xD7,
  /* 0xD8 */   0xD8,
  /* 0xD9 */   0xD9,
  /* 0xDA */   0xDA,
  /* 0xDB */   0xDB,
  /* 0xDC */   0xDC,
  /* 0xDD */   0xDD,
  /* 0xDE */   0xDE,
  /* 0xDF */   0xDF,
  /* 0xE0 */   0xE0,
  /* 0xE1 */   0xE1,
  /* 0xE2 */   0xE2,
  /* 0xE3 */   0xE3,
  /* 0xE4 */   0xE4,
  /* 0xE5 */   0xE5,
  /* 0xE6 */   0xE6,
  /* 0xE7 */   0xE7,
  /* 0xE8 */   0xE8,
  /* 0xE9 */   0xE9,
  /* 0xEA */   0xEA,
  /* 0xEB */   0xEB,
  /* 0xEC */   0xEC,
  /* 0xED */   0xED,
  /* 0xEE */   0xEE,
  /* 0xEF */   0xEF,
  /* 0xF0 */   0xF0,
  /* 0xF1 */   0xF1,
  /* 0xF2 */   0xF2,
  /* 0xF3 */   0xF2,
  /* 0xF4 */   0xF4,
  /* 0xF5 */   0xF5,
  /* 0xF6 */   0xF6,
  /* 0xF7 */   0xF7,
  /* 0xF8 */   0xF8,
  /* 0xF9 */   0xF9,
  /* 0xFA */   0xFA,
  /* 0xFB */   0xFB,
  /* 0xFC */   0xFC,
  /* 0xFD */   0xFD,
  /* 0xFE */   0xFE,
  /* 0xFF */   0xFF
};

const unsigned char HSIRCTraits::TOUPPER_TAB[] = {
  /* 0  */     0,
  /* 1  */     1,
  /* 2  */     2,
  /* 3  */     3, 
  /* 4  */     4, 
  /* 5  */     5, 
  /* 6  */     6, 
  /* 7 BEL */  7, 
  /* 8  \b */  8,
  /* 9  \t */  9,
  /* 10 \n */  10,
  /* 11 \v */  11,
  /* 12 \f */  12,
  /* 13 \r */  13, 
  /* 14 */     14, 
  /* 15 */     15,
  /* 16 */     16,
  /* 17 */     17,
  /* 18 */     18, 
  /* 19 */     19, 
  /* 20 */     20, 
  /* 21 */     21, 
  /* 22 */     22, 
  /* 23 */     23, 
  /* 24 */     24,
  /* 25 */     25,
  /* 26 */     26,
  /* 27 */     27,
  /* 28 */     28,
  /* 29 */     29,
  /* 30 */     30,
  /* 31 */     31,
  /* SP */     32, 
  /* ! */      33, 
  /* " */      34, 
  /* # */      35,
  /* $ */      36,
  /* % */      37,
  /* & */      38,
  /* ' */      39,
  /* ( */      40,
  /* ) */      41,
  /* * */      42,
  /* + */      43,
  /* , */      44,
  /* - */      45,
  /* . */      46,
  /* / */      47,
  /* 0 */      48,
  /* 1 */      49,
  /* 2 */      50,
  /* 3 */      51,
  /* 4 */      52,
  /* 5 */      53,
  /* 6 */      54,
  /* 7 */      55,
  /* 8 */      56,
  /* 9 */      57,
  /* : */      58,
  /* ; */      59,
  /* < */      60,
  /* = */      61,
  /* > */      62,
  /* ? */      63,
  /* @ */      64,
  /* A */      65,
  /* B */      66,
  /* C */      67,
  /* D */      68,
  /* E */      69,
  /* F */      70,
  /* G */      71,
  /* H */      72,
  /* I */      73,
  /* J */      74,
  /* K */      75,
  /* L */      76,
  /* M */      77,
  /* N */      78,
  /* O */      79,
  /* P */      80,
  /* Q */      81,
  /* R */      82,
  /* S */      83,
  /* T */      84,
  /* U */      85,
  /* V */      86,
  /* W */      87,
  /* X */      88,
  /* Y */      89,
  /* Z */      90,
  /* [ */      91,
  /* \ */      92,
  /* ] */      93,
  /* ^ */      94,
  /* _ */      95,
  /* ` */      96,
  /* a */      'A',
  /* b */      'B',
  /* c */      'C',
  /* d */      'D',
  /* e */      'E',
  /* f */      'F',
  /* g */      'G',
  /* h */      'H',
  /* i */      'I',
  /* j */      'J',
  /* k */      'K',
  /* l */      'L',
  /* m */      'M',
  /* n */      'N',
  /* o */      'O',
  /* p */      'P', 
  /* q */      'Q', 
  /* r */      'R',
  /* s */      'S',
  /* t */      'T',
  /* u */      'U',
  /* v */      'V',
  /* w */      'W',
  /* x */      'X',
  /* y */      'Y',
  /* z */      'Z',
  /* { */      '[', 
  /* | */      '\\',
  /* } */      ']',
  /* ~ */      126,
  /* del  */   127,
  /* 0x80 */   0x80,
  /* 0x81 */   0x81,
  /* 0x82 */   0x82,
  /* 0x83 */   0x83,
  /* 0x84 */   0x84,
  /* 0x85 */   0x85,
  /* 0x86 */   0x86,
  /* 0x87 */   0x87,
  /* 0x88 */   0x88,
  /* 0x89 */   0x89,
  /* 0x8A */   0x8A,
  /* 0x8B */   0x8B,
  /* 0x8C */   0x8C,
  /* 0x8D */   0x8D,
  /* 0x8E */   0x8E,
  /* 0x8F */   0x8F,
  /* 0x90 */   0x90,
  /* 0x91 */   0x91,
  /* 0x92 */   0x92,
  /* 0x93 */   0x93,
  /* 0x94 */   0x94,
  /* 0x95 */   0x95,
  /* 0x96 */   0x96,
  /* 0x97 */   0x97,
  /* 0x98 */   0x98,
  /* 0x99 */   0x99,
  /* 0x9A */   0x9A,
  /* 0x9B */   0x9B,
  /* 0x9C */   0x9C,
  /* 0x9D */   0x9D,
  /* 0x9E */   0x9E,
  /* 0x9F */   0x9F,
  /* 0xA0 */   0xA0,
  /* 0xA1 */   0xA1,
  /* 0xA2 */   0xA2,
  /* 0xA3 */   0xA3,
  /* 0xA4 */   0xA4,
  /* 0xA5 */   0xA5,
  /* 0xA6 */   0xA6,
  /* 0xA7 */   0xA7,
  /* 0xA8 */   0xA8,
  /* 0xA9 */   0xA9,
  /* 0xAA */   0xAA,
  /* 0xAB */   0xAB,
  /* 0xAC */   0xAC,
  /* 0xAD */   0xAD,
  /* 0xAE */   0xAE,
  /* 0xAF */   0xAF,
  /* 0xB0 */   0xB0,
  /* 0xB1 */   0xB1,
  /* 0xB2 */   0xB2,
  /* 0xB3 */   0xB3,
  /* 0xB4 */   0xB4,
  /* 0xB5 */   0xB5,
  /* 0xB6 */   0xB6,
  /* 0xB7 */   0xB7,
  /* 0xB8 */   0xB8,
  /* 0xB9 */   0xB9,
  /* 0xBA */   0xBA,
  /* 0xBB */   0xBB,
  /* 0xBC */   0xBC,
  /* 0xBD */   0xBD,
  /* 0xBE */   0xBE,
  /* 0xBF */   0xBF,
  /* 0xC0 */   0xC0,
  /* 0xC1 */   0xC1,
  /* 0xC2 */   0xC2,
  /* 0xC3 */   0xC3,
  /* 0xC4 */   0xC4,
  /* 0xC5 */   0xC5,
  /* 0xC6 */   0xC6,
  /* 0xC7 */   0xC7,
  /* 0xC8 */   0xC8,
  /* 0xC9 */   0xC9,
  /* 0xCA */   0xCA,
  /* 0xCB */   0xCB,
  /* 0xCC */   0xCC,
  /* 0xCD */   0xCD,
  /* 0xCE */   0xCE,
  /* 0xCF */   0xCF,
  /* 0xD0 */   0xD0,
  /* 0xD1 */   0xD1,
  /* 0xD2 */   0xD2,
  /* 0xD3 */   0xD3,
  /* 0xD4 */   0xD4,
  /* 0xD5 */   0xD5,
  /* 0xD6 */   0xD6,
  /* 0xD7 */   0xD7,
  /* 0xD8 */   0xD8,
  /* 0xD9 */   0xD9,
  /* 0xDA */   0xDA,
  /* 0xDB */   0xDB,
  /* 0xDC */   0xDC,
  /* 0xDD */   0xDD,
  /* 0xDE */   0xDE,
  /* 0xDF */   0xDF,
  /* 0xE0 */   0xE0,
  /* 0xE1 */   0xE1,
  /* 0xE2 */   0xE2,
  /* 0xE3 */   0xE3,
  /* 0xE4 */   0xE4,
  /* 0xE5 */   0xE5,
  /* 0xE6 */   0xE6,
  /* 0xE7 */   0xE7,
  /* 0xE8 */   0xE8,
  /* 0xE9 */   0xE9,
  /* 0xEA */   0xEA,
  /* 0xEB */   0xEB,
  /* 0xEC */   0xEC,
  /* 0xED */   0xED,
  /* 0xEE */   0xEE,
  /* 0xEF */   0xEF,
  /* 0xF0 */   0xF0,
  /* 0xF1 */   0xF1,
  /* 0xF2 */   0xF2,
  /* 0xF3 */   0xF2,
  /* 0xF4 */   0xF4,
  /* 0xF5 */   0xF5,
  /* 0xF6 */   0xF6,
  /* 0xF7 */   0xF7,
  /* 0xF8 */   0xF8,
  /* 0xF9 */   0xF9,
  /* 0xFA */   0xFA,
  /* 0xFB */   0xFB,
  /* 0xFC */   0xFC,
  /* 0xFD */   0xFD,
  /* 0xFE */   0xFE,
  /* 0xFF */   0xFF 
};
