/***************************************************************************
                           ircclientconnection.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: ircclientconnection.cpp,v 1.15 2002/05/29 19:43:46 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ircclientconnection.hpp"
#include "ircmessage.hpp"


IRCClientConnection::IRCClientConnection( Poller* poller, 
					  len_type inBufLen,
					  len_type outBufLen )
   : m_inBuf( *this, 
	      inBufLen,
	      6 ),
     m_outBuf( *this, 
	       outBufLen ),
     m_outstream( &m_outBuf ),
     m_pollerClient( this ),
     m_loginState( DO_LOGIN )
{
}

void
IRCClientConnection::onInput()
{
   const IRCServerMessage* msg = NULL;
   
   // read error
   if( m_inBuf.fill() == File::eof )
      close();


   while( !m_inBuf.readCompletely() )
   {
      msg = m_inBuf.read();

      if( msg != NULL )
	 onMessage( *msg );
      else 
      {
#ifdef DEBUG
	 std::cout << "IRCClientConnection: received incomplete message. skipping" << std::endl;
#endif
	 break;
      }
   }


   if( msg != NULL || m_inBuf.isFull() )
      // we want the buffer to be 'flushed'
      // whenever a complete message has been retrieved
      // completely ( terminated by '\n' ) and the whole
      // buffer has been processed
      m_inBuf.flush();


   if( m_outBuf.isFilled() )
   {
      m_pollerClient.unregisterEvent( PollerFD::EVT_IN );
      m_pollerClient.registerEvent( PollerFD::EVT_OUT );
   }
}

void
IRCClientConnection::onOutput()
{
  if( m_loginState == DO_LOGIN )
  {
     onDoLogin();

     m_loginState = DID_LOGIN;

     // make connection also listen for input
     m_pollerClient.registerEvent( PollerFD::EVT_IN );
  }

  if( m_outBuf.doFlush() == File::eof )
     close();
  else
  {
     m_pollerClient.unregisterEvent( PollerFD::EVT_OUT );
     m_pollerClient.registerEvent( PollerFD::EVT_IN );
  }
}

void
IRCClientConnection::onConnected()
{
   m_inBuf.resetFlags();
   
   // reset login state
   m_loginState = DO_LOGIN;
   

   /*
    * only register EVT_OUT after connect()
    * we want to print login first and receive data then
    */ 
   m_pollerClient.unregisterAllEvents();
   m_pollerClient.registerEvent( PollerFD::EVT_OUT );
}

void
IRCClientConnection::onClosing()
{ 
   m_pollerClient.unregister(); 
}

