/***************************************************************************
                           ircmessagetokenizer.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: ircmessagetokenizer.cpp,v 1.10 2002/03/19 21:09:27 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "ircmessagetokenizer.hpp"

IRCMessageTokenizer::IRCMessageTokenizer( int numMsgTokens )
  : IRCTokenizer(),
    m_msgTokenizer( numMsgTokens, TOK_DELIM, MSG_DELIM ),
    m_lastMessage()
{
}

IRCMessageTokenizer::int_type
IRCMessageTokenizer::update( reference_type buf, len_type length )
{
   bool clientMessage = false;

   // actual number of parsed bytes by both tokenizer
   len_type        parsedLen   = 0;

   if( *buf == ':' )
   {
      ++buf;
      --length;
      ++parsedLen;
      clientMessage = true;
   }

   const len_type  rxBytes     = IRCTokenizer::update( buf, length );
   // update number of processed bytes
   parsedLen += rxBytes;

   // irc token. these are the tokens before ':' or '\n'
   IRCToken* const ircTokens   = const_cast<IRCToken*>( IRCTokenizer::getToken() );
   // number of irc token
   int_type        ircTokenNum = IRCTokenizer::getTokenNum();

   // number of msg token. 
   len_type        msgTokenNum = 1;
   // the message token is the last token of a server
   // message or the parsed long token after a ':'
   IRCToken*       msgTokens   = &ircTokens[ircTokenNum-1];



   // check for an parser error
   if( rxBytes == length && buf[rxBytes] != '\n' )
      return -1;
   
   if( buf[rxBytes] != '\n' )
   {
      /*
       * [+/-]1 must be used because buf[rxBytes] points to the 
       * delimiter always
       */
      // we get here whenever a long token (-> " :" ) does exist
      const reference_type msgBegin    = buf+ rxBytes + 1;
      const len_type       msgLen      = length - rxBytes - 1;
      const len_type       parsedBytes =
	 m_msgTokenizer.update( msgBegin, msgLen );

      
      // update number of parsed bytes
      parsedLen += parsedBytes + 1;


      // check for an parser error
      if( parsedBytes == msgLen && msgBegin[parsedLen] != '\n' )
	 return -1;


      msgTokens   = const_cast<IRCToken*>( m_msgTokenizer.getToken() );
      msgTokenNum = m_msgTokenizer.getTokenNum();


      IRCToken& target = msgTokens[msgTokenNum-1];

      len_type parsedBytesEffective = parsedBytes;

      // remove '\r'
      if( target[target.length()-1] == '\r' )
      {
	 target.assign( target.c_str(), target.length()-1 );

	 /*
	   work around whitespace parser bug
	 */
	 // decrement number of parsed tokens whenever the last token is empty ( contains only '\r' )
	 if( target.empty() )
	   --msgTokenNum;

	 --parsedBytesEffective;
      }

      if( ircTokenNum < TOK_NUM )
      {
         // assign the last long token to the irc token array
	 ircTokens[ircTokenNum].assign( msgBegin, parsedBytesEffective );

	 ++ircTokenNum;
      }
   }
   else 
   {
      IRCToken& target = ircTokens[ircTokenNum-1];

      // remove '\r'
      if( target[target.length()-1] == '\r' )
	 target.assign( target.c_str(), target.length()-1 );
   }

   // update IRCServerMessage object
   m_lastMessage.assign( ircTokens, ircTokenNum,
			 msgTokens, msgTokenNum,
			 clientMessage );

   return parsedLen;
}
