/***************************************************************************
                           ircserverconnection.cpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: ircserverconnection.cpp,v 1.38 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <ircserverconnection.hpp>
#include <ctime>
#include <iostream>
extern "C" {
#include <sys/time.h>
#include <stdarg.h>
}

#include "config.h"

/*
 * define it to make this class print what
 * it is printing
 */
#define VERBOSE_PRINTING

// todo: implement usage of timeout !

static long getUNIXTimestamp()
{
   return time( NULL );
}

const IRCToken IRCServerConnection::CMD_PASS    = "PASS";
const IRCToken IRCServerConnection::CMD_SERVER  = "SERVER";
const IRCToken IRCServerConnection::CMD_PING    = "PING";
const IRCToken IRCServerConnection::CMD_PONG    = "PONG";
const IRCToken IRCServerConnection::CMD_NICK    = "NICK";
const IRCToken IRCServerConnection::CMD_MODE    = "MODE";
const IRCToken IRCServerConnection::CMD_QUIT    = "QUIT";
const IRCToken IRCServerConnection::CMD_WHOIS   = "WHOIS";
const IRCToken IRCServerConnection::ERR_433     = "433"; // nickname in use
const IRCToken IRCServerConnection::ERR_436     = "436"; // nickname collision
const IRCToken IRCServerConnection::CMD_PRIVMSG = "PRIVMSG";
const IRCToken IRCServerConnection::CMD_ERROR   = "ERROR";
const IRCToken IRCServerConnection::CMD_NOTICE  = "NOTICE";
const IRCToken IRCServerConnection::CMD_SJOIN   = "SJOIN";
const IRCToken IRCServerConnection::CMD_JOIN    = "JOIN";
const IRCToken IRCServerConnection::CMD_SQUIT   = "SQUIT";
const IRCToken IRCServerConnection::CMD_KILL    = "KILL";
#if SERVER_TYPE == SERVER_HYBRID || SERVER_TYPE == SERVER_BAHAMUT
const IRCToken IRCServerConnection::CMD_SVINFO  = "SVINFO";
#else
const IRCToken IRCServerConnection::CMD_NETINFO = "NETINFO";
#endif
const IRCToken IRCServerConnection::RPL_WHOISUSER   = "311";
const IRCToken IRCServerConnection::RPL_WHOISSERVER = "312";
const IRCToken IRCServerConnection::RPL_ENDOFWHOIS  = "318";


const IRCServerConnection::CmdHandler IRCServerConnection::servMsgHandlers[] =
{
   { CMD_NICK,    &IRCServerConnection::cmdServerNickHandler },
   { CMD_PING,    &IRCServerConnection::cmdPingHandler       },
   { ERR_433,     &IRCServerConnection::cmd433Handler        },
   { ERR_436,     &IRCServerConnection::cmd436Handler        },
   { CMD_ERROR,   &IRCServerConnection::cmdErrorHandler      },
#if SERVER_TYPE == SERVER_HYBRID || SERVER_TYPE == SERVER_BAHAMUT
   { CMD_SVINFO,  &IRCServerConnection::cmdSvinfoHandler     },
#elif SERVER_TYPE == SERVER_UNREAL
   { CMD_NETINFO, &IRCServerConnection::cmdNetinfoHandler    },
#endif
   { "",          NULL } // this line is _mandatory_
};

const IRCServerConnection::CmdHandler IRCServerConnection::clientMsgHandlers[] =
{
   { CMD_PRIVMSG, &IRCServerConnection::cmdPrivmsgHandler    },
   { CMD_NOTICE,  &IRCServerConnection::cmdNoticeHandler     },   
   { CMD_MODE,    &IRCServerConnection::cmdModeHandler       },
   { CMD_QUIT,    &IRCServerConnection::cmdQuitHandler       },
   { CMD_KILL,    &IRCServerConnection::cmdKillHandler       },
   { CMD_NICK,    &IRCServerConnection::cmdClientNickHandler },
   { CMD_WHOIS,   &IRCServerConnection::cmdWhoisHandler      },
   { ERR_433,     &IRCServerConnection::cmd433Handler        },
   { ERR_436,     &IRCServerConnection::cmd436Handler        },
   { "",          NULL } // this line is _mandatory_
};

IRCServerConnection::IRCServerConnection( const IRCToken &servname, 
					  const IRCToken &Password,
					  const IRCToken &description,
					  Poller* poller,
					  len_type inputBufLen,
					  len_type outputBufLen )
   : IRCClientConnection( poller,
			  inputBufLen,
			  outputBufLen ),
     password(Password),
     hostname(servname), 
     myDescription( description )
{
#ifdef VERBOSE_MEMORY
   cout << "IRCServerConnection] created" << endl;
#endif
};

IRCServerConnection::~IRCServerConnection()
{
#ifdef VERBOSE_MEMORY
   cout << "IRCServerConnection] destroyed" << endl;
#endif
}

void IRCServerConnection::onDoLogin()
{
   // prepare pollerclient, etc. 
   IRCClientConnection::onConnected();

   cout << "Sending login data to remote server ..." << endl;

   // BAHAMUT 

   // *********************************************************************** BAHAMUT
#if SERVER_TYPE == SERVER_BAHAMUT
   long timestamp = getUNIXTimestamp();
  
   tcp() << CMD_PASS << ' ' << password  << endl;
   tcp() << CMD_SERVER << ' ' << hostname << " 1 :" << myDescription << endl;
   tcp() << CMD_SVINFO << " 3 1 0 :" << timestamp << endl;

   // ***********************************************************************  HYBRID
#elif SERVER_TYPE == SERVER_HYBRID
   tcp() << CMD_PASS << ' ' << password << " :TS" << endl; 
   tcp() << CMD_SERVER << ' ' << hostname << " 1 :" << myDescription << endl;

   /* unused currently ( the protocol is just too ugly ;-)
   // *********************************************************************** UNDERNET
   #elif SERVER_TYPE == SERVER_UNDERNET
   long timestamp = getUNIXTimestamp();

   tcp() << CMD_PASS << ' ' << password << endl;
   tcp() << CMD_SERVER << ' ' << hostname << " 1 " << timestamp << ' ' << timestamp
   << " P09" << " :" << myDescription << endl;

   # ifdef DEBUG
   cout << CMD_PASS << ' ' << password << endl;
   cout << CMD_SERVER << ' ' << hostname << " 1 " << timestamp << ' ' << timestamp
   << " P09" << " :" << myDescription << endl;
   # endif

   // ***********************************************************************
   */

   // *********************************************************************** UNREAL
#elif SERVER_TYPE == SERVER_UNREAL
   tcp() << CMD_PASS << ' ' << password << endl;
   tcp() << CMD_SERVER << ' ' << hostname << " 1 :" << myDescription << endl;
   // ***********************************************************************

   /* invalid server type */
#else 
# error !!!BAD SERVER TYPE!!!
#endif

   flushOutput();

};

void IRCServerConnection::onDisconnected()
{
   cout << "Disconnected from server" << endl;
}

void
IRCServerConnection::onMessage( const IRCServerMessage& msg )
{
   const IRCToken& cmd=msg.getCommand();

   // print received and parsed message
#ifdef VERBOSE_PRINTING
   cout << "IRCServerConnection: msg: ";
   const IRCToken* token = msg.getToken();
   for( int i = 0; i < msg.length() ; ++i )
      cout << '\'' << *token++ << "' " ;
   cout << endl;
#endif   

   const CmdHandler* hook = msg.isClientMessage() ? 
      &clientMsgHandlers[0] : &servMsgHandlers[0] ;

   for( ; hook->handler; ++hook )
   {
      if( hook->name == cmd )
      {
	 ( this->*(hook->handler) )( msg );
	 break;
      }
   }
#ifdef DEBUG
   if( !hook->handler )
      cout << "IRCServerConnection::onMessage(): uncatched event: " 
	   << cmd << endl;
#endif
};

void IRCServerConnection::introduceUser( const IRCToken &nick,
					 const IRCToken &ident,
					 const IRCToken &address,
					 const IRCToken &modes,
					 const IRCToken &realname )
{
   long timestamp=getUNIXTimestamp();

#if SERVER_TYPE == SERVER_BAHAMUT
   tcp() << CMD_NICK << ' ' << nick << " 1 " << timestamp << ' ' << modes << ' ' 
	 << ident << ' ' << address << ' ' << hostname << " 0 :" << realname << endl;

#elif SERVER_TYPE == SERVER_HYBRID 
   tcp() << CMD_NICK << ' ' << nick << " 1 " << timestamp << ' ' << modes << ' ' 
	 << ident << ' ' << address << ' ' << hostname << " :" << realname << endl;

#elif SERVER_TYPE == SERVER_UNREAL 
   tcp() << CMD_NICK << ' ' << nick << " 1 " << timestamp << ' ' << ident 
	 << ' ' << address << ' ' << hostname << " 0 :" << realname << endl;
   printSetUserMode( nick, modes );
#endif 
}

void
IRCServerConnection::removeUser(const IRCToken &nick)
{
   tcp() << ':' << nick << ' ' << CMD_QUIT << ' ' << "terminated" << endl;
}

void
IRCServerConnection::printPong( const IRCToken& msg )
{
   tcp() << ':' << hostname << ' ' << CMD_PONG << " :"  << msg << endl;
}

void
IRCServerConnection::printMsg(const IRCToken &sender, const IRCToken &target, const IRCToken &msg)
{
   tcp() << ':' << sender << ' ' << CMD_PRIVMSG << ' ' << target << " :" << msg << endl;
}

void
IRCServerConnection::printMsg( const IRCToken &sender, const IRCToken &target, const unsigned char* format, ... )
{
   tcp() << ':' << sender << ' ' << CMD_PRIVMSG << ' ' << target << " :";

   va_list ap;
   va_start( ap, format );

   real_servprintf( format, ap );

   va_end( ap );
}

void IRCServerConnection::printNotice(const IRCToken &sender, const IRCToken &target, const IRCToken &msg)
{
   tcp() << ':' << sender << ' ' << CMD_NOTICE << ' ' << target << " :" << msg << endl;
}

void IRCServerConnection::printNotice( const IRCToken &sender, const IRCToken &target, const unsigned char *format , ... )
{
   tcp() << ':' << sender << ' ' << CMD_NOTICE << ' ' << target << " :";

   va_list ap;
   va_start( ap, format );
  
   real_servprintf( format, ap );

   va_end( ap );
}

void
IRCServerConnection::printNotice( const IRCToken& sender,
				  const IRCToken& target,
				  const char* format,
				  ... )
{
   tcp() << ':' << sender << ' ' << CMD_NOTICE << ' ' << target << " :";

   va_list ap;
   va_start( ap, format );
  
   real_servprintf( (const unsigned char*)format, ap );

   va_end( ap );  
}

// this requires an U:line in most cases
void IRCServerConnection::printSetUserMode( const IRCToken &target, const IRCToken &modes )
{
   tcp() << ':' << target << ' ' << CMD_MODE << ' ' << target << " :" << modes << endl;
}

void IRCServerConnection::joinChannel(const IRCToken &user, const IRCToken &channel)
{
#if SERVER_TYPE == SERVER_UNREAL 
   tcp() << ':' << user << ' ' << CMD_JOIN << ' ' << channel << endl;
#else 
   long ts=getUNIXTimestamp();
   tcp() << ':' << hostname << ' ' << CMD_SJOIN << ' ' << ts << ' ' << channel << " +  :" << user << endl;
#endif
}

void IRCServerConnection::printSquit( const IRCToken &nickname, const IRCToken &cause )
{
   tcp() << ':' << nickname << ' ' << CMD_SQUIT << ' ' << hostname << " :" << cause << endl;
}

void IRCServerConnection::printSquit( const IRCToken &nickname, const unsigned char* format, ... )
{
   tcp() << ':' << nickname << ' ' << CMD_SQUIT << ' ' << hostname << " :" ;

   va_list ap;
   va_start( ap, format );
  
   real_servprintf( format, ap );
  
   va_end( ap );
}

void IRCServerConnection::printKill( const IRCToken &from, const IRCToken &to, const IRCToken &cause )
{
   tcp() << ':' << from << ' ' << CMD_KILL << ' ' << to << " :" << cause << endl;
}

void IRCServerConnection::printKill( const IRCToken &from, const IRCToken &to, const unsigned char* format, ... )
{
   tcp() << ':' << from << ' ' << CMD_KILL << ' ' << to << " :";

   va_list ap;
   va_start( ap, format );
  
   real_servprintf( format, ap );
  
   va_end( ap );
}

void IRCServerConnection::printQuit( const IRCToken &from, const IRCToken &cause )
{
   tcp() << ':' << from << ' ' << CMD_QUIT << " :" << cause << endl;
}

void IRCServerConnection::printQuit( const IRCToken &from, const unsigned char* format, ... )
{
   tcp() << ':' << from << ' ' << CMD_QUIT << " :" ;

   va_list ap;
   va_start( ap, format );
  
   real_servprintf( format, ap );
  
   va_end( ap );
}

void IRCServerConnection::servprintf( const unsigned char* format, ... )
{
   va_list ap;
   va_start( ap, format );

   real_servprintf( format, ap );

   va_end( ap );

}

void IRCServerConnection::real_servprintf( const unsigned char * format, 
					   va_list &ap )
{
   int len = 0;
   const unsigned char* oldC;

   for( const unsigned char* c = oldC = format; c && *c != '\0' ; ++c ) {
      if( *c == '%' ) {
	 tcp().write( (char*)oldC, len );
	 ++c;

	 if( *c == 's' ) {
	    const IRCToken *s = va_arg( ap, const IRCToken* );
	    tcp() << *s;
	 }
	 else if( *c == 'i' || *c == 'd' ) {
	    int i = va_arg( ap, int );
	    tcp() << i;
	 }
	 else if( *c == 'c' ) {
	    int character = va_arg( ap, int );
	    tcp() << static_cast<unsigned char>( character );
	 }
	 else if( *c == 'l' ) {
	    unsigned long l = va_arg( ap, unsigned long );
	    tcp() << l;
	 }
	 else if( *c == 'B' ) {
	    tcp() << (char) 2;
	 }
	 else if( *c == 'U' ) {
	    tcp() << (char) 31;
	 }
	 else if( *c == 'b' ) {
	    const int b = (int) va_arg( ap, int );
	    tcp() << ( b ? "true" : "false" );
	 }
	 else {
	    tcp() << *c;
	 }
      
	 oldC = c+1;
	 len = 0;
      }
      else ++len ;
   }

   tcp().write( (char*)oldC, len ) << endl;
}

void
IRCServerConnection::cmdPingHandler( const IRCServerMessage& msg )
{
   printPong( msg.getMessage() );
}

void
IRCServerConnection::cmdServerNickHandler( const IRCServerMessage& msg )
{
#if SERVER_TYPE == SERVER_HYBRID || SERVER_TYPE == SERVER_BAHAMUT
   if( msg.length() > 8 )
      onNickIntroduce(msg.getTarget(), // nickname
		      msg.getToken( IRCServerMessage::IDX_OPT3 ),  // username
		      msg.getToken( IRCServerMessage::IDX_OPT4 ), // address
		      msg.getMessage() ); // realname
#elif SERVER_TYPE == SERVER_UNREAL
   if( msg.length() > 8 )
      onNickIntroduce( msg.getTarget(),
		       msg.getToken( IRCServerMessage::IDX_OPT2 ),
		       msg.getToken( IRCServerMessage::IDX_OPT3 ),
		       msg.getMessage() );
#else
# error "Invalid server type defined. Refusing compilation"
#endif 
   else
      assert( false );
}

void
IRCServerConnection::cmdClientNickHandler( const IRCServerMessage& msg )
{
   onNickChange( msg.getToken(IRCServerMessage::IDX_HOSTMASK), msg.getToken(IRCServerMessage::IDX_TARGET) );  
}

void 
IRCServerConnection::cmdPrivmsgHandler( const IRCServerMessage& msg )
{
   onPrivmsg( msg.getToken( IRCServerMessage::IDX_HOSTMASK ),
	      msg.getToken( IRCServerMessage::IDX_TARGET ),
	      msg.getMessageToken(),
	      msg.messageTokenNum() ); 
}

void
IRCServerConnection::cmdNoticeHandler( const IRCServerMessage& msg )
{
   onNotice(msg.getSender() ,
	    msg.getToken( IRCServerMessage::IDX_TARGET ),
	    msg.getMessageToken(),
	    msg.messageTokenNum() );   
}

void
IRCServerConnection::cmdModeHandler( const IRCServerMessage& msg )
{
   onMode( msg );
}

void
IRCServerConnection::cmdQuitHandler( const IRCServerMessage& msg )
{
   onQuit( msg.getToken( IRCServerMessage::IDX_HOSTMASK ),
	   msg.getMessageToken(),
	   msg.messageTokenNum() );	 
}

void
IRCServerConnection::cmdKillHandler( const IRCServerMessage& msg )
{
   onKill( msg.getToken( IRCServerMessage::IDX_HOSTMASK ),
	   msg.getToken( IRCServerMessage::IDX_TARGET ), 
	   msg.getMessageToken(),
	   msg.messageTokenNum() );
}

void
IRCServerConnection::cmd433Handler( const IRCServerMessage& msg )
{
   onNicknameInUse( msg.getToken( IRCServerMessage::IDX_TARGET ) );
}

void
IRCServerConnection::cmd436Handler( const IRCServerMessage& msg )
{
   onNicknameCollision( msg.getToken( IRCServerMessage::IDX_TARGET ) );
}

void
IRCServerConnection::cmdErrorHandler( const IRCServerMessage& msg )
{
   onServerError( msg);
}

void
IRCServerConnection::cmdWhoisHandler( const IRCServerMessage& msg )
{
   onWhois( msg.getSender(), msg.getTarget() );
}

void
#if SERVER_TYPE == SERVER_HYBRID || SERVER_TYPE == SERVER_BAHAMUT
IRCServerConnection::cmdSvinfoHandler( const IRCServerMessage& msg )
#elif SERVER_TYPE == SERVER_UNREAL
   IRCServerConnection::cmdNetinfoHandler( const IRCServerMessage& msg )
#endif
{
   onLogedin();
}
