/***************************************************************************
                           irctokenizer.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: irctokenizer.cpp,v 1.8 2002/03/19 21:09:27 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "irctoken.hpp"
#include "irctokenizer.hpp"

#include <iostream>

const IRCTokenizer::int_type  IRCTokenizer::TOK_NUM    = 15;
const IRCTokenizer::data_type IRCTokenizer::MSG_DELIM  = '\n';
const IRCTokenizer::data_type IRCTokenizer::TOK_DELIM  = ' ';
const IRCTokenizer::data_type IRCTokenizer::LTOK_DELIM = ':';

IRCTokenizer::IRCTokenizer()
   : Tokenizer( TOK_NUM, TOK_DELIM, MSG_DELIM )
{
}

IRCTokenizer::len_type
IRCTokenizer::update( reference_type buf, size_type length )
{
   
   IRCToken*             tokenIter    = m_token;
   const IRCToken* const tokenIterMax = m_token + ( m_maxTokenNum - 2 );
   reference_type        bufIter      = buf;
   const reference_type  bufIterMax   = buf+length;
   reference_type        tokenStart   = NULL;

   const data_type       whitespace   = TOK_DELIM;
   const data_type       endline      = MSG_DELIM;
   const data_type       longArgDelim = LTOK_DELIM;
   bool                  quit         = false;


   // update member
   m_tokenNum = 0;


   /**
    * tokenization is done in 4 steps
    * 1.) skip leading whitespaces.
    * 2.) iteratate through buffer until endline or whitespace 
    *     has been found.
    * 3.) build token.
    * 4.) quit if newline has been read
    * --
    * This is done as long as newline and longArgDelim have not
    * been reached and the maximum number of tokens has not been
    * reached yet.
    */
   for(; true ; ++tokenIter )
   {

      // do some first filtering
      // at this point token building 
      // has not begun. Therefore instant
      // returning  is safe.
      for( ; true; ++bufIter )
      {
	 if( bufIter == bufIterMax || *bufIter == endline )
	    return bufIter - buf;
	 else if( *bufIter != whitespace )
	    break;
      }

      if( tokenIter == tokenIterMax ) break;

      
      tokenStart = bufIter;
      

      // stop parsing whenever longarg has been read
      if( *bufIter == longArgDelim ) 
	 return bufIter - buf;


      // skip every token which is not equal to whitespace 
      // and endline
      // do all this as long as address of bufIter is smaller 
      // than the address of bufIterMax
      for( ; true ; ++bufIter )
      {
	 if( bufIter == bufIterMax )
	 {
	    quit = true;
	    break;
	 }

	 if( *bufIter == whitespace )
	    break;
	 else if( *bufIter == endline )
	 {
	    quit = true;
	    break;
	 }

      }

      // assign parsed token
      tokenIter->assign( tokenStart, bufIter - tokenStart );

      // update member
      ++m_tokenNum;

 
      // quit if newline has been read or the buffer has reached end
      if( quit ) return bufIter - buf;
   }
   /*
    * we get here whenever tokenIter == tokenIterMax;
    */

   if( *bufIter == longArgDelim )
      ++bufIter;

   tokenStart = bufIter;
   
   while( bufIter < bufIterMax && *bufIter != endline ) ++bufIter;
   
   tokenIter->assign( tokenStart, bufIter - tokenStart );

   return bufIter - buf;
}
