/***************************************************************************
                           poller.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: poller.cpp,v 1.13 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "poller.hpp"
#include <cstdio>

using std::cout;
using std::endl;

Poller::Poller( int MaxCapacity )
  : maxCapacity( MaxCapacity ),
    m_clientNum( 0 ),
    m_maxSocketIndex( 0 )
{
  pollFDs     = new PollerFD[maxCapacity];
  pollSockets = new Pollable*[maxCapacity];
}

Poller::~Poller()
{
   removeAllClients();

   delete[] pollFDs;
   delete[] pollSockets;
}

int
Poller::runSyscallPoll( int timeout )
{
  if( m_clientNum == 0 ) return -2;

  int pollStatus = poll( pollFDs, m_maxSocketIndex, timeout );
  
  if( pollStatus > 0 )
     onPolling( pollStatus );

  return pollStatus;
}

PollerFD*
Poller::addPollableClient( Pollable* client )
{
   if( isFull() )
   {
#ifdef DEBUG
      cout << "Poller::addPollableClient(): could not add client. Queue is full" << endl;
#endif
      return NULL;
   }

   Pollable** socket = pollSockets;

   // because we have called isFull() we do not have to check insertIndex < maxCapacity
    int insertIndex = 0;
   for( ; socket[insertIndex] != NULL; ++insertIndex );

    ++m_clientNum;
   
    pollFDs[insertIndex]     = client;
    pollSockets[insertIndex] = client;

    m_maxSocketIndex = m_maxSocketIndex == insertIndex ? insertIndex + 1 : m_maxSocketIndex;


#ifdef DEBUG
    cout << "Poller::addPollableClient(): added client at " << insertIndex << endl;
#endif


    return &pollFDs[insertIndex];
}

bool
Poller::removePollableClient( const Pollable* client )
{
   if( client == NULL )
      return false;

   for(  int i = 0; i < m_maxSocketIndex; ++i )
   {
      if( pollSockets[i] == client )
      {
	 unregisterClient( i );

	 --m_clientNum;

	 if( ( i + 1 ) == m_maxSocketIndex )
	    // element at the highest position has been removed. 
	    // find new max index
	    findNewMaxSocketIndex();

#ifdef DEBUG
	 cout << "Poller: removed client from position: " << i << endl;
#endif
	 return true;
      }
   }
   
   return false;
}

void
Poller::removeAllClients()
{
   for(  int i = m_maxSocketIndex-1; m_clientNum && i > -1; --i )
      if( pollSockets[i] != NULL )
      {
	 --m_clientNum;

	 unregisterClient( i );
      }
}

void
Poller::onPolling( int num )
{
   for(  int i = 0 ; num > 0 && i < m_maxSocketIndex ; ++i )
   {
      int revents = pollFDs[i].getREvents();
      if( revents != 0 ) 
      {
	 if( revents & ( POLLIN | POLLPRI ) )
	    pollSockets[i]->onInput();
	 else if( revents & POLLOUT )
	    pollSockets[i]->onOutput();
	 else if( revents & ( POLLERR | POLLNVAL ) )
	    pollSockets[i]->onError();
	 else if( revents & POLLHUP )
	    pollSockets[i]->onHangup();
	 else continue;

	 --num;
      }
   }
}

void
Poller::findNewMaxSocketIndex()
{
   for( --m_maxSocketIndex; m_maxSocketIndex > 0 && pollSockets[m_maxSocketIndex-1] == NULL; ) --m_maxSocketIndex;
}

void
Poller::unregisterClient( int index )
{
   pollSockets[index]->onRemoved();
   pollSockets[index] = NULL;

   pollFDs[index].fd = -1;
   pollFDs[index].unregisterAllEvents();
}
