/***************************************************************************
                           pollerclient.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: pollerclient.cpp,v 1.12 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cassert>
#include <sys/time.h>
#include "pollerclient.hpp"

using std::cout;
using std::endl;

PollerClient::PollerClient( Pollable* client )
  : m_poller( NULL ),
    m_socket( client ),
    m_pollerFD( NULL )
{
}

PollerClient::PollerClient( Poller* poller, Pollable* client,
			    bool autoRegister )
   : m_poller( poller ),
     m_socket( client ),
     m_pollerFD( NULL )
{
   if( autoRegister )
      registerClient();
}

bool PollerClient::registerClient()
{
   if( m_poller == NULL )
      return false;

   m_pollerFD = m_poller->addPollableClient( const_cast<PollerClient*>( this ) );

   if( m_pollerFD == NULL )
   {
      onRemoved();
      return false;
   }

   return true;
}

bool PollerClient::registerClient( Poller* poller )
{
   m_poller = poller;

   return registerClient();
}

void PollerClient::unregister()
{
#ifdef DEBUG
   cout << "PollerClient::unregister()" << endl;
#endif

   if( m_pollerFD )
   {
      // removePollableClient will also call this->onRemoved()
      m_poller->removePollableClient( this );
      m_pollerFD = NULL;
   }
}

MeasuringPollerClient::MeasuringPollerClient( Poller* poller,
					      Pollable* socket,
					      bool autoregister )
   : PollerClient( poller, socket, autoregister ),
     m_lastIOEvent()
{
   updateLastIOEvent();
}

void
MeasuringPollerClient::onInput()
{
   updateLastIOEvent();
   PollerClient::onInput();
}

void
MeasuringPollerClient::onOutput()
{
   updateLastIOEvent();
   PollerClient::onOutput();
}

void
MeasuringPollerClient::updateLastIOEvent()
{
   m_lastIOEvent = time( NULL );

   // this function is of highest importance for the whole class
   // if 'time()' fails this object is rather useless
   assert( m_lastIOEvent != -1 );
}
