#include "request.hpp"
#include "config.h"

/***************************************************************************
                           request.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: request.cpp,v 1.11 2002/05/21 19:55:44 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


namespace httpd
{

const Request::HeaderTab Request::MESSAGEHEADERTAB[] = 
{
  { http::REQ_HOST,           &Request::setHost,           NULL },
  { http::REQ_USER_AGENT,     &Request::setUserAgent,      NULL },
  { http::REQ_REFERER,        &Request::setReferer,        NULL },
  { http::GENERAL_CONNECTION, &Request::setConnectionType, NULL },
  { http::ENT_CONTENT_TYPE,   &Request::setContentType,    NULL },
  { http::REQ_CONTENT_TYPE,   &Request::setContentType,    NULL },
  { "",                       NULL,                        NULL }
};

Request::Request( const HTTPClientConnection* from, bool requiresBody )
   : m_from( from ),
     m_requiresBody( requiresBody ),
     m_requestStartline(),
     m_referer(),
     m_host(),
     m_userAgent(),
     m_connectionType( http::HTTPCONN_UNKNOWN )
{
#ifdef VERBOSE_MEMORY
  cout << "Request] created" << endl;
#endif
}

bool
Request::appendMessageHeader( const http::MessageHeader& header )
{
   using http::HTTPString;

   const HeaderTab*  iter      = &MESSAGEHEADERTAB[0];
   const bool        valueSet  = header.fieldValueSet();
   const HTTPString& fieldName = header.getFieldName();

   while( iter->valueHandler )
   {
      if( iter->fieldName == fieldName )
      {
	 if( valueSet && iter->valueHandler ) 
	    (this->*iter->valueHandler)( header.getFieldValue() );
	 else if( iter->valueOmitHandler )
	    (this->*iter->valueOmitHandler)();

	 return true;
      }

      ++iter;
   }

#ifdef DEBUG
   cout << "RequestappendMessageHeader(): warning: uncatched request-header: " << fieldName << endl;
#endif

   return false;
}

void
Request::setStartline( const http::RequestStartline& startline )
{
   m_requestStartline = startline; 

   if( m_requestStartline.getProtocolVersion() == http::HTTPVER_110 )
      m_connectionType = http::HTTPCONN_CHUNKED;
   else
      m_connectionType = http::HTTPCONN_CLOSE;
}

template <class Req>
Request* _makeRequest( const HTTPClientConnection* from )
{
   return new Req( from );
}

typedef Request* (*makeRequestFunc)( const HTTPClientConnection* conn );

static const makeRequestFunc REQUEST_MAKER_TAB[] =
{
   &_makeRequest<OPTIONSRequest>,
   &_makeRequest<GETRequest>,
   &_makeRequest<HEADRequest>,
   &_makeRequest<POSTRequest>,
   &_makeRequest<PUTRequest>,
   &_makeRequest<DELETERequest>,
   &_makeRequest<TRACERequest>,
   &_makeRequest<CONNECTRequest>,
   &_makeRequest<UnknownRequest>
};

Request*
make_request( http::httpmethod_type method, const HTTPClientConnection* from )
{
   return (*REQUEST_MAKER_TAB[method])( from );
}

RequestBody*
POSTRequest::getRequestBody()
{
   // create only one requestBody instance

   if( m_requestBody == NULL )
   {
      if( http::MessageHeader::PlainTextContentType.getFieldValue() == m_contentType )
	 m_requestBody = new PlainTextPOSTRequestBody();

#ifdef DEBUG
      else
	 cout << "POSTRequest::getRequestBody(): warning. Unknown Content-Type: " << m_contentType << endl;
#endif

   }

   return m_requestBody;
}

}//EON http
