/***************************************************************************
                           requestbody.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: requestbody.cpp,v 1.2 2002/05/25 16:15:36 d3cline Exp $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <cstring> /* for memcpy */
#include "requestbody.hpp"
#include "config.h"

using std::cout;
using std::endl;

namespace httpd
{

RequestBodyImpl::RequestBodyImpl( len_type bufLen )
   : m_inputBuffer( new HTTPTraits::char_type[bufLen] ),
     m_inBufLen( bufLen )
{
}

RequestBodyImpl::len_type
RequestBodyImpl::read( FileInBuf& f )
{
   File::len_type readLen = f.read( m_inputBuffer, m_inBufLen );
   if( readLen < 1 )
      // signalize error
      return -1;
   else
      return addData( this->m_inputBuffer, readLen );
}

LineRequestBody::LineRequestBody()
   : m_lineBuffer( new HTTPTraits::char_type[MAX_LINELENGTH] ),
     m_usedLen( 0 )
{
}

LineRequestBody::len_type
LineRequestBody::read( FileInBuf& f )
{
   const bool wasFull = f.isFull();

   len_type readLen = f.read( m_lineBuffer+m_usedLen, MAX_LINELENGTH - m_usedLen );

   if( readLen < 1 )
      return -1;
   else
      return !onInput( m_lineBuffer, m_usedLen + readLen ) ? -1 : wasFull ? readLen : 0;
}

bool
LineRequestBody::onInput( HTTPTraits::char_type* data, int length )
{
   HTTPTraits::char_type*       iter      = data;
   HTTPTraits::char_type*       lineBegin = iter;
   HTTPTraits::char_type* const iterEnd   = iter + length;
   unsigned int                 readLines = 0;  
   

   for( ; iter < iterEnd; ++iter )
   {
      if( *iter == '\n' )
      {
	 int lineLen = iter - lineBegin;

	 if( *( iter -1 ) == '\r' )
	    --lineLen;

	 onReadLine( lineBegin, lineLen );

	 // clear `incomplete line in buffer flag'
	 m_usedLen = 0;

	 // update line begin
	 lineBegin = iter + 1;

	 ++readLines;
      }
   }

   if( *( iter-1 ) != '\n' )
   {
      /*
       * received incomplete line
       */
      this->m_usedLen = iter - lineBegin;

      // move memory to the front of buffer
      ::memmove( this->m_lineBuffer, lineBegin, iter - lineBegin );
   }
   
   /*
    * netscape does not provide any content-length header. therefore we have to guess
    * when the form data is complete. 
    */
   return readLines != 0;
}

void
PlainTextPOSTRequestBody::onReadLine( const HTTPTraits::char_type* rawData, int length )
{
   using http::HTTPString;

   // find '=' in data
   const HTTPTraits::char_type*       iter    = rawData;
   const HTTPTraits::char_type* const iterEnd = iter + length;
   const HTTPTraits::char_type*       lTokEnd = NULL; // shall point to 'PlainTextPostMethodRequestBody::DIV' after looping
   
   for( ; iter < iterEnd; ++iter )
   {
      if( *iter == this->DIV )
      {
	    lTokEnd = iter;
	    break;
      }
   }

   if( lTokEnd == NULL )
   {
      /*
       * no '=' in string.
       */
      HTTPString key( rawData, length );

#ifdef DEBUG
      cout << "PlainTextPostMethodRequestBody::onReadLine(): key = '" << key << '\'' << endl;
#endif      

      this->m_dataMap.insert( make_pair( key, HTTPString() ) );
   }
   else
   {
      HTTPString key( rawData, lTokEnd - rawData );
      HTTPString val( lTokEnd + 1, iterEnd - lTokEnd - 1 );

#ifdef DEBUG
      cout << "PlainTextPostMethodRequestBody::onReadLine(): key = '" << key << "' val = '" << val << '\'' << endl;
#endif
      
      this->m_dataMap.insert( make_pair( key, val ) );
   }
}

} //eon httpd
