/***************************************************************************
                           requeststartline.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: requeststartline.cpp,v 1.8 2002/05/21 19:55:44 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "requeststartline.hpp"

namespace http
{

const RequestStartline::StringMethod RequestStartline::STRING_TO_METHOD_TAB[] =
{
   { "OPTIONS", REQ_METHOD_OPTIONS },
   { "GET",     REQ_METHOD_GET     },
   { "HEAD",    REQ_METHOD_HEAD    },
   { "POST",    REQ_METHOD_POST    },
   { "PUT",     REQ_METHOD_PUT     },
   { "DELETE",  REQ_METHOD_DELETE  },
   { "TRACE",   REQ_METHOD_TRACE   },
   { "CONNECT", REQ_METHOD_CONNECT },
   { "",        REQ_METHOD_UNKNOWN }
};

const RequestStartline::StringProt RequestStartline::STRING_TO_PROT_TAB[] =
{
   { "HTTP/1.1",  HTTPVER_110     },
   { "HTTP/1.0",  HTTPVER_100     },
   { "",          HTTPVER_UNKNOWN }
};

httpprotversion_type RequestStartline::m_defaultProtVersion = HTTPVER_UNKNOWN;

RequestStartline::RequestStartline()
  : m_method( REQ_METHOD_UNKNOWN ),
    m_url(),
    m_protocolVersion( HTTPVER_UNKNOWN )
{
}

RequestStartline::RequestStartline( const HTTPString& method,
				    const HTTPString& uri )
   : m_method( stringToMethod( method ) ),
     m_url( uri ),
     m_protocolVersion( m_defaultProtVersion )
{
}

RequestStartline::RequestStartline( const HTTPString& method,
				    const HTTPString& uri,
				    const HTTPString& version )
  : m_method( stringToMethod( method ) ),
    m_url( uri ),
    m_protocolVersion( stringToProtVersion( version ) )
{
}

RequestStartline::RequestStartline( httpmethod_type method,
				    const uri::URL uri,
				    httpprotversion_type version )
   : m_method( method ),
     m_url( uri ),
     m_protocolVersion( version )
{
}

RequestStartline::RequestStartline( const RequestStartline& rsl )
   : m_method( rsl.m_method ),
     m_url( rsl.m_url ),
     m_protocolVersion( rsl.m_protocolVersion )
{
}

const RequestStartline&
RequestStartline::operator=( const RequestStartline& rsl )
{
   if( &rsl != this )
   {
      m_method          = rsl.m_method;
      m_url             = rsl.m_url;
      m_protocolVersion = rsl.m_protocolVersion;
   }

   return *this;
}

void
RequestStartline::reset()
{
  m_method  = REQ_METHOD_UNKNOWN;
  m_protocolVersion = m_defaultProtVersion;
}

void
RequestStartline::update( const HTTPString& method,
			  const HTTPString& uri,
			  const HTTPString& version )
{
   m_method = stringToMethod( method );
   m_url    = uri;
   m_protocolVersion = stringToProtVersion( version );
}

void
RequestStartline::update( const HTTPString& method,
			  const HTTPString& uri )
{
   m_method = stringToMethod( method );
   m_url    = uri;
   m_protocolVersion = m_defaultProtVersion;
}

httpprotversion_type
RequestStartline::stringToProtVersion( const HTTPString& version )
{
   for( const StringProt* iter = &STRING_TO_PROT_TAB[0]; 
	;
	++iter )
   {
      if( iter->m_protString == version ||
	  iter->m_protString.empty() )
	 return iter->m_protVal; 
   }

   return HTTPVER_UNKNOWN;
}

httpmethod_type
RequestStartline::stringToMethod( const HTTPString& method )
{
   for( const StringMethod* iter = &STRING_TO_METHOD_TAB[0];
	;
	++iter )
   {
      if( iter->m_methodString == method ||
	  iter->m_methodString.empty() )
	 return iter->m_methodVal; 
   }
   
   return REQ_METHOD_UNKNOWN;
}

} //EON http

