/***************************************************************************
                           reponse.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: response.cpp,v 1.10 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "response.hpp"
#include "request.hpp"

namespace httpd
{
Response::Response( http::ResponseStatusline::responsecode_type responseCode,
		    http::httpprotversion_type protocolVersion,
		    const Request* const request )
   : m_messageHeaders(),
     m_statusLine( protocolVersion, responseCode ),
     m_responseBody( NULL ),
     m_request( request ),
     m_bodyEncoding( http::HTTPCONN_CLOSE ),
     m_connectionType( request->getConnectionType() )
{
   if( m_connectionType == http::HTTPCONN_UNKNOWN )
   {
      if( request->getRequestStartline().getProtocolVersion() == http::HTTPVER_110 )
	 m_connectionType = http::HTTPCONN_KEEPALIVE;
      else
	 m_connectionType = http::HTTPCONN_CLOSE;
   }

#ifdef VERBOSE_MEMORY
   cout << "Response] created" << endl;
#endif
}

typedef Response* (responseMakerFunc)( http::ResponseStatusline::responsecode_type respCode,
                                       http::httpprotversion_type procotolVersion,
				       const Request* const request );

Response* makeResponseGeneric( http::ResponseStatusline::responsecode_type respCode, 
                               http::httpprotversion_type protVersion,
			       const Request* const request )
{
   return new Response( respCode, protVersion, request );
} 

static responseMakerFunc* RESPONSEMAKER_FUNC_TAB[] =
{
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric,
   &makeResponseGeneric
};

Response* make_response( http::ResponseStatusline::responsecode_type respCode,
                         http::httpprotversion_type protocolVersion,
			 const Request* const request )
{  
   return (*RESPONSEMAKER_FUNC_TAB[respCode])( respCode, protocolVersion, request );
}

}//EON httpd
