#include "responsebody.hpp"
#include "response.hpp"
#include "util.hpp"
#include "request.hpp"
#include "httpd.hpp"
#include "hsclientlist.hpp"
#include "hsircclientlist.hpp"
#include "vhostlist.hpp"

namespace httpd
{

const DynamicFileResponseBody::printfunc_type DynamicFileResponseBody::PRINTFUNC_TAB[] =
{
   &DynamicFileResponseBody::printServerName,                     // 00
   &DynamicFileResponseBody::printServerDescription,              // 01
   &DynamicFileResponseBody::printRemoteHost,                     // 02
   &DynamicFileResponseBody::printRemotePassword,                 // 03
   &DynamicFileResponseBody::printHostservNickname,               // 04
   &DynamicFileResponseBody::printHostservUsername,               // 05
   &DynamicFileResponseBody::printHostservAddress,                // 06
   &DynamicFileResponseBody::printHostservRealname,               // 07
   &DynamicFileResponseBody::printHostservModes,                  // 08
   &DynamicFileResponseBody::printHostservChannel,                // 09
   &DynamicFileResponseBody::printClientDefaultPermissions,       // 0A
   &DynamicFileResponseBody::printVHostPublicPolicy,              // 0B
   &DynamicFileResponseBody::printHostservConfigFilename,         // 0C
   &DynamicFileResponseBody::printVHostsConfigFilename,           // 0D
   &DynamicFileResponseBody::printUsersConfigfilename,            // 0E
   &DynamicFileResponseBody::printHTTPdDocumentRoot,              // 0F
   &DynamicFileResponseBody::printHTTPdDefaultDocument,           // 10
   &DynamicFileResponseBody::printHTTPdBindInterface,             // 11
   &DynamicFileResponseBody::printHTTPdPort,                      // 12
   &DynamicFileResponseBody::printHTTPdMaxClientTimeout,          // 13
   &DynamicFileResponseBody::printHTTPdMaxClientNum,              // 14
   &DynamicFileResponseBody::printIgnore,                         // 15
   &DynamicFileResponseBody::printIgnore,                         // 16
   &DynamicFileResponseBody::printIgnore,                         // 17
   &DynamicFileResponseBody::printIgnore,                         // 18
   &DynamicFileResponseBody::printIgnore,                         // 19
   &DynamicFileResponseBody::printIgnore,                         // 1A
   &DynamicFileResponseBody::printIgnore,                         // 1B
   &DynamicFileResponseBody::printIgnore,                         // 1C
   &DynamicFileResponseBody::printIgnore,                         // 1D
   &DynamicFileResponseBody::printIgnore,                         // 1E
   &DynamicFileResponseBody::printIgnore,                         // 1F
   &DynamicFileResponseBody::printIgnore,                         // 20
   &DynamicFileResponseBody::printIgnore,                         // 21
   &DynamicFileResponseBody::printIgnore,                         // 22
   &DynamicFileResponseBody::printIgnore,                         // 23
   &DynamicFileResponseBody::printIgnore,                         // 24
   &DynamicFileResponseBody::printIgnore,                         // 25
   &DynamicFileResponseBody::printIgnore,                         // 26
   &DynamicFileResponseBody::printIgnore,                         // 27
   &DynamicFileResponseBody::printIgnore,                         // 28
   &DynamicFileResponseBody::printIgnore,                         // 29
   &DynamicFileResponseBody::printIgnore,                         // 2A
   &DynamicFileResponseBody::printIgnore,                         // 2B
   &DynamicFileResponseBody::printIgnore,                         // 2C
   &DynamicFileResponseBody::printIgnore,                         // 2D
   &DynamicFileResponseBody::printIgnore,                         // 2E
   &DynamicFileResponseBody::printIgnore,                         // 2F
   &DynamicFileResponseBody::printIRCClientNum,                   // 30
   &DynamicFileResponseBody::printHSClientNum,                    // 31
   &DynamicFileResponseBody::printVHostNum                        // 32
};

DynamicFileResponseBody::DynamicFileResponseBody( const HSConfiguration* config,
						  const HSIRCClientList* ircClients,
						  const HSClientList* hsClients,
						  const VHostList* vhosts,
						  const http::HTTPString& filename )
   : FileResponseBody( filename ),
     m_streamBuffer( new HTTPTraits::char_type[ ResponseBody::getMaxChunkSize() ] ),
     m_sout( (char*)m_streamBuffer, ResponseBody::getMaxChunkSize() ),
     // :-)
     m_configuration( config ),
     m_ircclients( ircClients ),
     m_hsclients( hsClients ),
     m_vhosts( vhosts ),
     m_lastReadPos( NULL ),
     m_lastReadMax( NULL ),
     m_lastCharOutquote( false )
{   
}

int
DynamicFileResponseBody::printJunk( ostream& out )
{
   if( !out )
      return 0;

   // points everytime to the 'first' character which is not `FORMAT_PREFIX'
   // we use this pointer to make the loop below
   // only write data into m_streamBuffer whenever
   // a special character has been read
   HTTPTraits::char_type* lastNormChar = NULL;
   
   // will get set below every time when we try to copy static data into 
   // writebuffer which has not got enough space left
   bool printingIncomplete = false;

   // will get set before static data gets written
   int  pcountBefore       = 0;

   while( true )
   {
      if( m_lastReadPos )
	 lastNormChar = m_lastReadPos;
      else
      {
	 /*
	  * we get here whenever the whole 
	  * buffer has been processed completely
	  */
	 const int readLen = FileResponseBody::fillBuffer();
	 if( readLen == 0 )
	    /*
	     * we cannot just return here because it may be that we have looped one or
	     * more times (and therefore the buffer has been filled with data which is 
	     * about to get printed)
	     */
	    goto end;

	 m_lastReadPos = m_buffer;
	 lastNormChar  = m_lastReadPos;
	 m_lastReadMax = m_buffer + readLen;
      }

      /*
       * here we process the content in m_buffer
       */


      for( ; m_lastReadPos < m_lastReadMax; ++m_lastReadPos )
      {
 	 if( m_lastCharOutquote )
	    // permit interpretation of characters after FORMAT_OUTQUOTE was read
	    m_lastCharOutquote = false;

	 else if( *m_lastReadPos == FORMAT_PREFIX )
	 {
	    /*
	     * make dynamic content
	     */

	    pcountBefore = m_sout.pcount();

	    // first step: write static data into output buffer
	    int writeLen = m_lastReadPos - lastNormChar;
	    
	    m_sout.write( (char*)lastNormChar, writeLen );
	    if( !m_sout )
	    {
	       printingIncomplete = true;
	       goto end;
	    }


	    const int readMax = m_lastReadMax - m_lastReadPos;

	    if( readMax < 3 )
	       // bad situation here:
	       // the buffer contains less than 3 characters
	       //
	       // we will have to implement a parametrised version 
	       // of fillBuffer() (*hint*) or due to the outer loop
	       // we will loop infinitely
	       goto end;

	    pcountBefore = m_sout.pcount();

	    // increment m_lastReadPos as it points to 'FORMAT_PREFIX'
	    print( util::hexStringToNum( ++m_lastReadPos, 2 ) );

	    if( !m_sout )
	    {
	       /*
		* print error. output buffer not great enough ... :/
		*/

	       if( pcountBefore == 0 )
		  // if pcountBefore == 0 then printing will fail forever
		  assert( false );

	       m_sout.seekp( pcountBefore );


	       // dirty line:
	       // m_lastReadPos has been increment -> m_lastReadPos - lastNormChar == 1
	       // -> bytesShould = m_lastReadPos - lastNormChar = 1;
	       // means:
	       // 'm_lastReadPos -= bytesShould - ( bytesWritten - pcountBefore )' will 
	       // decrement m_lastReadPos only. m_lastReadPos will point to FORMAT_PREFIX.
	       // the next time printJunk() gets called. therefore we will in this branch

	       pcountBefore       = m_sout.pcount();
	       printingIncomplete = true;
	       lastNormChar       = m_lastReadPos-1;
	       goto end;
	    }

	    m_lastReadPos += 2;
	    lastNormChar   = m_lastReadPos;
	 }
	 else if( *m_lastReadPos == FORMAT_OUTQUOTE )
	    m_lastCharOutquote = true;
      } // EOfor



      pcountBefore = m_sout.pcount();

      const int writeLen = m_lastReadPos - lastNormChar;
      m_sout.write( (char*)lastNormChar, writeLen );

      if( !m_sout )
      {
	 printingIncomplete = true;
	 goto end;
      }

      /*
       * we get here whenever we have printed the complete input buffer and did not touch
       * output buffer boundary
       */
      m_lastReadPos = NULL;

   }//EOWhile

   /*
    * do the actual transmitting
    */
  end:
   const int bytesWritten = m_sout.pcount();

   // m_sout acts upon m_streamBuffer
   out.write( (char*)m_streamBuffer, bytesWritten );


   if( printingIncomplete )
   {
      /*
       * we get here whenever writing of static data
       * failed. therefore we may reposition read pointer
       * without modifiying the output result
       * 
       * _WARNING_: whenever you change this code segment 
       *            you have to change `if( !m_sout )' in 
       *            (*m_lastReadPos == FORMAT_PREFIX ) branch
       */

      // make m_lastReadPos point to the last transmitted
      // data. 
      const int bytesShould  = m_lastReadPos - lastNormChar;
      m_lastReadPos         -= bytesShould - ( bytesWritten - pcountBefore );
   }
   else
      // we may assume all the data in input buffer has been printed successfully
      // therefore assign 'NULL' to m_lastReadPos to make this function refill
      // the buffer next it gets called
      m_lastReadPos = NULL;

   // reset all flags ( -> buffer overflow, etc. )
   m_sout.clear();
   // and make the buffer look empty
   m_sout.seekp( 0 );

#ifdef DEBUG
   cout << "DynamicFileResponseBody::printJunk(): bytes written: " << bytesWritten << endl;
#endif

   return bytesWritten;
}

void
DynamicFileResponseBody::print( int num )
{
   const size_t numT = static_cast<size_t>( num );

   if( numT > ( sizeof( PRINTFUNC_TAB ) / sizeof( printfunc_type ) ) || numT < 0 )
   {
      cout << "Warning: illegal format opcode: " << num << endl;
      return;
   }

   (this->*PRINTFUNC_TAB[num])();
}

void
DynamicFileResponseBody::printIRCClientNum()
{ m_sout << m_ircclients->size(); }

void
DynamicFileResponseBody::printHSClientNum()
{ m_sout << m_hsclients->size(); }

void
DynamicFileResponseBody::printVHostNum()
{ m_sout << m_vhosts->size(); }
  
}//EON httpd


