/***************************************************************************
                           socket.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: socket.cpp,v 1.5 2002/05/25 16:15:36 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <iostream>
#include "socket.hpp"
extern "C" {
#include <cstdio>
}


#define DEBUG

using std::cout;
using std::endl;
using std::cerr;

int TCPPort::DEF_BACKLOG = 5;


TCPSocket::TCPSocket()
{
  fd = createSocket();
}

int
TCPSocket::createSocket()
{
  fd = socket( PF_INET, SOCK_STREAM, 6 );
  if( fd < 0 ) throw this;
  return fd;
}

TCPPort::TCPPort( const InetHostAddress &addr,
		  TCPSocket::port_type portNum,
		  int backlog )
{
  setupSocket( addr.getHostent(), portNum, backlog );
}

void 
TCPPort::listen( )
{
  sockaddr_in *clientAddrIn = NULL;
  socklen_t clientAddrInLen;

  int clientFD = accept( fd, 
			 (sockaddr*) clientAddrIn,
			 &clientAddrInLen );
  if( clientFD == -1 ) {
#ifdef DEBUG
    cerr << "ERROR -> on accept" << endl;
    perror( "socketlistener" );
#endif
  }
  else {
    onConnected( clientFD,
		 *clientAddrIn, 
		   clientAddrInLen );
  }
}

void
TCPPort::setupSocket( const hostent * const he, 
		      TCPSocket::port_type port, int backlog )
{
  cout << "Setting up socket" << endl;
  sockaddr_in sockIn;
  memcpy( &sockIn.sin_addr.s_addr, he->h_addr, he->h_length );
  sockIn.sin_family = AF_INET;
  sockIn.sin_port=htons( port );

  if( ::bind( fd, (sockaddr*) &sockIn, sizeof( sockIn ) ) < 0 ) {
    perror( "bind()");
    throw( this );
  }

  if( ::listen( fd, backlog ) == -1 ) {
    perror( "listen()" );
    throw( this );
  }
}

