/***************************************************************************
                           tcpclientconnection.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: tcpclientconnection.cpp,v 1.12 2002/05/23 15:02:17 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tcpclientconnection.hpp"

TCPClientConnection::len_type
TCPClientConnection::connect( const InetHostAddress& addr,
			      TCPSocket::port_type port )
{
   const hostent *he=addr.getHostent();
   if( !he ) throw this;
   
   sockaddr_in sockIn;
   memcpy( &sockIn.sin_addr.s_addr, he->h_addr, he->h_length );
   sockIn.sin_family = AF_INET;
   sockIn.sin_port=htons( port );

   // call hook
   onConnecting();
   
   if( ::connect( fd, (sockaddr*)(&sockIn), sizeof(sockIn)) == -1 )
   { // connect error
      close();
      return File::eof;
   } 
    
   // call hook
   onConnected();

   return (len_type) 1;
}

TCPClientConnection::len_type
TCPClientConnection::read( data_reference buf,
			   len_type len )
{
   len_type rxLen = ::recv( File::getFD(),
			    buf,
			    len,
			    MSG_NOSIGNAL | MSG_DONTWAIT );

   // read error / remote hangup
   if( rxLen < 1 ) return File::eof;
   
   return rxLen;
}

TCPClientConnection::len_type
TCPClientConnection::write( const data_reference buf,
			    len_type len )
{
   len_type rxLen = ::write( File::getFD(),
			     buf,
			     len );
   
   // write error / remote hangup
   if( rxLen < 0 ) 
      return File::eof;
   
   return rxLen;
}

void
TCPClientConnection::close()
{
   // call hook
   onClosing();
   
   TCPSocket::close();
   
   fd = -1;
   
   // call hook
   onClosed();
  
}
