/***************************************************************************
                           tcpconnection.cpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: tcpconnection.cpp,v 1.8 2002/03/09 12:49:34 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <tcpconnection.hpp>
extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <sys/poll.h>
}

using namespace std;

// todo:
//   implement checks for disconnect status in recv, peek, etc. methods
//

TCPConnection::TCPConnection()
  : sd(-1), runPoll(false), tcpStream(NULL), addr(), port(0)
{
#ifdef VERBOSE_MEMORY
  cout << "TCPConnection] created" << endl;
#endif
}

TCPConnection::TCPConnection(const PSInetHostAddress &Addr, tpport_t Port)
  : sd(-1), runPoll(false), tcpStream(NULL), addr(Addr), port(Port)
{
#ifdef VERBOSE_MEMORY
  cout << "TCPConnection] created" << endl;
#endif
}

TCPConnection::~TCPConnection()
{
  Close();
#ifdef VERBOSE_MEMORY
  cout << "TCPConnection] destroyed" << endl;
#endif
}

int TCPConnection::Recv(char *buf, size_t len)
{
  if( sd == -1 ) throw this;
  return recv( sd, buf, len, 0 );
}

int TCPConnection::Peek(char *buf, size_t len)
{
  if( sd == -1 ) throw this;
  return recv( sd, buf, len, MSG_PEEK );
}

ssize_t TCPConnection::Write(const char *buf, size_t len)
{
  if(sd==-1) throw this;
  ssize_t s = write( sd, buf, len );
  return s;
}

void TCPConnection::Close()
{
  if( sd!=-1 ) {
    delete tcpStream;
    close( sd );
    sd = -1;
  }
}

void TCPConnection::Connect()
{
  if( addr && port!=0) Connect( addr, port );
}

void TCPConnection::Connect(const PSInetHostAddress &Addr, tpport_t Port)
{
  if( tcpStream != NULL ) {
    cerr << "TCPConnection] ERROR. Please report this bug." << endl
	 << "TCPConnection] tried to reinitiate connection with host without having disconnected." << endl;
    exit( 1 );
  }

  if( &Addr != &addr ) addr=Addr;
  port=Port;

  const hostent *he=Addr.getHostent();
  if( !he ) throw this;

  sockaddr_in sockIn;
  memcpy( &sockIn.sin_addr.s_addr, he->h_addr, he->h_length );
  sockIn.sin_family = AF_INET;
  sockIn.sin_port=htons( port );

  if( (sd= socket( PF_INET, SOCK_STREAM, 6)) == -1 ) throw this; 
  if( -1 == connect( sd, (sockaddr*)(&sockIn), sizeof(sockIn)) ) throw this;  

  tcpStream=new spofstream( sd );
  
  onConnect();
}

void TCPConnection::startPolling()
{
#ifdef DEBUG
  cout << "TCPConnection] startPolling called" << endl;
#endif
  if( sd==-1 ) throw this;

  if( runPoll ) {
    cout << "Bug: startPolling() called twice" << endl;
  }

  pollfd *sfd=new pollfd();  
  sfd->fd=sd;
  sfd->events = POLLIN;
  
  int status=-1;
  for( runPoll=true; runPoll && (status=poll(sfd, 1, -1)) != -1; ) {
    if( status == 0 ) onTimeout();
    else onPolling();
  }
  Close();  
  delete sfd;
  runPoll = false;

  onDisconnected();
}
