#include "socket.hpp"
#include "poller.hpp"
#include "pollerclient.hpp"

class MyTCPPort : public TCPPort, public Pollable
{
public:
  MyTCPPort( Poller* poller, const InetHostAddress& addr, port_type port );

  virtual len_type write( const data_reference,
			  len_type len )
  { return 0; }
  virtual len_type read( data_reference, 
			 len_type )
  { return 0; }
  
  virtual const File* getPollableFile() const
  { return this; }
  
protected:
  virtual void onInput();
  virtual void onOutput()
  {}
  virtual void onError()
  { delete this; }
  virtual void onHangup()
  { delete this; }
  virtual void onRemoved()
  { delete this; }
  virtual void onConnected( int sockFD,
			    const sockaddr_in& from,
			    socklen_t fromLen );
private:
  Poller* m_poller;
};

class MyTCPSocket : public TCPSocket, public Pollable
{
public:
  MyTCPSocket( Poller* poller, fd_type fd );
  
  virtual ~MyTCPSocket()
  { delete [] m_buf; }


  virtual len_type read( data_reference buf,
			 len_type len )
  {  
    len_type rxLen = ::recv( File::getFD(),
			     buf,
			     len,
			     MSG_NOSIGNAL | MSG_DONTWAIT );
    
    // read error / remote hangup
    if( rxLen < 0 ) return File::eof;
    
    return rxLen;
  }
  virtual len_type write( const data_reference,
			  len_type len )
  { return 0; }
  
  virtual const File* getPollableFile() const 
  { return this; }

protected:
  virtual void onInput();
  virtual void onOutput()
  {}
  virtual void onError()
  { delete this; }
  virtual void onHangup()
  { delete this; }
  virtual void onRemoved()
  { delete this; }

private:
  data_type* m_buf;
      PollerClient* m_pollerClient;
};
		 

MyTCPPort::MyTCPPort( Poller* poller, const InetHostAddress& addr,
		      port_type port )
  : TCPPort( addr, port ),
    m_poller( poller )
{
  PollerClient* pC = new PollerClient( poller, this, true, true );
  pC->registerEvent( PollerFD::EVT_IN );
  pC->registerEvent( PollerFD::EVT_PRI );
  pC->registerEvent( PollerFD::EVT_HUP );
  pC->registerEvent( PollerFD::EVT_ERR );
}

void MyTCPPort::onInput()
{
  listen();
}

void MyTCPPort::onConnected( int sockFD,
		  const sockaddr_in& from,
		  socklen_t fromLen )
{
  new MyTCPSocket( m_poller, sockFD );
}

MyTCPSocket::MyTCPSocket( Poller* poller, fd_type FD )
  : TCPSocket( FD ),
    m_buf( new data_type[512] )
{
  m_pollerClient = new PollerClient( poller, this, true, true );
  m_pollerClient->registerEvent( PollerFD::EVT_IN | 
				 PollerFD::EVT_PRI |
				 PollerFD::EVT_HUP |
				 PollerFD::EVT_ERR );
}

void MyTCPSocket::onInput()
{
  len_type len = read( m_buf, 512 );
  if( len == File::eof || len == 0 )
  {
     m_pollerClient->unregister();
     return;
  }

  cout << "Client: -----" << endl;
  cout.write( m_buf, len );
  cout << "-----" << endl;
}

int main( int argc, char **argv )
{
  if( argc < 2 ) return EXIT_FAILURE;
  
  TCPSocket::port_type port = atoi( argv[1] );
  InetHostAddress addr = "0.0.0.0" ;
  
  Poller poller( 10 );
  MyTCPPort tcpPort( &poller, addr, port );
  
  while( true )
    poller.runSyscallPoll( -1 );


  return EXIT_SUCCESS;  
}
