/***************************************************************************
                           tokenizer.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: tokenizer.cpp,v 1.4 2002/03/19 21:09:27 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "tokenizer.hpp"

Tokenizer::Tokenizer( int tokNum,
		      data_type delim1,
		      data_type delim2 )
  : m_maxTokenNum( tokNum ),
    m_delim1( delim1 ),
    m_delim2( delim2 ),
    m_token( new IRCToken[tokNum] ),
    m_tokenNum( 0 )
{
}

Tokenizer::~Tokenizer()
{
  delete[] m_token;
}

Tokenizer::int_type
Tokenizer::update( reference_type buf,
			    len_type len )
{
   IRCToken*             tokenIter    = m_token;
   const IRCToken* const tokenIterMax = m_token + ( m_maxTokenNum - 1 );
   reference_type        bufIter      = buf;
   const reference_type  bufIterMax   = buf+len;
   reference_type        tokenStart   = NULL;
   bool                  quit         = false;

   // reset member
   m_tokenNum = 0;

   // call hook

   for( ; true; ++tokenIter )
   {
      // remove 'm_delim1'
      for( ; true; ++bufIter )
      {
	 if( bufIter == bufIterMax || *bufIter == m_delim2 )
	    goto end;
	 else if( *bufIter != m_delim1 )
	    break;
      }

      // we get here whenever we have found a token start
      tokenStart = bufIter;

      if( tokenIter == tokenIterMax )
      {
	 // iteratore to first m_delim2 or bufIterMax
	 while( bufIter < bufIterMax && *bufIter != m_delim2 )
	    ++bufIter;

	 tokenIter->assign( tokenStart, bufIter - tokenStart );

	 goto end;
      }

      for( ; true; ++bufIter )
      {
	 if( bufIter == bufIterMax )
	 {
	    quit = true;
	    break;
	 }
	 else if( *bufIter == m_delim1 || *bufIter == m_delim2 )
	 {
	    tokenIter->assign( tokenStart, bufIter - tokenStart );
	    break;
	 }
      }

      if( quit )
	 break;      
   }
  end:
   m_tokenNum = tokenIter - m_token;

   return bufIter - buf;
}
