#include <iomanip>
#include "util.hpp"


namespace util
{

using std::string;
using std::ostream;

int
countDigits( register int number, 
	     register int system )
{
   register int digits = 1;

   if( number < system )
     return 1;


   while( ( number/=system ) != 0 )
      ++digits;

   return digits;
}

static const char hexTab[] =
{
   '0',
   '1',
   '2',
   '3',
   '4',
   '5',
   '6',
   '7',
   '8',
   '9',
   'a',
   'b',
   'c',
   'd',
   'e',
   'f'
};

int
numToHexString( char* cache,
		register unsigned int num )
		
{
   register char* cacheIter = cache;
   register int   numAdded  = 1;
   register int   number    = num;

   while( number > 15 )
   {
      *cacheIter++  = hexTab[ number / 16 ];
      number       %= 16;
      ++numAdded;
   }
   *cacheIter = hexTab[ number ];

   return numAdded;
}

int hexStringToNum( register const HTTPTraits::char_type* cache,
		    register int cacheLen )
{
   register const HTTPTraits::char_type*       cacheIter = cache;
   register const HTTPTraits::char_type* const cacheEnd  = cache + cacheLen;
   register int                                value     = 0;
   
   while( cacheIter < cacheEnd )
   {
      if( HTTPTraits::isHexChar( *cacheIter ) )
      {
	 value *= 16;
	 value += HTTPTraits::hexCharToInt( *cacheIter );
      }
      ++cacheIter;
   }

   return value;
}

ostream&
printWeekDay( ostream& out, unsigned int day )
{
   switch( day )
   {
      case 0:
	 return out << "Mon";

      case 1:
	 return out << "Tue";

      case 2:
	 return out << "Wed";

      case 3:
	 return out << "Thu";

      case 4:
	 return out << "Fri";

      case 5:
	 return out << "Sat";

      default:
	 return out << "Sun";
   }
}

ostream&
printMonth( ostream& out, unsigned int month )
{
   switch( month )
   {
      case 0:
	 return out << "Jan";

      case 1:
	 return out << "Feb";

      case 2:
	 return out << "Mar";

      case 3:
	 return out << "Apr";

      case 4:
	 return out << "May";

      case 5:
	 return out << "Jun";

      case 6:
	 return out << "Jul";

      case 7:
	 return out << "Aug";

      case 8:
	 return out << "Sep";

      case 9:
	 return out << "Aug";

      case 10:
	 return out << "Nov";

      default:
	 return out << "Dec";
   }
}

string&
timeToString( string& target,
		   time_t t )
{
   using namespace std;

   ostringstream    stream;
   tm brokenTime;     


   // read time
   if( &brokenTime != gmtime_r( &t, &brokenTime ) )
      // error
      return target.erase();

   /*
    *  setup stream
    */
   // makes the buffer contain at most 27 characters
   // regular timestamps shouldn't overflow this buffer
//    stream.freeze( 28 );
   

   stream << resetiosflags( ios::adjustfield )
          // make numbers < 10 printed as 0<number>
	  << setiosflags( ios::right )
	  << setw( 2 )
	  << setfill( '0' );

   // print month
   printWeekDay( stream, brokenTime.tm_mon ) << ", ";
   
   // print numeric week day
   stream  << setw( 2 ) << brokenTime.tm_wday << ' ';
   
   // print month
   printMonth( stream, brokenTime.tm_mon ) << ' ';
   
   // print year
   stream << ( 1900 + brokenTime.tm_year ) << ' ';
   
   // print hour:min:sec
   stream << setw( 2 ) << brokenTime.tm_hour << ':' 
	  << setw( 2 ) << brokenTime.tm_min << ':' 
	  << setw( 2 ) << brokenTime.tm_sec;

//    stream << " GMT" << std::eos;
   
   target = stream.str();

   // uninitialize static buffer
//    stream.freeze( false );

   return target;
}

void
FileStats::update( const char* filename )
{
   m_stat = new struct stat();
   
   if( lstat( filename, m_stat ) == -1 )
   {
      // error
      delete m_stat;
      m_stat    = NULL;
      m_canRead  = false;
      m_canWrite = false;
      
      return;
   }

   uid_t  uid  = getuid();
   gid_t  gid  = getgid();
   mode_t mode = m_stat->st_mode;

   if( m_stat->st_uid != uid )
   {
      if( m_stat->st_gid != gid )
      {
	 // current process's owner is not owner of file
	 // nor in the file's group
	 m_canRead  = mode & S_IROTH;
	 m_canWrite = mode & S_IWOTH;
      }
      else
      {
	 // current process's owner is not owner of file
	 // but in the file's group
	 m_canRead = mode & S_IRGRP;
	 m_canRead = mode & S_IWGRP;
      }
   }
   else
   {
      // current process's owner is owner of file
      m_canRead  = m_stat->st_mode & S_IRUSR;
      m_canWrite = m_stat->st_mode & S_IWUSR;
   }
}

}//EON util
