/***************************************************************************
                           vhost.cpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: vhost.cpp,v 1.13 2002/03/13 14:21:49 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <vhost.hpp>

const IRCToken::size_type VHost::MAX_HOSTLEN         = 63;
const IRCToken::size_type VHost::MAX_USERLEN         = 9;
bool                    VHost::defaultPublicPolicy = false;

VHost::VHost( const string &Address, bool ispublic )
  : ident( NULL ), 
    address( new IRCToken( Address, true ) ), 
    references( new ClientSet() ),
    m_public( ispublic ),
    m_addressOnly( true )
{
  conformToRFC( );

#ifdef VERBOSE_MEMORY
  cout << "VHost] created: " << *address << endl;
#endif
}

VHost::VHost( const IRCToken& Address, bool ispublic )
  : ident( NULL ),
    address( new IRCToken( Address, true ) ),
    references( new ClientSet() ),
    m_public( ispublic ),
    m_addressOnly( true )
{
  conformToRFC();

#ifdef VERBOSE_MEMORY
  cout << "VHost] created: " << *address << endl;
#endif
}

VHost::VHost( const string  &Ident, const string &Address, 
	      bool ispublic )
  : ident( new IRCToken( Ident, true ) ), 
    address( new IRCToken( Address, true ) ), 
    references( new ClientSet() ),
    m_public( ispublic ),
    m_addressOnly( false )
{
  conformToRFC( );

#ifdef VERBOSE_MEMORY
  cout << "VHost] created: " << *ident << '@' << *address << endl;
#endif
}

VHost::VHost( const IRCToken& Ident, const IRCToken& Address,
	      bool ispublic )
  : ident( new IRCToken( Ident, true ) ),
    address( new IRCToken( Address, true ) ),
    references( new ClientSet() ),
    m_public( ispublic ),
    m_addressOnly( false )
{
  conformToRFC();
  
#ifdef VERBOSE_MEMORY
  cout << "VHost] created: " << *ident << '@' << *address << endl;
#endif
}  

VHost::~VHost()
{
#ifdef VERBOSE_MEMORY
  cout << "VHost] destroyed: " << *address << endl;
#endif

  delete references;
  delete address;
  if( !m_addressOnly )
    delete ident;
}

void VHost::removeReference( const HSClient *c )
{
  references->erase( c );
}

void VHost::addReference( const HSClient *c )
{
  references->insert( c );
}

void VHost::conformToRFC( )
{
  if( !m_addressOnly && ident->length() > MAX_USERLEN ) {
#ifdef DEBUG
    cout << "Warning: ident '"<< *ident << "' too long. Truncating to " << MAX_USERLEN << " characters" << endl;
#endif
    ident->erase( MAX_USERLEN );
  }

  if( address->length() > MAX_HOSTLEN ) {
#ifdef DEBUG
    cout << "Warning: address '" << *address << "' too long. Truncating to " << MAX_HOSTLEN << " characters" << endl;
#endif
    address->erase( MAX_HOSTLEN );
  }
}

ostream & operator<<( ostream &out, const VHost &vhost )
{
  if( vhost.addressOnly() ) 
    return out << vhost.getAddress();
  else 
    return out << vhost.getIdent() << '@' << vhost.getAddress();
}
