/***************************************************************************
                           vhostconfigfile.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: vhostconfigfile.cpp,v 1.6 2002/03/19 10:58:10 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <vhostconfigfile.hpp>
#include <algorithm>

const string VHostConfigFile::DEF_DELIM = "@";
const string VHostConfigFile::DEF_PUBLIC_USERNAME_DELIM = "::";

VHostConfigFile::VHostConfigFile( VHostList *l )
  : ConfigFile( DEF_DELIM, '#' ),
    vhosts( l )
{
#ifdef DEBUG
  cout << "VHostConfigFile] created" << endl;
#endif
}

VHostConfigFile::~VHostConfigFile()
{
#ifdef DEBUG
  cout << "VHostConfigFile] destroyed" << endl;
#endif
}

/*
 * l -> contains everything left of 'DEF_DELIM'
 * r -> contains everything right of 'DEF_DELIM'
 */
void VHostConfigFile::onParsedEntry( const string &l, const string &r )
{
   string::size_type idx = l.find( ':' );
   VHost*            v   = NULL;

   /*
    * find ':' in left hand.
    * if ':' exists parse out and check if ':' equals '1'.
    * result is given to the VHost constructor then.
    */
   if( idx != string::npos ) 
   {
      bool         isPublic = l.substr( static_cast< string::size_type >( 0 ), idx ) == "1";
      const string username = l.substr( idx + DEF_PUBLIC_USERNAME_DELIM.length() );

      if( !username.empty() )
	 v = new VHost( username, r, isPublic );
      else 
	 v = new VHost( r, isPublic );
   }
   else {
      // here: use defaultPolicy as 2/3 implicit parameter ;-)
      if( !l.empty() )
	 v = new VHost( r );
      else 
	 v = new VHost( l, r );
   }

   if( !vhosts->insertVHost( v ) )
   {
      cerr << "Warning: duplicate vhost: " << *v << endl;
      delete v;
   }
}

void VHostConfigFile::onSerialize( streambuf *buf )
{  
  ostream o( buf );
  writeVHosts( o, vhosts );
}

void VHostConfigFile::writeVHosts( ostream &o, VHostList *newVHosts )
{
  for_each( newVHosts->begin(), newVHosts->end(), Writer( o ) );
}

void VHostConfigFile::Writer::operator() ( const VHostList::storage_type& st )
{
   const VHost* const v = st.first;

   out << ( v->isPublic() ? '1' : '0' ) << DEF_PUBLIC_USERNAME_DELIM;

   if( !v->addressOnly() )
      out << v->getIdent();
   out << DEF_DELIM << v->getAddress() << '\n';
}
