/***************************************************************************
                           vhostlist.cpp
                           -------------------
    copyright            : (C) 2001,2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: vhostlist.cpp,v 1.11 2002/03/19 16:39:02 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <vhostlist.hpp>
#include <iostream>
#include <algorithm>

VHostList::VHostList( int initialCapacity )
   : m_nextId( 0 ),
     m_idxVector(),
     m_unusedIds()
{
   m_idxVector.reserve( initialCapacity );
#ifdef DEBUG
   cout << "VHostList] created" << endl;
#endif
}

VHostList::~VHostList()
{
   clear();

#ifdef DEBUG
   cout << "VHostList] destroyed" << endl;
#endif
}

bool
VHostList::insertVHost( const VHost *v )
{
   id_type id;
   bool updateNextId = false;

   assert( v );

   if( m_unusedIds.empty() )
   {
      id = m_nextId;
      updateNextId = true;
   }
   else
   {
      id = m_unusedIds.front();
      m_unusedIds.pop_front();
   }

   pair<iterator, bool> p = insert( make_pair( v, id ) );


   if( p.second )
   {
#ifdef DEBUG
      cout << "VHostList::insertvHost(): inserted vhost " << *v << " with id: " << id << endl;
#endif
      m_idxVector.push_back( v );

      if( updateNextId )
	 ++m_nextId;
   }

   return p.second;
}

bool
VHostList::eraseVHost( const VHost *v )
{
   bool ret = false;

   assert( v );

   iterator it = find( v );

   if( it != end () ) {
      const id_type id = it->second;

      m_unusedIds.push_back( id );
      
      m_idxVector.erase( m_idxVector.begin() + id );

      erase( it );

      delete v;
      ret = true;
   }

   return ret;
}

const VHost*
VHostList::getVHost( const IRCToken &address ) const
{
   const_iterator it = find_if( begin(), end(), 
				VHostAddressPredicate( address ) );

   if( it != end() ) 
      return it->first;
   else
      return NULL;
}

const VHost*
VHostList::getVHost( id_type id ) const
{
   if( id > m_nextId )
   {
#ifdef DEBUG
      cout << "VHostList::getVHost(): fatal error: 'id' out of range" << endl;
#endif
      return NULL;
   }
   
   return m_idxVector[id];
}

VHostList::id_type
VHostList::getVHostID( const VHost* v ) const
{
   const_iterator it = find( v );
   
   if( it != end() ) 
      return it->second;
   else
      return npos;
}

inline 
void
my_delete( const pair<const VHost*,unsigned int>& p )
{
   delete p.first;
}

void
VHostList::clear()
{
   for_each( begin(), end(), &my_delete );
	     

   map_type::clear();
}

void
VHostList::optimize()
{
   list_type::const_iterator itEnd = m_unusedIds.end();

   int maxMove = m_idxVector.size();

   for( list_type::const_iterator it = m_unusedIds.begin();
	it != itEnd && maxMove-- > 0; ++it )
   {
      const VHost* const v     = m_idxVector.back();
      const id_type      newId = *it;

      m_idxVector.pop_back();
      m_idxVector[newId] = v;
      (*this)[v]         = newId;
   }

   m_nextId = m_idxVector.size();
}
