/***************************************************************************
                           dbconvert.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: dbconvert.cpp,v 1.2 2002/03/13 14:19:08 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <list>
#include <string>
#include <algorithm>
extern "C" {
#include <sys/time.h>
}
#include <fstream>

using namespace std;

bool
processLine( string& line )
{
   // format is: <user>:<password>@<flags string>-><vhostname>

   string::size_type idx = line.find( ':' );
   if( idx == string::npos )
   {
      cout << "malformed entry: " << line << endl;
      return false;
   }


   idx = line.find( '@', idx );
   if( idx == string::npos )
   {
      cout << "malformed entry: " << line << endl;
      return false;
   }


   const string::size_type flagsStart = idx + 1;
   if( flagsStart == line.length() )
   {
      cout << "malformed entry: " << line << endl;
      return false;    
   }


   const string::size_type flagsEnd = line.find( "->", flagsStart );   
   if( ( flagsEnd - flagsStart ) < 2 )
   {
      cout << "hmmm. This line does not seem to be serialized by hostserv" << endl;
      return true;
   }
   
   // erase 2nd flag in flag string
   line.erase( flagsStart + 1, 1 );
   
   return true ;


}

int
main( int argc, char** argv )
{
   if( argc < 2 )
   {
      cout << "Usage: " << *argv << " <filename>" << endl;
      return EXIT_FAILURE;
   }

   string line;
   list<string> entries;
   ifstream db( argv[1] );
   
   while( db )
   {
      if( std::getline( db, line ) )
      {
	 if( processLine( line ) )
	    entries.push_back( line );
      }
      else 
	 break;
   }


   // serialize back
   timeval start, end;
   string filename = argv[1];
   filename += ".converted";
   ofstream dbOut( filename.c_str() );

   if( !dbOut )
   {
      cout << "Could not write processed data into " << filename << endl;
      return EXIT_FAILURE;
   }


   cout << "Writing processed data into " << filename << ' ' << flush;


   gettimeofday( &start, NULL );
   copy( entries.begin(), entries.end(), 
	 ostream_iterator<string>( dbOut, "\n" ) );
   gettimeofday( &end, NULL );
   
   
   cout << "... done in " 
	<< ( end.tv_sec - start.tv_sec ) << "s " 
	<< ( end.tv_usec - start.tv_usec ) << " ms"
	<< endl;

   cout << endl << endl
	<< "Converted database written into: " << filename 
	<< endl << endl << endl;

   return EXIT_SUCCESS;
}

