#include <string>
#include "hsclientlist.hpp"
#include "vhostlist.hpp"
#include "hsclientconfigfile.hpp"
#include "vhostconfigfile.hpp"
#define VERBOSE_TIMESTATS
#include "debug.hpp"

class HSClientConfigFileOld : public ConfigFile {
public:
  HSClientConfigFileOld( HSClientList *c, const VHostList *v );
  virtual ~HSClientConfigFileOld();
  
protected:
  virtual void onParsedEntry( const string &l, const string &r );
  virtual void onSerialize( streambuf *buf )
  {}
  
private:
  HSClientConfigFileOld( );
  HSClientConfigFileOld( const HSClientConfigFileOld & c );
  const HSClientConfigFileOld & operator = ( const HSClientConfigFileOld & );

  static const string DEF_DELIM;
  static const string FLAGS_VHOST_DELIM;
  static const string PASSWORD_FLAGS_DELIM;

  HSClientList*    m_clients;
  const VHostList* m_vhosts;
};


const string HSClientConfigFileOld::DEF_DELIM            = ":";
const string HSClientConfigFileOld::FLAGS_VHOST_DELIM    = "->";
const string HSClientConfigFileOld::PASSWORD_FLAGS_DELIM = "@";

HSClientConfigFileOld::HSClientConfigFileOld( HSClientList *l, 
					const VHostList *v )
  : ConfigFile( DEF_DELIM, '#' ),
    m_clients( l ),
    m_vhosts( v )
{
#ifdef VERBOSE_MEMORY
  cout << "HSClientConfigFileOld] created" << endl;
#endif
}

HSClientConfigFileOld::~HSClientConfigFileOld()
{
#ifdef VERBOSE_MEMORY
  cout << "HSClientConfigFileOld] destroyed" << endl;
#endif
}

void
HSClientConfigFileOld::onParsedEntry( const string &username,
				   const string &r )
{
  // r = password@flags[->[vhostID]]
  
  string::size_type passwordEnd = r.find( PASSWORD_FLAGS_DELIM );
  if( passwordEnd != string::npos ) {
    const string password = r.substr( static_cast< string::size_type >( 0 ),
				      passwordEnd );

    passwordEnd += PASSWORD_FLAGS_DELIM.length();
    string::size_type flagsEnd = r.find( FLAGS_VHOST_DELIM, passwordEnd );

    const string flagsString = r.substr( passwordEnd, flagsEnd-passwordEnd );
    int flags = HSClient::parseFlagsString( flagsString );
    const VHost *vhost = NULL;
    
    if( flagsEnd != string::npos && ( flagsEnd+= FLAGS_VHOST_DELIM.length() ) < r.length() ) {
      string vhostName = r.substr( flagsEnd );
      vhost = m_vhosts->getVHost( vhostName );
      if( !vhost ) {
	cout << "Warning: vhost for user " << username << " not found: " <<
	  vhostName << endl;
      }
    }

    HSClient* client = new HSClient( username, password,
				     const_cast<VHost*>( vhost ),
				     flags );

    if( !m_clients->insertClient( client ) )
    {
       cout << "UUUUUUH. Database is corrupt. Duplicate username found: " << username << endl;
       cout << "Omitting this client." << endl;

       delete client;
    }
  }
  else cout << "Ignoring malformed entry for client: " << username << endl;

}

int main( int argc, char** argv )
{
  if( argc < 4 ) 
  {
     cout << "Usage: " << *argv << " <vhost database> <unconverted user database file> <converted user database filename>" << endl;
     return EXIT_FAILURE;
  }

  VHostList    vhosts;
  HSClientList clients;

  VHostConfigFile vhostCfg( &vhosts );
  HSClientConfigFileOld clientsCfgOld( &clients, &vhosts );
  HSClientConfigFile clientsCfgNew( &clients, &vhosts );

  TIME_INIT();

  
  TIME_START();
  vhostCfg.process( string( argv[1] ) );


  cout << endl 
       << "Loaded vhost database." << endl
       << "The next steps may take a while." << endl
       << "It is a good idea to make a cup of coffee now." << endl
       << "Or watch the new Star Wars trailer..." << endl
       << endl;

  clientsCfgOld.process( string( argv[2] ) );

  
  cout << "Making converted database..." << flush;


  clientsCfgNew.setFilename( string( argv[3] ) );
  clientsCfgNew.serialize();
  TIME_STOP();

  
  cout << "done" << endl
       << endl
       << "Ready for takeoff" << endl;

  cout << "Duration: ";
  TIME_PRINT(cout);
}
