/***************************************************************************
                           randdb.cpp
                           -------------------
    copyright            : (C) 2002 by Martin Scholl
    email                : ms@deridiot.net

    $Id: randdb.cpp,v 1.3 2002/03/19 10:58:10 d3cline Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/**
 * creates a db of random user logins
 * this util is used to measure performance 
 * of hostserv at big networks
 */
#include <cstdio>
#include <iostream>
#include <fstream>
#include <string>
#include <algorithm>
#include "hsclientconfigfile.hpp"
#include "vhostconfigfile.hpp"
#include "vhostlist.hpp"
extern "C" {
#include <sys/time.h>
}

using namespace std;

string&
createTempName( string& dest, int len )
{
   register int     randNum;
   dest.erase();
   
   for( int i = 0; i < len; ++i )
   {
      randNum = rand();
      randNum %= 10 + ( 26 * 2 );

      if( randNum < 10 )
	 dest += randNum + '0';
      else if( randNum < 37 )
	 dest += (char)( randNum - 10  + 'a' );
      else 
	 dest += (char)( randNum - 36 + 'A' );
   }

   return dest;
}

void
fillVHostList( VHostList& vhosts, int num )
{
   string address;
   string ident;
   int randNum;

   address.reserve( 63 );
   ident.reserve( 9 );

   while( num-- )
   {
      createTempName( address, ( rand() % 63 ) + 1 );

      randNum = rand();

      VHost* v = NULL;

      if( randNum & 1 )
      {
	 createTempName( ident, ( rand() % 9 ) + 1  );
	 v = new VHost( address,
			ident,
			randNum & ( 1 << 16 ) );
      }
      else
      {
	 v = new VHost( address,
			randNum & ( 1 << 16 ) );
      }
      if( !vhosts.insertVHost( v ) )
	 delete v;
   }
}

int
main( int argc, char** argv )
{
   if( argc < 5 ) 
   {
      cout << "Usage: " << *argv << " <userdb> <usernum> <vhostdb> <vhostnum>" << endl;
      return EXIT_FAILURE;
   }
   
   VHostList                      vhosts;
   fstream                        vhostsDb( argv[3], ios::out | ios::trunc );
   fstream                        userDb( argv[1], ios::out | ios::trunc );      
   VHostConfigFile::Writer        vhostWriter( vhostsDb );
   HSClientConfigFile::Writer     clientWriter( userDb, &vhosts );
   VHostList::const_iterator      it         = vhosts.begin();
   VHostList::const_iterator      end        = vhosts.end();
   int                            numClients = atoi( argv[2] );
   int                            numVHosts  = atoi( argv[4] );
   int                            permission = 0;
   string                         username;
   string                         password;
   timeval                        tStart;
   timeval                        tEnd;

   assert( numClients > 0 );
   assert( numVHosts > 0 );


   cout << "Creating random vhosts .." << flush;

   gettimeofday( &tStart, NULL );
   fillVHostList( vhosts, numClients );
   gettimeofday( &tEnd, NULL );

   cout << "done (" 
	<< ( tEnd.tv_sec - tStart.tv_sec ) << "s " 
	<< ( tEnd.tv_usec - tStart.tv_usec ) << "ms)" 
	<< endl;



   username.reserve( 16 );
   password.reserve( 16 );


   cout << "Creating random clients ..." << flush;

   gettimeofday( &tStart, NULL );   
   while( numClients-- )
   {
      permission = rand();
      createTempName( username, ( rand() % 16 ) + 1  );
      createTempName( password, ( rand() % 11 ) + 6 );

      HSClient client( username, 
		       password,
		       const_cast<VHost*>( it->first ),
		       permission );


      clientWriter( &client );

      ++it;
      if( it == end ) 
	 it = vhosts.begin();
   }
   gettimeofday( &tEnd, NULL );

   cout << "done (" 
	<< ( tEnd.tv_sec - tStart.tv_sec ) << "s " 
	<< ( tEnd.tv_usec - tStart.tv_usec ) << "ms)" 
	<< endl;


   cout << "Writing vhosts db ..." << flush;

   gettimeofday( &tStart, NULL );   
   for_each( vhosts.begin(), vhosts.end(), vhostWriter );
   gettimeofday( &tEnd, NULL );

   cout << "done (" 
	<< ( tEnd.tv_sec - tStart.tv_sec ) << "s " 
	<< ( tEnd.tv_usec - tStart.tv_usec ) << "ms)" 
	<< endl;


}

